C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Arrall version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 ARRALL 06DECEMBER2001
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for miscellaneous gridded 
C      terms.  These miscellaneous gridded terms can be the input to
C      the different packages of the U.S. Geological Survey 
C      Modular Three-Dimensional Finite-Differences Ground-Water Flow Model
C      MODFLOW (McDonald and Harbaugh. 
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCBUDT  : STARTING ARRAY INDEX FOR CELL_BY_CELL BUDGET TERMS
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE ARRALL (
     I                   OUTUNT,
     I                   LENX,
     I                   NLAY,NROW,NCOL,
     B                   ISUM,
     O                   LCBUDT,
     E                   *)
C-----------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        NLAY,NROW,NCOL,
     &        ISUM,
     &        LCBUDT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''         MISCELLANEOUS GRIDDED TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 06DECEMBER2001 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Calculate storage parameter NRCL
C
      NRCL=NROW*NCOL*NLAY
C
C2----Allocate space for arrays
C
      LCBUDT=ISUM
      ISP=NRCL
      ISUM=ISUM+ISP
      ISUM1=ISUM-1
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF GRIDDED TERMS                '',
     &             10X,I15,'' NRCL'')')
     &          NRCL
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR GRIDDED TERMS         '',
     &             10X,I15,'' LCBUDT'')')
     &          LCBUDT
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR MISCELLANEOUS GRIDDED TERMS PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I8)')
     &          ISUM1,LENX
C
C4----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Arrall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Arrall_Subroutine'')')
         RETURN 1
      ENDIF
C
C5----Return
C
      RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Arrrp1 version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 ARRRP1 08DECEMBER2001
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user defined files for cell_by_cell flow terms.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING HEDRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     ARRFIL  : FILE CONTAINING CELL_BY_CELL FLOW TERMS FILE
C     ARRUNT  : FORTRAN UNIT NUMBER CELL_BY_CELL FLOW TERMS FILE
C     CTLFIL  : FILE CONTAINING THE CONTROL LINES
C     CTLUNT  : FORTRAN UNIT NUMBER FOR U3DREL CONTROL LINES
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C
C    Outputs:
C
C     BUDARY  : ARRAY FOR CELL_BY_CELL FLOW TERMS
C
C     Program:
C
C     ANAME   : CHARACTER ARRAY (6,8) CONTAINING ARRAY NAMES
C     K       : LOOP INDEX
C     KK      : PASSED LAYER NUMBER
C     LOC     : STARTING INDEX FOR ARRAY
C
C-------------------------------------------------------------------------
      SUBROUTINE ARRRP1 (
     I                   OUTFIL,OUTUNT,
     I                   ARRFIL,ARRUNT,
     I                   CTLFIL,CTLUNT,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,
     O                   BUDARY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION COVPMR(MAXPMR),
     &          BUDARY(NCOL*NROW*NLAY)
C
      CHARACTER*(*) OUTFIL,ARRFIL,CTLFIL
C
      INTEGER OUTUNT,ARRUNT,CTLUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*16 BUDTXT,TEXT
      CHARACTER*24 ANAME
C
      INTEGER ICOL,IROW,ILAY,LAYER,KK,BCFFLG,CBCUNT
C
      DATA BUDTXT /'MISCELLANEO TERM'/
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                    Cell_By_Cell Flow Array
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''        MISCELLANEOUS GRIDDED TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 14MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
CL----Set local variables
C
      MODFLG=INT(COVPMR(1))
      MODFMT=MODFLG-3
      MODFMT=INT(COVPMR(12))/100
      IDCNTR=MOD(INT(COVPMR(12)),100)
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
      HNOFLO=COVPMR(15)
      LOC=0
C
      WRITE(*,'(
     &          ''      .Reading cell_by_cell miscellaneous terms'')')
      WRITE(OUTUNT,'(
     &          ''      .Reading cell_by_cell miscellaneous terms'')')
      WRITE(*,'(
     &             ''      ..for Modflow stress period '',I10)')
     &             MODPER
      WRITE(OUTUNT,'(
     &             ''      ..for Modflow stress period '',I10)')
     &             MODPER
      WRITE(*,'(
     &             ''      ..for Modflow time step '',I10)')
     &             MODSTP
      WRITE(OUTUNT,'(
     &             ''      ..for Modflow time step '',I10)')
     &             MODSTP
C
C1---Array format in ARRFIL file
C
      IF(ARRUNT.LT.0) THEN
         WRITE(*,'(
     &             ''      .Reading binary formatted '',
     &             ''miscellaneous gridded terms '')')
         WRITE(OUTUNT,'(
     &              ''      .Reading binary formatted '',
     &              ''miscellaneous gridded terms '')')
C
C1A---Binary format in ARRFIL file
C
         CBCUNT=ABS(ARRUNT)
         WRITE(*,'(
     &             ''      ..recorded in file '',A)')
     &             ARRFIL (1:INDEX(ARRFIL,' ')-1)
         WRITE(*,'(
     &             ''      ..on unit '',I10)')
     &                                                 CBCUNT 
         WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
         WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 CBCUNT 
C
C1B---Text format in ARRFIL file
C
      ELSE IF(ARRUNT.GT.0) THEN
          WRITE(*,'(
     &              ''      .Reading text formatted '',
     &              ''miscellaneous gridded terms '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 ARRUNT
          WRITE(OUTUNT,'(
     &              ''      .Reading text formatted '',
     &              ''miscellaneous gridded terms '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 ARRUNT
C
C1C---Error no file containing miscellaneous gridded terms
C
      ELSE
         WRITE(*,'(
     &             ''      .No file containing miscellaneous '',
     &             ''terms specified in MODTOOLS name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No file containing miscellaneous '',
     &             ''terms specified in MODTOOLS name file'')')
         GO TO 9999
      ENDIF
C
C2---Control CTLFIL file
C
      IF(CTLUNT.NE.0) THEN
          WRITE(*,'(
     &              ''      .Using control lines '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 CTLUNT
          WRITE(OUTUNT,'(
     &              ''      .Using control lines '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 CTLUNT
      ELSE
       IF(IDCNTR.EQ.2) THEN
          WRITE(*,'(
     &              ''      .Need file containing U2DREL control '',
     &              ''lines for format of miscellaneous terms file'')') 
          WRITE(OUTUNT,'(
     &              ''      .Need file containing U2DREL control '',
     &              ''lines for format of miscellaneous terms file'')') 
       ELSE
          WRITE(*,'(
     &              ''      .Need file containing U3DREL control '',
     &              ''lines for format of miscellaneous terms file'')') 
          WRITE(OUTUNT,'(
     &              ''      .Need file containing U3DREL control '',
     &              ''lines for format of miscellaneous terms file'')') 
       ENDIF
         GO TO 9999
      ENDIF
C
C3----Set array
C
      DO 300 KK=1,NLAY
      DO 300 JJ=1,NCOL
      DO 300 II=1,NROW 
         LOC=LOC+1
 300     BUDARY(LOC)=HNOFLO
C
C3----Read 2-D array
C
      IF(IDCNTR.EQ.2) THEN
         CALL U2DARY (BUDARY,BUDTXT,NROW,NCOL,NLAY,
     &                ARRFIL,ARRUNT,CTLFIL,CTLUNT,OUTUNT,
     &                MODPER,MODSTP,
     E                *9999)
C
C4----Read 3-D array
C
      ELSE
         CALL U3DARY (BUDARY,BUDTXT,NROW,NCOL,NLAY,
     &                ARRFIL,ARRUNT,CTLFIL,CTLUNT,OUTUNT,
     &                MODPER,MODSTP,
     E                *9999)
      ENDIF
C
        GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Arrrp1_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Arrrp1_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Ptharr version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION (see below)
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a Info file ARRFIL using the miscellaneous gridded
C      terms that could be input to the different packages of the 
C      U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C      IVIEW
C           => 0 Mapview --> all layers
C                 One info files of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 1 Vertical row profile [appended to row coverage]
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) CELL_BY_CELL BUDGET TERMS
C
C           => 2 Vertical column profile
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 3 Vertical particle profile
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 4 Mapview --> user specified layer
C                 One info file of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) LAYER_xx = CELL_BY_CELL BUDGET TERMS FOR CHOSEN LAYER
C
C           => 5 Mapview --> user specified ibound value
C                 One info file of cell-by-cell budget terms
C                 (this ibound value can be used to differentiate model
C                  layers into geological units.)
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL BUDGET TERMS 
C                       (these terms are computed as a summation of the 
C                         cell-by-cell budget terms for each layer which
C                         match the chosen ibound value.)
C           => 6 Mapview --> uppermost active cell
C                 One info file of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL BUDGET TERMS
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     ARRFIL  : COVERAGE ROOT NAME
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR)
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NDELX)
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY)
C     LAYCBD  : ARRAY FOR CONFINING LAYERS TYPE
C     BUDARY  : ARRAY FOR LAYER/UNIT CELL_BY_CELL BUDGETS
C
C    Outputs: None
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PTHARR (
     I                   OUTUNT,
     I                   ARRFIL,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,LRWCL,
     I                   IBOUND,LAYCBD,
     I                   BUDARY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     1          COVPMR(MAXPMR),
     1          LAYCBD(NLAY),
     2          LRWCL(2,NCOL*NROW),
     3          IBOUND(NCOL,NROW,NLAY),
     3          BUDARY(NCOL,NROW,NLAY)
C
      CHARACTER*(*) ARRFIL
      INTEGER OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER (MAXFIL=3,
     &           MAXITM=5)
C
      REAL BUDIBD,MINTRM,MAXTRM
C
      CHARACTER*128 LABFIL,FMTSTR,BUFFER
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL
      CHARACTER*16 COVTYP,ITEM,ITEMS(MAXITM),ARCCMD,ACTION,
     &             LAYITM,STRITM,RELITM,JONTYP
      CHARACTER*10 CHRLAY,CHRWCL,LAYFMT,ITMFMT
      CHARACTER*6 MODTXT
      CHARACTER*1 TYPITM,TYPES(MAXITM)
C
      INTEGER DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN,
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM)
C
      INTEGER 
     &        IRWCL,IVIEW,
     &        IDPART,ICOLOR,TIMSTP,OLDSTP,
     &        STRCHR,STRCOL,STRROW,STRLAY,STRIBD,STRDEL,STRCLL,
     &        PRECOL,PREROW,PRELAY,PREIBD,
     &        FINCHR,FINCOL,FINROW,FINLAY,FINIBD,FINDEL,FINCLL,
     &        ROW,COLUMN,CELL,BADIBD,COVDEX,COL,ROWCOL,SEQNUM,
     &        NCELLS,
     &        NUMCOL,NUMROW,NUMLAY,
     &        IRWCOL
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA FILES /'budgetitems',
     &            'budgetitem',
     &            'arccmd'/
      DATA IOUT /91,92,93/
      DATA ITEMS /'SEQNUM','ROWCOL','ROW','COLUMN','LAYER_'/
      DATA DECIMLS /4*0, 6/
      DATA TYPES /'I','C',2*'I','F'/
      DATA WIDTHS /4*10,8/
      DATA OUTPUTS /4*10,16/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''         MISCELLANEOUS GRIDDED TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 11SEPTEMBER2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                  GIS OUTPUT MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C0----Set local variables
C
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
      MODFLG=INT(COVPMR(1))
      MODFMT=MODFLG-3
      MODFMT=INT(COVPMR(12))/100
      IDCNTR=MOD(INT(COVPMR(12)),100)
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
C
      IF(NPER.EQ.0) THEN
         MODPER=1
         MODSTP=1
      ENDIF
C
      MODTXT='UNKTRM'
C
C1---Find maximum and minimum values
C
      MINTRM=1.0E14
      MAXTRM=1.0E-14
C
C1A---Mapview for single layer (iview=0,4)
C
      IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
         DO 1010 IROW=1,NROW
            DO 1020 ICOL=1,NCOL
             IF(IVIEW.LE.0) THEN
                DO 1030 ILAY=1,NLAY
                   IBDCEL=IBOUND(ICOL,IROW,ILAY)
                 IF(IBDCEL.NE.0) THEN
                    BUDGET=BUDARY(ICOL,IROW,ILAY)
                  IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
                  IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
                 ENDIF
 1030           CONTINUE
             ELSE
                IBDCEL=IBOUND(ICOL,IROW,IRWCL)
              IF(IBDCEL.NE.0) THEN
                 BUDGET=BUDARY(ICOL,IROW,IRWCL)
               IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
               IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
              ENDIF
             ENDIF
 1020       CONTINUE
 1010    CONTINUE
C
C1B---Cross-sectional view (iview=1,2,3)
C
      ELSE IF(IVIEW.GE.1 .AND. IVIEW.LE.3) THEN
C
       IF(IVIEW.EQ.1) THEN
          STRCLL=1
          FINCLL=NCOL
       ELSE IF(IVIEW.EQ.2) THEN
          STRCLL=NCOL+1
          FINCLL=STRCLL+NROW-1
       ELSE
          STRCLL=1
          FINCLL=IRWCL
       ENDIF
C
         DO 1100 N=STRCLL,FINCLL
            NUMCLL=NUMCLL+1
          IF(IVIEW.LE.1) THEN
             IROW=IRWCL
             ICOL=NUMCLL
          ELSE IF(IVIEW.EQ.2) THEN
             IROW=NUMCLL
             ICOL=IRWCL
          ELSE
             ICOL=LRWCL(1,NUMCLL)
             IROW=LRWCL(2,NUMCLL)
          ENDIF
            DO 1110 ILAY=1,NLAY
               IBDCEL=IBOUND(ICOL,IROW,ILAY)
             IF(IBDCEL.NE.0) THEN
                NCELLS=NCELLS+1
                BUDGET=BUDARY(ICOL,IROW,ILAY)
              IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
              IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
             ENDIF
 1110       CONTINUE
 1100    CONTINUE
C
C1C---Mapview for single ibound value (iview=5)
C
      ELSE IF(IVIEW.EQ.5) THEN
         LAYIBD=IRWCL
         DO 1200 IROW=1,NROW
            DO 1210 ICOL=1,NCOL
               BUDIBD=0.0
               IBDFLG=0
               ICLFLG=0
               DO 1220 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.EQ.LAYIBD) THEN
                   IBDFLG=1
                   BUDIBD=BUDIBD+BUDARY(ICOL,IROW,ILAY)
                ENDIF
 1220          CONTINUE
C
               IF(IBDFLG.GT.0) THEN
                IF(BUDIBD.GT.MAXTRM) MAXTRM=BUDGET
                IF(BUDIBD.LT.MINTRM) MINTRM=BUDGET
               ENDIF
 1210       CONTINUE
 1200    CONTINUE
C
C1D---Mapview for uppermost active cell (iview=6)
C
      ELSE
         DO 1300 IROW=1,NROW
            DO 1310 ICOL=1,NCOL
               DO 1320 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.NE.0) THEN
                 IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
                 IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
                   GO TO 1310
                ENDIF
 1320          CONTINUE
 1310       CONTINUE
 1300    CONTINUE
      ENDIF
C
C1E---Calculate term attribute definition
C
      WRITE(*,'('' Maximum '',F15.6)') MAXTRM
      WRITE(*,'('' Minimum '',F15.6)') MINTRM
      IMAXTRM=-14
C
      DO 1400 I=1,28
         ARRTRM=1.0*(10**FLOAT(IMAXTRM))
         ARRTRM=MAXTRM*ARRTRM
       IF(ARRTRM.GE.1.0) GO TO 1402
         IMAXTRM=IMAXTRM+1
 1400 CONTINUE
C
 1402 IMINTRM=-14
C
      DO 1410 I=1,28
         ARRTRM=1.0*(10**FLOAT(IMINTRM))
         ARRTRM=ABS(MINTRM)*ARRTRM
        IF(ARRTRM.GE.0.0) GO TO 1412
         IMINTRM=IMINTRM+1
 1410 CONTINUE
C
 1412   MAXDEC=IMAXTRM
      IF(IMINTRM.GT.IMAXTRM) MAXDEC=IMINTRM 
         MAXDEC=MAXDEC+2
      IF(MAXDEC.LE.IMAXTRM0) MAXDEC=1
C
      IF(MAXDEC.LT.10) THEN
         WRITE(FMTSTR,'(I1)') MAXDEC
         FINCHR=1
      ELSE
         WRITE(FMTSTR,'(I2)') MAXDEC
         FINCHR=2
      ENDIF
      DECIMLS(MAXITM)=MAXDEC
C
      IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
         FMTSTR='(I10,'','',A,'','',2(I10,'',''),200(F16.'//
     &          FMTSTR (1:FINCHR)//','',''))'
      ELSE IF(IVIEW.GE.1 .AND. IVIEW.LE.3) THEN
         FMTSTR='(I10,'','',A,'','',3(I10,'',''),200(F16.'//
     &          FMTSTR (1:FINCHR)//','',''))'
      ELSE IF(IVIEW.EQ.5) THEN
         FMTSTR='(I10,'','',A,'','',3(I10,'',''),200(F16.'//
     &          FMTSTR (1:FINCHR)//','',''))'
      ELSE
         FMTSTR='(I10,'','',A,'','',4(I10,'',''),200(F16.'//
     &          FMTSTR (1:FINCHR)//','',''))'
      ENDIF
C
C2----Delete and create files for attribute definitions, attribute value, 
C      and coverage arc data and tolerance/dangle values
C
      NSTAT=2
      ITALK=1
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) 
     &   FILES(2) = ARRFIL(1:INDEX(ARRFIL,' ')-1)
         WRITE(*,'('' Here'')')
C
C2A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      DO 200 N=1,MAXFIL-1
         CALL DELETE_FILE (FILES(N),*9999)
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      WIDTHS(1)=LENROW+LENCOL
      OUTPUTS(1)=LENROW+LENCOL
      WIDTHS(2)=LENROW+LENCOL
      OUTPUTS(2)=LENROW+LENCOL
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
C
C2C----Write standard attribute definitions
C
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) THEN
         DO 210 K=1,4
            DECIML=DECIMLS(K)
            ITEM=ITEMS(K)
            OUTPUT=OUTPUTS(K)
            TYPITM=TYPES(K)
            WIDTH=WIDTHS(K)
            WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  210    CONTINUE
C
C2Ca----Write attribute definitions IVIEW = all layers
C
        IF(IVIEW.LE.0) THEN
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           DO 230 ILAY=1,NLAY
            IF(ILAY.LT.10) THEN
               WRITE(CHRLAY,'(I1)') ILAY
            ELSE IF(ILAY.GT.9 .AND. ILAY.LT.100) THEN
               WRITE(CHRLAY,'(I2)') ILAY
            ELSE IF(ILAY.GT.99 .AND. ILAY.LT.1000) THEN
               WRITE(CHRLAY,'(I3)') ILAY
            ELSE IF(ILAY.GT.999 .AND. ILAY.LT.1000) THEN
               WRITE(CHRLAY,'(I4)') ILAY
            ENDIF
              ITEM='LAYER_'//CHRLAY
              WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                        I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  230      CONTINUE
C
C2Cb----Write attribute definitions IVIEW = single layer
C
        ELSE IF(IVIEW.EQ.4) THEN
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           ILAY=IRWCL
         IF(ILAY.LT.10) THEN
            WRITE(CHRLAY,'(I1)') ILAY
         ELSE IF(ILAY.GT.9 .AND. ILAY.LT.100) THEN
            WRITE(CHRLAY,'(I2)') ILAY
         ELSE IF(ILAY.GT.99 .AND. ILAY.LT.1000) THEN
            WRITE(CHRLAY,'(I3)') ILAY
         ELSE IF(ILAY.GT.999 .AND. ILAY.LT.1000) THEN
            WRITE(CHRLAY,'(I4)') ILAY
         ENDIF
           ITEM='LAYER_'//CHRLAY
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
C
C2Cb----Write attribute definitions IVIEW = ibound value
C
        ELSE IF(IVIEW.EQ.5) THEN
           ITEM='IBOUND'
           DECIML=0
           OUTPUT=10
           TYPITM='I'
           WIDTH=10
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
C
           ITEM=MODTXT
           DECIML=DECIMLS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
C
C2Ca----Write attribute definitions IVIEW = uppermost active cell
C
        ELSE IF(IVIEW.EQ.6) THEN
           DECIML=DECIMLS(4)
           ITEM='LAYER'
           OUTPUT=LENLAY
           TYPITM=TYPES(4)
           WIDTH=LENLAY
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           ITEM=MODTXT
           WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         ENDIF
          CALL CLOSE_FILE (FILES(1),*9999)
C
C2D----Write standard attribute definitions
C
      ELSE
         DECIML=0
         ITEM=ARRFIL (1:INDEX(ARRFIL,' ')-1)//'-id'
         OUTPUT=5
         TYPITM='B'
         WIDTH=4
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         DECIML=DECIMLS(MAXITM)
         ITEM=ITEMS(MAXITM)
         OUTPUT=OUTPUTS(MAXITM)
         TYPITM=TYPES(MAXITM)
         WIDTH=WIDTHS(MAXITM)
         ITEM=MODTXT
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         CALL CLOSE_FILE (FILES(1),*9999)
      ENDIF
C
C3----Build cell-by-cell terms
C
      BADIBD=0
      BADCEL=0
      NUMCLL=0
      NCELLS=0
      NUMCON=0
C
C3M---Message
C
      WRITE(*,'(
     &          ''      .Miscellaneous Gridded Terms '')')
      WRITE(OUTUNT,'(
     &          ''      .Miscellaneous Gridded Terms '')')
C
      IF(IVIEW.LE.0) THEN
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for all layers '',I10)')
     &         NLAY
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for all layers '',I10)')
     &         NLAY
      ELSE IF(IVIEW.EQ.1) THEN
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''along row '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''along row '',I10,'' now...'')')
     &         IRWCL
      ELSE IF(IVIEW.EQ.2) THEN
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''along column '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''along column '',I10,'' now...'')')
     &         IRWCL
      ELSE IF(IVIEW.EQ.3) THEN
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''along particle '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''along particle '',I10,'' now...'')')
     &         IRWCL
      ELSE IF(IVIEW.EQ.4) THEN
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for layer '',I10)')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for layer '',I10)')
     &         IRWCL
      ELSE IF(IVIEW.EQ.5) THEN
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for ibound value '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for ibound value '',I10,'' now...'')')
     &         IRWCL
      ELSE
         WRITE(*,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for uppermost active cell...'')')
         WRITE(OUTUNT,'(
     &             ''      .Processing miscellaneous gridded terms '',
     &             ''for uppermost active cell...'')')
      ENDIF
C
C3H----Data header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,
     &               '' MISCELLANEOUS CELL-BY-CELL VALUES '')')
C
      IF(IVIEW.EQ.0) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR ALL LAYERS'')')
      ELSE IF(IVIEW.EQ.1) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS ALONG ROW DIRECTION'')')
      ELSE IF(IVIEW.EQ.2) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS ALONG COLUMN DIRECTION'')')
      ELSE IF(IVIEW.EQ.3) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS ALONG PARTICLE DIRECTION'')')
      ELSE IF(IVIEW.EQ.4) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR LAYER '',I10)')
     &         IRWCL
      ELSE IF(IVIEW.EQ.5) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR IBOUND '',I10)')
     &         IRWCL
      ELSE
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR'',
     &               '' UPPER MOST ACTIVE CELL'')')
      ENDIF
C
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &               ''    CELL VALUE                '',
     &               ''       ROW'',
     &               ''    COLUMN'',
     &               ''     LAYER'',
     &               ''    IBOUND'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
C
C3----Build mapview for single layer (iview=0,4)
C
      IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
         DO 300 IROW=1,NROW
            DO 310 ICOL=1,NCOL
               SEQNUM=(IROW-1)*NCOL+ICOL
               NUMCLL=(IROW-1)*NCOL+ICOL
               ROWCOL=IROW*(10**LENCOL)+ICOL
               WRITE(CHRWCL,'(I10)') ROWCOL
               FINCHR=10
               STRCHR=FINCHR-(LENROW+LENCOL-1)
             IF(IVIEW.LE.0) THEN
                WRITE(IOUT(2),FMTSTR)
     &                       SEQNUM,
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       (BUDARY(ICOL,IROW,NUMLAY), NUMLAY=1,NLAY)
                DO 320 ILAY=1,NLAY
                   IBDCEL=IBOUND(ICOL,IROW,ILAY)
                 IF(IBDCEL.NE.0) THEN
                    NCELLS=NCELLS+1
                    BUDGET=BUDARY(ICOL,IROW,ILAY)
                  IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
                  IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
                    WRITE(OUTUNT,'(10X,F20.10,10X,4I10)')
     &                          BUDGET,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                 ELSE
                    BADIBD=BADIBD+1
                    WRITE(OUTUNT,'(10X,
     &                  ''INACTIVE CELL                 '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                 ENDIF
 320            CONTINUE
              ELSE
                IBDCEL=IBOUND(ICOL,IROW,IRWCL)
               IF(IBDCEL.NE.0) THEN
                  NCELLS=NCELLS+1
                  BUDGET=BUDARY(ICOL,IROW,IRWCL)
                IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
                IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
                   WRITE(IOUT(2),FMTSTR)
     &                       SEQNUM,
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       BUDARY(ICOL,IROW,IRWCL)
                  WRITE(OUTUNT,'(10X,F20.10,10X,4I10)')
     &                          BUDGET,
     &                          IROW,
     &                          ICOL,
     &                          IRWCL,
     &                          IBDCEL
               ELSE
                  BADIBD=BADIBD+1
                  WRITE(OUTUNT,'(10X,
     &                  ''INACTIVE CELL                 '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          IRWCL,
     &                          IBDCEL
               ENDIF
              ENDIF
  310       CONTINUE
  300    CONTINUE
C
C4----Build cross-sectional view (iview=1,2,3)
C
      ELSE IF(IVIEW.GE.1 .AND. IVIEW.LE.3) THEN
C
       IF(IVIEW.EQ.1) THEN
          STRCLL=1
          FINCLL=NCOL
       ELSE IF(IVIEW.EQ.2) THEN
          STRCLL=NCOL+1
          FINCLL=STRCLL+NROW-1
       ELSE
          STRCLL=1
          FINCLL=IRWCL
       ENDIF
C
C4B----Loop to build cell
C
         DO 400 N=STRCLL,FINCLL
            NUMCLL=NUMCLL+1
          IF(IVIEW.LE.1) THEN
             IROW=IRWCL
             ICOL=NUMCLL
          ELSE IF(IVIEW.EQ.2) THEN
             IROW=NUMCLL
             ICOL=IRWCL
          ELSE
             ICOL=LRWCL(1,NUMCLL)
             IROW=LRWCL(2,NUMCLL)
          ENDIF
C
C4Ba----Write attributes of each cell
C
            DO 410 ILAY=1,NLAY
               IBDCEL=IBOUND(ICOL,IROW,ILAY)
             IF(IBDCEL.NE.0) THEN
                NCELLS=NCELLS+1
                BUDGET=BUDARY(ICOL,IROW,ILAY)
              IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
              IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
                WRITE(IOUT(2),'(I10,'','',
     &                          F16.6,'','')')
     &                       NCELLS,
     &                       BUDGET
                WRITE(OUTUNT,'(10X,F20.6,10X,4I10)')
     &                          BUDGET,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
C
              IF(LAYCBD(ILAY).GT.0.0) THEN
                 NUMCON=NUMCON+1
                 WRITE(OUTUNT,'(10X,
     &                   ''CONFINING LAYER CELL          '',
     &                   4I10)')
     &                           IROW,
     &                           ICOL,
     &                           ILAY,
     &                           IBDCEL
                 NCELLS=NCELLS+1
                 BUDGET=0.0
                 WRITE(IOUT(2),'(I10,'','',
     &                           F16.6,'','')')
     &                          NCELLS,
     &                          BUDGET
              ENDIF
             ELSE
                BADIBD=BADIBD+1
                WRITE(OUTUNT,'(10X,
     &                  ''INACTIVE CELL                 '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
             ENDIF
  410       CONTINUE
  400    CONTINUE
C
C5----Build mapview for single ibound value (iview=5)
C
      ELSE IF(IVIEW.EQ.5) THEN
         LAYIBD=IRWCL
         BADCEL=0
C
         DO 500 IROW=1,NROW
            DO 510 ICOL=1,NCOL
               BUDIBD=0.0
               IBDFLG=0
               ICLFLG=0
               DO 520 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.EQ.LAYIBD) THEN
                   IBDFLG=1
                   BUDIBD=BUDIBD+BUDARY(ICOL,IROW,ILAY)
                ENDIF
                IF(IBDCEL.NE.LAYIBD .AND. IBDCEL.NE.0) THEN
                   ICLFLG=IBDCEL
                ENDIF
  520          CONTINUE
C
               IF(IBDFLG.GT.0) THEN
                  NCELLS=NCELLS+1
                  SEQNUM=(IROW-1)*NCOL+ICOL
                  NUMCLL=(IROW-1)*NCOL+ICOL
                  ROWCOL=IROW*(10**LENCOL)+ICOL
                  WRITE(CHRWCL,'(I10)') ROWCOL
                  FINCHR=10
                  STRCHR=FINCHR-(LENROW+LENCOL-1)
                IF(BUDIBD.GT.MAXTRM) MAXTRM=BUDGET
                IF(BUDIBD.LT.MINTRM) MINTRM=BUDGET
                  WRITE(IOUT(2),'(
     &                            I10,'','',
     &                            A,'','',
     &                            3(I10,'',''),
     &                            F16.3,'','')')
     &                      SEQNUM,
     &                      CHRWCL (STRCHR:FINCHR),
     &                      IROW,
     &                      ICOL,
     &                      LAYIBD,
     &                      BUDIBD
                  WRITE(OUTUNT,'(10X,F20.6,10X,4I10)')
     &                          BUDGET,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
               ELSE IF(IBDFLG.EQ.0 .AND. ICLFLG.NE.0) THEN
                  ILAY=-999
                  IBDCEL=-999
                  BADCEL=BADCEL+1
                  WRITE(OUTUNT,'(10X,
     &                  ''IBOUND != '',
     &                  I10,10X,4I10)')
     &                          LAYIBD,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
               ELSE
                  BADIBD=BADIBD+1
                  CHRWCL='   NONE   '
                  WRITE(OUTUNT,'(10X,
     &                      ''INACTIVE CELL                 '',
     &                      2I10,2A10)')
     &                              IROW,
     &                              ICOL,
     &                              CHRWCL,
     &                              CHRWCL
               ENDIF
  510       CONTINUE
  500    CONTINUE
C
C6----Build mapview for uppermost active cell (iview=6)
C
      ELSE
         DO 600 IROW=1,NROW
            DO 610 ICOL=1,NCOL
               DO 620 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.NE.0) THEN
                   NCELLS=NCELLS+1
                   BUDGET=BUDARY(ICOL,IROW,ILAY)
                IF(BUDGET.GT.MAXTRM) MAXTRM=BUDGET
                IF(BUDGET.LT.MINTRM) MINTRM=BUDGET
                   SEQNUM=(IROW-1)*NCOL+ICOL
                   NUMCLL=(IROW-1)*NCOL+ICOL
                   ROWCOL=IROW*(10**LENCOL)+ICOL
                   WRITE(CHRWCL,'(I10)') ROWCOL
                   FINCHR=10
                   STRCHR=FINCHR-(LENROW+LENCOL-1)
                   WRITE(IOUT(2),'(
     &                             I10,'','',
     &                             A,'','',
     &                             4(I10,'',''),
     &                             F16.3,'','')')
     &                          SEQNUM,
     &                          CHRWCL (STRCHR:FINCHR),
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          BUDGET
                   GO TO 612
                ENDIF
  620          CONTINUE
  612          IF(IBDCEL.NE.0) THEN
                  WRITE(OUTUNT,'(10X,F20.6,10X,4I10)')
     &                          BUDGET,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
               ELSE
                  BADIBD=BADIBD+1
                  CHRWCL='   NONE   '
                  WRITE(OUTUNT,'(10X,
     &                      ''INACTIVE CELL                 '',
     &                      2I10,A10,I10)')
     &                              IROW,
     &                              ICOL,
     &                              CHRWCL,
     &                              IBDCEL
               ENDIF
  610       CONTINUE
  600    CONTINUE
      ENDIF
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(//)')
C
C7----Close attribute files
C
      CALL CLOSE_FILE (FILES(2),*9999)
C
C8----Messages
C
C
C8A----Message to user
C
      WRITE(*,'(''      .Miscellaneous gridded term file '',A,
     &          '' consists of '')')
     &        ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(*,'(''      .'',I10,'' cells'')') NCELLS
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      IF(IVIEW.EQ.5) 
     &   WRITE(*,'(''      ..'',I10,'' cells were '',
     &          ''ommited (wrong ibound)'')') BADCEL
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''in confining layer'')') NUMCON
      WRITE(*,'(''      . Maximum value found in terms '',
     &          G20.10)') MAXTRM
      WRITE(*,'(''      . Minimum value found in terms '',
     &          G20.10)') MINTRM
C
C8B----Message to operational output file
C
      WRITE(OUTUNT,'(''      .Miscellaneous gridded term file '',
     &               A,
     &               '' consists of '')')
     &            ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(OUTUNT,'(''      .'',I10,'' cells'')') NCELLS
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      IF(IVIEW.EQ.5) 
     &   WRITE(OUTUNT,'(''      ..'',I10,'' cells were '',
     &          ''ommited (wrong ibound)'')') BADCEL
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''in confining layer'')') NUMCON
      WRITE(OUTUNT,'(''      . Maximum value found in terms '',
     &          G20.10)') MAXTRM
      WRITE(OUTUNT,'(''      . Minimum value found in terms '',
     &          G20.10)') MINTRM
C
C9----Arc Operations to coverage and attribute files
C
      NSTAT=2
      ITALK=1
      CALL DELETE_FILE (FILES(MAXFIL),*9999)
      CALL OPEN_FILE (IOUT(MAXFIL),FILES(MAXFIL),NSTAT,ITALK,*9999)
C
      IF(IVIEW.EQ.1 .OR. IVIEW.EQ.2 .OR. IVIEW.EQ.3) THEN
         NSTAT=1
         ITALK=1
         TMPFIL='arccrs'
         CALL OPEN_FILE (IOUT(1),TMPFIL,NSTAT,ITALK,*9999)
C
 902     READ(IOUT(1),'(A)',END=904) BUFFER
         WRITE(IOUT(MAXFIL),'(A)') BUFFER
         GO TO 902
C
 904     CALL CLOSE_FILE (TMPFIL,*9999)
         CALL DELETE_FILE (TMPFIL,*9999)
      ENDIF
C
C9A----Create attribute files and "add from" attributes
C
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) THEN
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
C
C9B---Join attribute files (if needed)
C
      ELSE
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         ACTION='kill'
         WRITE(IOUT(MAXFIL),'(A,3(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
         ARCCMD='joinitem'
         LABFIL=ARRFIL (1:INDEX(ARRFIL,' ')-1)//'.pat'
         INPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         RELITM=ARRFIL (1:INDEX(ARRFIL,' ')-1)//'-id'
         STRITM='thick'
         JONTYP='linear'
         WRITE(IOUT(MAXFIL),'(A,6(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             INPFIL (1:INDEX(INPFIL,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ENDIF
      CALL CLOSE_FILE (FILES(MAXFIL),*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &    '(//,'' Abnormal termination of Ptharr_Subroutine'')')
      WRITE(OUTUNT,
     &    '(//,'' Abnormal termination of Ptharr_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
