C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Cbcall version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 CBCALL 06DECEMBER2001
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for cell-by-cell flow terms:
C      constant_heads, flow_front_face, flow_right_face, and
C      flow_lower_face polygon coverages and attributes.  These
C      cell-by-cell flow terms are output of the Block-Centered Package of
C      the by the U.S. Geological Survey Modular Three-Dimensional
C      Finite-Differences Ground-Water Flow model MODFLOW or MODFLOWARC
C      (McDonald and Harbaugh, 1988; Orzol and McGrath, 1992).
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR MODTOOLS OUTPUT LSTING FILE
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     NLAY    : NUMBER OF LAYERS IN MODEL GRID
C     NROW    : NUMBER OF ROWS IN MODEL GRID
C     NCOL    : NUMBER OF COLUMNS IN MODEL GRID
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCCHDB  : STARTING ARRAY INDEX FOR CELL_BY_CELL CONSTANT_HEAD TERMS
C     LCFRFB  : STARTING ARRAY INDEX FOR CELL_BY_CELL FLOW_RIGHT_FACE TERMS
C     LCFFFB  : STARTING ARRAY INDEX FOR CELL_BY_CELL FLOW_FRONT_FACE TERMS
C     LCFLFB  : STARTING ARRAY INDEX FOR CELL_BY_CELL FLOW_LOWER_FACE TERMS
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE CBCALL (
     I                   OUTUNT,
     I                   LENX,
     I                   NLAY,NROW,NCOL,
     B                   ISUM,
     O                   LCCHDB,LCFFFB,LCFLFB,LCFRFB,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        NLAY,NROW,NCOL,
     &        ISUM,
     &        LCCHDB,LCFFFB,LCFLFB,LCFRFB
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL FLOW TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 06DECEMBER2001 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Calculate storage parameter NRCL
C
      NRCL=NROW*NCOL*NLAY
C
C2----Allocate space for arrays
C
      LCCHDB=ISUM
      ISUM=ISUM+NRCL
      LCFFFB=ISUM
      ISUM=ISUM+NRCL
      LCFLFB=ISUM
      ISUM=ISUM+NRCL
      LCFRFB=ISUM
      ISUM=ISUM+NRCL
      ISUM1=ISUM-1
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF CELL FLOW TERMS ATTRIBUTES   '',
     &             10X,I15,'' NRCL'')')
     &          NRCL
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CONSTANT HEAD TERMS   '',
     &             10X,I15,'' LCCHDB'')')
     &          LCCHDB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR FLOW FRONT FACE TERMS '',
     &             10X,I15,'' LCFFFB'')')
     &          LCFFFB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR FLOW LOWER FACE TERMS '',
     &             10X,I15,'' LCFLFB'')')
     &          LCFLFB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR FLOW RIGHT FACE TERMS '',
     &             10X,I15,'' LCFRFB'')')
     &          LCFRFB
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      ISP=4*NRCL
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR CELL-BY-CELL FLOW TERMS PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I8)')
     &          ISUM1,LENX
C
C4----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Cbcall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Cbcall_Subroutine'')')
         RETURN 1
      ENDIF
C
C5----Return
C
      RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Cbcrp1 version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 CBCRP1 14JUNE2005
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user specified files for cell_by_cell flow terms output by
C      the Block-Centered Package [BCF1, BCF2, or BCF3] of the U.S. Geological
C      Survey Modular Three-Dimensional Finite-Differences Ground-Water Flow 
C      Model MODFLOW (McDonald and Harbaugh, 1988).
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING HEDRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     BCFFIL  : FILE CONTAINING CELL_BY_CELL FLOW TERMS FILE
C     BCFUNT  : FORTRAN UNIT NUMBER CELL_BY_CELL FLOW TERMS FILE
C     CTLFIL  : FILE CONTAINING THE CONTROL LINES
C     CTLUNT  : FORTRAN UNIT NUMBER FOR U3DREL CONTROL LINES
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C
C    Outputs:
C
C     CHDTRM  : ARRAY FOR MODEL CONSTANT_HEAD_FLOWS
C     FRFTRM  : ARRAY FOR MODEL FLOW_RIGHT_FACE_FLOWS
C     FFFTRM  : ARRAY FOR MODEL FLOW_FRONT_FACE_FLOWS
C     FLFTRM  : ARRAY FOR MODEL FLOW_LOWER_FACE_FLOWS
C
C     Program:
C
C     ANAME   : CHARACTER ARRAY (6,8) CONTAINING ARRAY NAMES
C
C-------------------------------------------------------------------------
      SUBROUTINE CBCRP1 (
     I                   OUTFIL,OUTUNT,
     I                   BCFFIL,BCFUNT,
     I                   CTLFIL,CTLUNT,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,
     O                   CHDTRM,FFFTRM,FLFTRM,FRFTRM,
     E                   *)
C-----------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     &          COVPMR(MAXPMR),
     &          CHDTRM(NCOL*NROW*NLAY),
     &          FRFTRM(NCOL*NROW*NLAY),
     &          FFFTRM(NCOL*NROW*NLAY),
     &          FLFTRM(NCOL*NROW*NLAY)
C
      CHARACTER*(*) OUTFIL,BCFFIL,CTLFIL
C
      INTEGER OUTUNT,BCFUNT,CTLUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*16 TEXT
      CHARACTER*4 ANAME(6,12)
      CHARACTER*16 BUFTXT
C
      INTEGER ICOL,IROW,ILAY,LAYER,KK,BCFFLG,CBCUNT
      EXTERNAL READ_UBUDSV
C
      DATA ANAME(1,1),ANAME(2,1),ANAME(3,1),ANAME(4,1),ANAME(5,1),
     1   ANAME(6,1) /'    ','    ','   C','ONST','ANT ','HEAD'/
      DATA ANAME(1,2),ANAME(2,2),ANAME(3,2),ANAME(4,2),ANAME(5,2),
     1   ANAME(6,2) /'    ','    ','  FL','OW R','IGHT','FACE'/
      DATA ANAME(1,3),ANAME(2,3),ANAME(3,3),ANAME(4,3),ANAME(5,3),
     1   ANAME(6,3) /'    ','    ','  FL','OW F','RONT','FACE'/
      DATA ANAME(1,4),ANAME(2,4),ANAME(3,4),ANAME(4,4),ANAME(5,4),
     1   ANAME(6,4) /'    ','    ','  FL','OW L','OWER','FACE'/
C
C1*********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                       Block-Centered Operations
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C1*********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL FLOW TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 14JUNE2005 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1L----Set local variables
C
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
C
C1----Print message
C
      WRITE(*,'(
     &             ''      .Reading cell_by_cell '',
     &             ''flow terms from Block_Centered Flow Package'')')
      WRITE(*,'(
     &             ''      ..for Modflow stress period '',I10)')
     &             MODPER
      WRITE(*,'(
     &             ''      ..for Modflow time step '',I10)')
     &             MODSTP
C
      WRITE(OUTUNT,'(
     &             ''      .Reading cell_by_cell '',
     &             ''flow terms from Block_Centered Flow Package'')')
      WRITE(OUTUNT,'(
     &             ''      ..for Modflow stress period '',I10)')
     &             MODPER
      WRITE(OUTUNT,'(
     &             ''      ..for Modflow time step '',I10)')
     &             MODSTP
C
C2----Set cell-by-cell array
C
      LOC=0
C
      DO 200 KK=1,NLAY
      DO 200 JJ=1,NCOL
      DO 200 II=1,NROW 
         LOC=LOC+1
         CHDTRM(LOC)=HNOFLO
         FRFTRM(LOC)=HNOFLO
         FFFTRM(LOC)=HNOFLO
 200     FLFTRM(LOC)=HNOFLO
C
C3---Unformatted cell-by-cell flow terms
C
      IF(BCFUNT.LT.0) THEN
         WRITE(*,'(
     &             ''      ..for unformatted cell_by_cell '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      ..for unformatted cell_by_cell '',
     &             ''flow terms '')')
C
C3A---Open unformatted file
C
         CBCUNT=ABS(BCFUNT)
         WRITE(*,'(''      ....recorded in file '',A)')
     &             BCFFIL (1:INDEX(BCFFIL,' ')-1)
         WRITE(*,'(''      ....on unit '',I10)')
     &                                                CBCUNT
         WRITE(OUTUNT,'(''      ....recorded in file '',A)')
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
         WRITE(OUTUNT,'(''      ....on unit '',I10)')
     &                                                CBCUNT
C
C3B---Loop to read from unformatted file
C
         BCFFLG=0
         WRITE(*,'(
     &             ''      .....reading cell_by_cell constant head '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      .....reading cell_by_cell constant head '',
     &             ''flow terms '')')
C
         BUFTXT='   CONSTANT HEAD'
         CALL READ_UBUDSV (
     I                     BCFFIL,
     I                     BUFTXT,
     I                     MODPER,
     I                     MODSTP,
     O                     CHDTRM(1),
     E                     BCFFLG)
C
       IF(BCFFLG.EQ.NLAY) THEN
          WRITE(*,'(
     &              ''      .....loaded cell_by_cell constant head '',
     &              ''flow terms '')')
          WRITE(OUTUNT,'(
     &              ''      .....loaded cell_by_cell constant head '',
     &              ''flow terms '')')
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR ALL LAYERS '',/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             BUFTXT,CBCUNT
       ELSE
          WRITE(*,'(
     &             ''      ..Did not find cell_by_cell constant head '',
     &             ''flow terms in file '',A)')
     &             BCFFIL (1:INDEX(BCFFIL,' ')-1)
          WRITE(OUTUNT,'(
     &             ''      ..Did not find cell_by_cell constant head '',
     &             ''flow terms in file '',A)')
     &             BCFFIL (1:INDEX(BCFFIL,' ')-1)
          GO TO 9998
       ENDIF
C
C3C---Loop to read flow right face
C
         BCFFLG=0
         WRITE(*,'(
     &             ''      .....reading cell_by_cell flow right face '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      .....reading cell_by_cell flow right face '',
     &             ''flow terms '')')
C
         BUFTXT='FLOW RIGHT FACE '
         CALL READ_UBUDSV (
     I                     BCFFIL,
     I                     BUFTXT,
     I                     MODPER,
     I                     MODSTP,
     O                     FRFTRM(1),
     E                     BCFFLG)
C
       IF(BCFFLG.EQ.NLAY) THEN
          WRITE(*,'(
     &              ''      .....loaded cell_by_cell flow lower face '',
     &              ''flow terms '')')
          WRITE(OUTUNT,'(
     &              ''      .....loaded cell_by_cell flow lower face '',
     &              ''flow terms '')')
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR ALL LAYERS '',/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             BUFTXT,CBCUNT
       ELSE
          WRITE(*,'(
     &              ''      ..Did not find cell_by_cell '',
     &              ''flow right face flow terms in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..Did not find cell_by_cell '',
     &              ''flow right face flow terms in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          GO TO 9998
       ENDIF
C
C3D---Loop to read flow front face
C
         BCFFLG=0
         WRITE(*,'(
     &             ''      .....reading cell_by_cell flow front face '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      .....reading cell_by_cell flow front face '',
     &             ''flow terms '')')
C
         BUFTXT='FLOW FRONT FACE '
         CALL READ_UBUDSV (
     I                     BCFFIL,
     I                     BUFTXT,
     I                     MODPER,
     I                     MODSTP,
     O                     FFFTRM(1),
     E                     BCFFLG)
C
       IF(BCFFLG.EQ.NLAY) THEN
          WRITE(*,'(
     &              ''      .....loaded cell_by_cell flow front face '',
     &              ''flow terms '')')
          WRITE(OUTUNT,'(
     &              ''      .....loaded cell_by_cell flow front face '',
     &              ''flow terms '')')
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR ALL LAYERS '',/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             BUFTXT,CBCUNT
       ELSE
          WRITE(*,'(
     &              ''      ..Did not find cell_by_cell '',
     &              ''flow front face flow terms in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..Did not find cell_by_cell '',
     &              ''flow front face flow terms in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          GO TO 9998
       ENDIF
C
C3E---Loop to read flow lower face
C
         BCFFLG=0
         WRITE(*,'(
     &             ''      .....reading cell_by_cell flow lower face '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      .....reading cell_by_cell flow lower face '',
     &             ''flow terms '')')
C
         BUFTXT='FLOW LOWER FACE '
         CALL READ_UBUDSV (
     I                     BCFFIL,
     I                     BUFTXT,
     I                     MODPER,
     I                     MODSTP,
     O                     FLFTRM(1),
     E                     BCFFLG)
C
       IF(BCFFLG.EQ.NLAY) THEN
          WRITE(*,'(
     &              ''      .....loaded cell_by_cell flow lower face '',
     &              ''flow terms '')')
          WRITE(OUTUNT,'(
     &              ''      .....loaded cell_by_cell flow lower face '',
     &              ''flow terms '')')
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR ALL LAYERS '',/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             BUFTXT,CBCUNT
       ELSE
          WRITE(*,'(
     &              ''      ..Did not find cell_by_cell '',
     &              ''flow lower face flow terms in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..Did not find cell_by_cell '',
     &              ''flow lower face flow terms in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          GO TO 9998
       ENDIF
C
C4---Text formatted cell-by-cell flow terms
C
      ELSE IF(BCFUNT.GT.0) THEN
         WRITE(*,'(
     &             ''      ..reading text formatted cell_by_cell '',
     &             ''flow terms '')')
         WRITE(OUTUNT,'(
     &             ''      ..reading text formatted cell_by_cell '',
     &             ''flow terms '')')
C
C4A----Open control file CTLFIL and text formatted file BCFFIL
C
       IF(CTLUNT.GT.0) THEN
          NSTAT=1
          ITALK=1
          CALL OPEN_FILE (CTLUNT,CTLFIL,NSTAT,ITALK,*9999)
          CALL OPEN_FILE (BCFUNT,BCFFIL,NSTAT,ITALK,*9999) 
          WRITE(OUTUNT,'(
     &              ''      ....on unit '',I10)')
     &                                                 BCFUNT
          WRITE(OUTUNT,'(
     &              ''      ...using control lines '')')
          WRITE(OUTUNT,'(
     &              ''      ....recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
       ELSE
          WRITE(*,'(
     &              ''      ..Can not read text formatted '',
     &              ''cell_by_cell flow terms '')')
          WRITE(*,'(
     &              ''      ....recorded in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          WRITE(*,'(
     &              ''      ....on unit '',I10)')
     &                                                 BCFUNT
          WRITE(*,'(
     &              ''      ..Need file containing U3DREL control '',
     &              ''lines for format of cell-by-cell flow file'')') 
          WRITE(OUTUNT,'(
     &              ''      ..Can not read text formatted '',
     &              ''cell_by_cell flow terms '')')
          WRITE(OUTUNT,'(
     &              ''      ....recorded in file '',A)')
     &              BCFFIL (1:INDEX(BCFFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....on unit '',I10)')
     &                                                 BCFUNT
          WRITE(OUTUNT,'(
     &              ''      ..Need file containing U3DREL control '',
     &              ''lines for format of cell-by-cell flow file'')') 
          CALL CLOSE_FILE (BCFFIL,*9999)
          GO TO 9999
       ENDIF
C
C4B---Loop to read from text formatted file BCFFIL
C
         CALL U3DREL (CHDTRM(1),ANAME(1,1),NROW,NCOL,NLAY,BCFUNT,
     &                OUTUNT)
       IF(NCOL.GT.1)
     &    CALL U3DREL (FRFTRM(1),ANAME(1,2),NROW,NCOL,NLAY,BCFUNT,
     &                 OUTUNT)
       IF(NROW.GT.1)
     &    CALL U3DREL (FFFTRM(1),ANAME(1,3),NROW,NCOL,NLAY,BCFUNT,
     &                 OUTUNT)
       IF(NLAY.GT.1)
     &    CALL U3DREL (FLFTRM(1),ANAME(1,4),NROW,NCOL,NLAY,BCFUNT,
     &                 OUTUNT)
C
C5---Close text formatted file BCFFIL
C
         CALL CLOSE_FILE (BCFFIL,*9999)
         CALL CLOSE_FILE (CTLFIL,*9999)
C
C6---Error no file containing cell-by-cell flow terms
C
      ELSE
         WRITE(*,'(
     &             ''      ..No file containing cell_by_cell '',
     &             ''flow terms specified in MODPATH name file'')')
         WRITE(OUTUNT,'(
     &             ''      ..No file containing cell_by_cell '',
     &             ''flow terms specified in MODPATH name file'')')
         GO TO 9999
      ENDIF
        GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 6660 WRITE(*,'(//,
     &          '' Unable to read header line for '',
     &          ''constant head terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''constant head terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 6661 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''constant head terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read values for '',
     &          ''constant head terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 6662 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''constant head terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''constant head terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 7770 WRITE(*,'(//,
     &          '' Unable to read header line for '',
     &          ''flow right face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''flow right face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 7771 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''flow right face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read values for '',
     &          ''flow right face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 7772 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow right face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow right face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 8880 WRITE(*,'(//,
     &          '' Unable to read header line for '',
     &          ''flow front face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''flow front face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 8881 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''flow front face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read values for '',
     &          ''flow front face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 8882 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow front face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow front face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 9990 WRITE(*,'(//,
     &          '' Unable to read header line for '',
     &          ''flow lower face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''flow lower face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''flow lower face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read values for '',
     &          ''flow lower face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 9992 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow lower face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''flow lower face terms from file '',A)') 
     &          BCFFIL (1:INDEX(BCFFIL,' ')-1)
      GO TO 9998
 9998 CALL CLOSE_FILE (BCFFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Cbcrp1_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Cbcrp1_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthcbc version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C
C---VERSION 6.1 PTHCBC 14DECEMBER2001
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a Info file BCFFIL using the cell-by-cell 
C      flow terms that are output by the Block-Centered package of the 
C      U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C      IVIEW
C           => 0 Mapview --> all layers
C                 Four info files of cell-by-cell flow terms
C                  --> CBCFIL_CHD FOR CELL_BY_CELL CONSTANT_HEADS
C                  --> CBCFIL_FRF FOR CELL_BY_CELL FLOW_RIGHT_FACE
C                  --> CBCFIL_FFF FOR CELL_BY_CELL FLOW_FRONT_FACE
C                  --> CBCFIL_FLF FOR CELL_BY_CELL FLOW_LOWER_FACE
C             Info File Structure:
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4+NLAY) LAYER_XX = CELL_BY_CELL FLOW TERMS
C                             AND XX = CHOSEN LAYER
C
C           => 1 Vertical row profile
C                 Cell-by-cell flow terms are appended to cell coverage
C              => (n,1) CHDTRM CELL_BY_CELL CONSTANT_HEADS TERMS
C              => (n,2) FRFTRM CELL_BY_CELL FLOW_RIGHT_FACE
C              => (n,3) FRFTRM CELL_BY_CELL FLOW_FRONT_FACE
C              => (n,4) FRFTRM CELL_BY_CELL FLOW_LOWER_FACE
C
C           => 2 Vertical column profile
C                 Cell-by-cell flow terms are appended to cell coverage
C              => (n,1) CHDTRM CELL_BY_CELL CONSTANT_HEADS TERMS
C              => (n,2) FRFTRM CELL_BY_CELL FLOW_RIGHT_FACE
C              => (n,3) FRFTRM CELL_BY_CELL FLOW_FRONT_FACE
C              => (n,4) FRFTRM CELL_BY_CELL FLOW_LOWER_FACE
C
C           => 3 Vertical particle profile
C                 Cell-by-cell flow terms are appended to cell coverage
C              => (n,1) CHDTRM CELL_BY_CELL CONSTANT_HEADS TERMS
C              => (n,2) FRFTRM CELL_BY_CELL FLOW_RIGHT_FACE
C              => (n,3) FRFTRM CELL_BY_CELL FLOW_FRONT_FACE
C              => (n,4) FRFTRM CELL_BY_CELL FLOW_LOWER_FACE
C
C           => 4 Mapview --> user specified layer
C                 Four info file of cell-by-cell flow terms
C                  --> CBCFIL_CHD FOR CELL_BY_CELL CONSTANT_HEADS
C                  --> CBCFIL_FRF FOR CELL_BY_CELL FLOW_RIGHT_FACE
C                  --> CBCFIL_FFF FOR CELL_BY_CELL FLOW_FRONT_FACE
C                  --> CBCFIL_FLF FOR CELL_BY_CELL FLOW_LOWER_FACE
C             Info File Structure:
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) LAYER_XX = CELL_BY_CELL FLOW TERMS FOR CHOSEN LAYER
C                                  AND XX = CHOSEN LAYER
C
C           => 5 Mapview --> user specified ibound value
C                 Info file of cell-by-cell flow terms
C                 (this ibound value can be used to differentiate model
C                  layers into geological units)
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL FLOW TERMS 
C                       (these terms are computed as a summation of a weighted
C                         value based upon the saturated thickness of each 
C                         layer that has the chosen ibound value divided by
C                         the total saturated of this ibound value over all 
C                         layers times the cell-by-cell flow term for 
C                         each layer)
C
C           => 6 Mapview --> uppermost active cell
C                 One info file of cell-by-cell flow terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL FLOW TERMS
C
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     CBCFIL  : ARC/INFO FILE NAME THAT WILL CONTAIN CELL-BY-CELL FLOW TERMS
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     COVPMR  : ARRAY FOR COVERAGE PARAMETERS (MAXPMR)
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NDELX)
C     IBOUND  : ARRAY FOR IBOUND VALUES (NCOL,NROW,NLAY)
C     CHDTRM  : ARRAY FOR CONSTANT HEAD VALUES (NCOL,NROW,NLAY)
C     FRFTRM  : ARRAY FOR FLOW RIGHT FACE VALUES (NCOL,NROW,NLAY)
C     FFFTRM  : ARRAY FOR FLOW FRONT FACE VALUES (NCOL,NROW,NLAY)
C     FLFTRM  : ARRAY FOR FLOW LOWER FACE VALUES (NCOL,NROW,NLAY)
C
C   Inputs/Outputs: None
C
C    Outputs: None
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PTHCBC (
     I                   OUTUNT,
     I                   CBCFIL,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,LRWCL,
     I                   IBOUND,LAYCBD,
     I                   CHDTRM,FRFTRM,FFFTRM,FLFTRM,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     1          COVPMR(MAXPMR),
     1          LAYCBD(NLAY),
     2          LRWCL(2,NROW*NCOL),
     3          IBOUND(NCOL,NROW,NLAY),
     3          CHDTRM(NCOL,NROW,NLAY),
     3          FRFTRM(NCOL,NROW,NLAY),
     3          FFFTRM(NCOL,NROW,NLAY),
     3          FLFTRM(NCOL,NROW,NLAY)
C
      CHARACTER*(*) CBCFIL
C
      INTEGER OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER (MAXFIL=9,
     &           MAXITM=5)
C
      CHARACTER*128 TMPCOV,LABFIL,BUFFER
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL
      CHARACTER*16 COVTYP,ITEM,ITEMS(MAXITM),ARCCMD,ACTION,
     &             LAYITM,STRITM,RELITM,JONTYP
      CHARACTER*10 CHRLAY,CHRWCL,LAYFMT,ITMFMT
      CHARACTER*1 TYPITM,TYPES(MAXITM)
C
      INTEGER DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN,
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM)
C
      INTEGER
     &        IDPART,ICOLOR,TIMSTP,OLDSTP,
     &        STRCHR,STRCOL,STRROW,STRLAY,STRIBD,STRDEL,STRCLL,
     &        PRECOL,PREROW,PRELAY,PREIBD,
     &        FINCHR,FINCOL,FINROW,FINLAY,FINIBD,FINDEL,FINCLL,
     &        ROW,COLUMN,CELL,BADIBD,COVDEX,COL,ROWCOL,SEQNUM,
     &        NCELLS,NARCS,BADCEL,
     &        NUMCOL,NUMROW,NUMLAY,
     &        IRWCOL,IRWCL,IVIEW
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA FILES /'chditems','frfitems','fffitems','flfitems',
     &            'chditem','frfitem','fffitem','flfitem',
     &            'arccmd'/
      DATA IOUT /90,91,92,93,
     &           90,91,92,93,94/
      DATA ITEMS /'SEQNUM','ROWCOL','ROW','COLUMN','LAYER_'/
      DATA DECIMLS /4*0, 6/
      DATA TYPES /'I','C',2*'I','F'/
      DATA WIDTHS /4*10,8/
      DATA OUTPUTS /4*10,16/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL FLOW TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 14DECEMBER2001 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                  GIS OUTPUT MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1L----Set local variables
C
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
C
C2----Delete and create files for attribute definitions, attribute value, 
C      and coverage arc data and tolerance/dangle values
C
      NSTAT=2
      ITALK=1
C
C2A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      DO 200 N=1,MAXFIL
         CALL DELETE_FILE (FILES(N),*9999)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      WIDTHS(1)=LENROW+LENCOL
      OUTPUTS(1)=LENROW+LENCOL
      WIDTHS(2)=LENROW+LENCOL
      OUTPUTS(2)=LENROW+LENCOL
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
C
C2C----Write standard attribute definitions
C
      IF(IVIEW.EQ.0 .OR. IVIEW.EQ.4) THEN
         DO 210 M=1,4
            CALL OPEN_FILE (IOUT(M),FILES(M),NSTAT,ITALK,*9999)
            DO 220 K=1,4
               DECIML=DECIMLS(K)
               ITEM=ITEMS(K)
               OUTPUT=OUTPUTS(K)
               TYPITM=TYPES(K)
               WIDTH=WIDTHS(K)
               WRITE(IOUT(M),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                         I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  220          CONTINUE
C
C2Ca----Write attribute definitions IVIEW = all layers
C
        IF(IVIEW.LE.0) THEN
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           DO 230 NUMLAY=1,NLAY
            IF(NUMLAY.LT.10) THEN
               WRITE(CHRLAY,'(I1)') NUMLAY
            ELSE IF(NUMLAY.GT.9 .AND. NUMLAY.LT.100) THEN
               WRITE(CHRLAY,'(I2)') NUMLAY
            ELSE IF(NUMLAY.GT.99 .AND. NUMLAY.LT.1000) THEN
               WRITE(CHRLAY,'(I3)') NUMLAY
            ELSE IF(NUMLAY.GT.999 .AND. NUMLAY.LT.1000) THEN
               WRITE(CHRLAY,'(I4)') NUMLAY
            ENDIF
              ITEM='LAYER_'//CHRLAY
              WRITE(IOUT(M),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                        I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  230      CONTINUE
C
C2Cb----Write attribute definitions IVIEW = single layer or ibound value
C
        ELSE
           DECIML=DECIMLS(MAXITM)
           ITEM=ITEMS(MAXITM)
           OUTPUT=OUTPUTS(MAXITM)
           TYPITM=TYPES(MAXITM)
           WIDTH=WIDTHS(MAXITM)
           NUMLAY=IRWCL
         IF(NUMLAY.LT.10) THEN
            WRITE(CHRLAY,'(I1)') NUMLAY
         ELSE IF(NUMLAY.GT.9 .AND. NUMLAY.LT.100) THEN
            WRITE(CHRLAY,'(I2)') NUMLAY
         ELSE IF(NUMLAY.GT.99 .AND. NUMLAY.LT.1000) THEN
            WRITE(CHRLAY,'(I3)') NUMLAY
         ELSE IF(NUMLAY.GT.999 .AND. NUMLAY.LT.1000) THEN
            WRITE(CHRLAY,'(I4)') NUMLAY
         ENDIF
           ITEM='LAYER_'//CHRLAY
           WRITE(IOUT(M),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                     I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         ENDIF
           CALL CLOSE_FILE (FILES(M),*9999)
           NINDEX=INDEX(CBCFIL,' ')-1
         IF(NINDEX.LT.28) THEN
            FILES(M+4)=CBCFIL (1:NINDEX)//'_'//FILES(M) (1:3)
            CALL DELETE_FILE (FILES(M+4),*9999)
         ELSE
            FILES(M+4)=CBCFIL (1:28)//'_'//FILES(M) (1:3)
            CALL DELETE_FILE (FILES(M+4),*9999)
         ENDIF
  210  CONTINUE
C
C2Ca----Write attribute definitions IVIEW = uppermost active cell
C
      ELSE IF(IVIEW.EQ.6 .OR. IVIEW.EQ.5) THEN
         CALL OPEN_FILE (IOUT(1),FILES(1),NSTAT,ITALK,*9999)
         FILES(5)=CBCFIL
         CALL DELETE_FILE (FILES(5),*9999)
         DO 240 K=1,4
            DECIML=DECIMLS(K)
            ITEM=ITEMS(K)
            OUTPUT=OUTPUTS(K)
            TYPITM=TYPES(K)
            WIDTH=WIDTHS(K)
            WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                 ITEM,
     &                                 WIDTH,
     &                                 OUTPUT,
     &                                 TYPITM,
     &                                 DECIML
  240       CONTINUE
C
         DECIML=0
       IF(IVIEW.EQ.5) THEN
          ITEM='IBOUND'
          OUTPUT=10
          TYPITM='I'
          WIDTH=10
          WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
       ELSE
          DECIML=DECIMLS(4)
          ITEM='LAYER'
          OUTPUT=LENLAY
          TYPITM=TYPES(4)
          WIDTH=LENLAY
          WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
          ITEM='IBOUND'
          WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
       ENDIF
         DECIML=DECIMLS(MAXITM)
         ITEM=ITEMS(MAXITM)
         OUTPUT=OUTPUTS(MAXITM)
         TYPITM=TYPES(MAXITM)
         WIDTH=WIDTHS(MAXITM)
         DO 250 M=1,4
          IF(M.EQ.1) ITEM='CHDTRM'
          IF(M.EQ.2) ITEM='FRFTRM'
          IF(M.EQ.3) ITEM='FFFTRM'
          IF(M.EQ.4) ITEM='FLFTRM'
            WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  250    CONTINUE
         CALL CLOSE_FILE (FILES(1),*9999)
C
C2D----Write standard attribute definitions 
C
      ELSE
         CALL OPEN_FILE (IOUT(1),FILES(1),NSTAT,ITALK,*9999)
         DECIML=0
         ITEM=CBCFIL (1:INDEX(CBCFIL,' ')-1)//'-id'
         OUTPUT=5
         TYPITM='B'
         WIDTH=4
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
         DECIML=DECIMLS(MAXITM)
         ITEM=ITEMS(MAXITM)
         OUTPUT=OUTPUTS(MAXITM)
         TYPITM=TYPES(MAXITM)
         WIDTH=WIDTHS(MAXITM)
         DO 260 M=1,4
          IF(M.EQ.1) ITEM='CHDTRM'
          IF(M.EQ.2) ITEM='FRFTRM'
          IF(M.EQ.3) ITEM='FFFTRM'
          IF(M.EQ.4) ITEM='FLFTRM'
            WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
  260    CONTINUE
         CALL CLOSE_FILE (FILES(1),*9999)
      ENDIF
C
C3----Build cell-by-cell terms
C
      BADIBD=0
      BADCEL=0
      NUMCLL=0
      NCELLS=0
      NUMCON=0
C
C3M---Message
C
      WRITE(*,'(
     &             ''      .Block-Centered Package '')')
      WRITE(OUTUNT,'(
     &             ''      .Block-Centered Package '')')
C
      IF(IVIEW.LE.0) THEN
         WRITE(*,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''for all layers '',I10)')
     &         NLAY
         WRITE(OUTUNT,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''for all layers '',I10)')
     &         NLAY
      ELSE IF(IVIEW.EQ.1) THEN
         WRITE(*,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''along row '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''along row '',I10,'' now...'')')
     &         IRWCL
      ELSE IF(IVIEW.EQ.2) THEN
         WRITE(*,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''along column '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''along column '',I10,'' now...'')')
     &         IRWCL
      ELSE IF(IVIEW.EQ.3) THEN
         WRITE(*,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''along particle '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''along particle '',I10,'' now...'')')
     &         IRWCL
      ELSE IF(IVIEW.EQ.4) THEN
         WRITE(*,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''for layer '',I10)')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      ..Processing cell-by-cell flow terms '',
     &             ''for layer '',I10)')
     &         IRWCL
      ELSE IF(IVIEW.EQ.5) THEN
         WRITE(*,'(
     &             ''      ..Processing the cell-by-cell flow terms '',
     &             ''for ibound value '',I10,'' now...'')')
     &         IRWCL
         WRITE(OUTUNT,'(
     &             ''      ..Processing the cell-by-cell flow terms '',
     &             ''for ibound value '',I10,'' now...'')')
     &         IRWCL
      ELSE
         WRITE(*,'(
     &             ''      ..Processing the cell-by-cell flow terms '',
     &             ''for uppermost active cell...'')')
         WRITE(OUTUNT,'(
     &             ''      ..Processing the cell-by-cell flow terms '',
     &             ''for uppermost active cell...'')')
      ENDIF
C
C3H----Data header
C
      WRITE(OUTUNT,'(//)')
C
      IF(IVIEW.EQ.0) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR ALL LAYERS'')')
      ELSE IF(IVIEW.EQ.1) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS ALONG ROW DIRECTION'')')
      ELSE IF(IVIEW.EQ.2) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS ALONG COLUMN DIRECTION'')')
      ELSE IF(IVIEW.EQ.3) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS ALONG PARTICLE DIRECTION'')')
      ELSE IF(IVIEW.EQ.4) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR LAYER '',I10)')
     &         IRWCL
      ELSE IF(IVIEW.EQ.5) THEN
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR IBOUND '',I10)')
     &         IRWCL
      ELSE
         WRITE(OUTUNT,'(40X,
     &               '' CELLS CONDITIONS MAPVIEW FOR'',
     &               '' UPPER MOST ACTIVE CELL'')')
      ENDIF
C
      WRITE(OUTUNT,'(10X,108(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &            ''       ROW'',
     &            ''    COLUMN'',
     &            ''     LAYER'',
     &            ''    IBOUND'',
     &            '' CONSTANT HEAD   '',
     &            '' FLOW RIGHT FACE '',
     &            '' FLOW FRONT FACE ''
     &            '' FLOW LOWER FACE '')')
      WRITE(OUTUNT,'(10X,108(''-''))')
C
C3---Build mapview for single layer (iview=0,4)
C
      IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
C
C3A----Loading files containing item attribute data
C
         CALL OPEN_FILE (IOUT(5),FILES(5),NSTAT,ITALK,*9999)
         CALL OPEN_FILE (IOUT(6),FILES(6),NSTAT,ITALK,*9999)
         CALL OPEN_FILE (IOUT(7),FILES(7),NSTAT,ITALK,*9999)
         CALL OPEN_FILE (IOUT(8),FILES(8),NSTAT,ITALK,*9999)
C
C3Ba----All layers IVIEW = 0
C
         DO 300 IROW=1,NROW
            DO 310 ICOL=1,NCOL
               SEQNUM=(IROW-1)*NCOL+ICOL
               NUMCLL=(IROW-1)*NCOL+ICOL
               ROWCOL=IROW*(10**LENCOL)+ICOL
               WRITE(CHRWCL,'(I10)') ROWCOL
               FINCHR=10
               STRCHR=FINCHR-(LENROW+LENCOL-1)
             IF(IVIEW.LE.0) THEN
                WRITE(IOUT(5),'(
     &                          I10,'','',
     &                          A,'','',
     &                          2(I10,'',''),
     &                          200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        (CHDTRM(ICOL,IROW,NUMLAY), NUMLAY=1,NLAY)
                WRITE(IOUT(6),'(
     &                          I10,'','',
     &                          A,'','',
     &                          2(I10,'',''),
     &                          200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        (FRFTRM(ICOL,IROW,NUMLAY), NUMLAY=1,NLAY)
                WRITE(IOUT(7),'(
     &                          I10,'','',
     &                          A,'','',
     &                          2(I10,'',''),
     &                          200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        (FFFTRM(ICOL,IROW,NUMLAY), NUMLAY=1,NLAY)
                WRITE(IOUT(8),'(
     &                          I10,'','',
     &                          A,'','',
     &                          2(I10,'',''),
     &                          200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        (FLFTRM(ICOL,IROW,NUMLAY), NUMLAY=1,NLAY)
                DO 320 ILAY=1,NLAY
                   IBDCEL=IBOUND(ICOL,IROW,ILAY)
                 IF(IBDCEL.NE.0) THEN
                    NCELLS=NCELLS+1
                    WRITE(OUTUNT,'(10X,4I10,4F17.6)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          CHDTRM(ICOL,IROW,ILAY),
     &                          FRFTRM(ICOL,IROW,ILAY),
     &                          FFFTRM(ICOL,IROW,ILAY),
     &                          FLFTRM(ICOL,IROW,ILAY)
                 ELSE
                    BADIBD=BADIBD+1
                    WRITE(OUTUNT,'(10X,4I10,20X,
     &                             ''           IBOUND = 0         '')')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                 ENDIF
 320            CONTINUE
C
C3Bb----Specified layer IVIEW = 4
C
             ELSE
                ILAY=IRWCL
                IBDCEL=IBOUND(ICOL,IROW,ILAY)
              IF(IBDCEL.NE.0) THEN
                 NCELLS=NCELLS+1
                 WRITE(IOUT(5),'(
     &                           I10,'','',
     &                           A,'','',
     &                           2(I10,'',''),
     &                           200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        CHDTRM(ICOL,IROW,ILAY)
                 WRITE(IOUT(6),'(
     &                           I10,'','',
     &                           A,'','',
     &                           2(I10,'',''),
     &                           200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        FRFTRM(ICOL,IROW,ILAY)
                 WRITE(IOUT(7),'(
     &                           I10,'','',
     &                           A,'','',
     &                           2(I10,'',''),
     &                          200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        FFFTRM(ICOL,IROW,ILAY)
                 WRITE(IOUT(8),'(
     &                           I10,'','',
     &                           A,'','',
     &                           2(I10,'',''),
     &                           200(F16.3,'',''))')
     &                        SEQNUM,
     &                        CHRWCL (STRCHR:FINCHR),
     &                        IROW,
     &                        ICOL,
     &                        FLFTRM(ICOL,IROW,ILAY)
                 WRITE(OUTUNT,'(10X,4I10,4F17.6)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          CHDTRM(ICOL,IROW,ILAY),
     &                          FRFTRM(ICOL,IROW,ILAY),
     &                          FFFTRM(ICOL,IROW,ILAY),
     &                          FLFTRM(ICOL,IROW,ILAY)
              ELSE
                 BADIBD=BADIBD+1
                 WRITE(OUTUNT,'(10X,4I10,20X,
     &                          ''INACTIVE CELL                 '')')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
              ENDIF
             ENDIF
  310       CONTINUE
  300    CONTINUE
C
C4----Build cross-sectional view
C
      ELSE IF(IVIEW.GE.1 .AND. IVIEW.LE.3) THEN
C
C4Aa----Create files containing Arc/Info item attribute files
C
         CALL OPEN_FILE (IOUT(5),FILES(5),NSTAT,ITALK,*9999)
C
       IF(IVIEW.EQ.1) THEN
          STRCLL=1
          FINCLL=NCOL
       ELSE IF(IVIEW.EQ.2) THEN
          STRCLL=NCOL+1
          FINCLL=NCOL+NROW
       ELSE
          STRCLL=1
          FINCLL=IRWCL
       ENDIF
C
C4B----Loop to find minimum and maximum elevations
C
         DO 400 N=STRCLL,FINCLL
            NUMCLL=NUMCLL+1
            DO 410 ILAY=1,NLAY
             IF(IVIEW.LE.1) THEN
                IROW=IRWCL
                ICOL=NUMCLL
             ELSE IF(IVIEW.EQ.2) THEN
                IROW=NUMCLL
                ICOL=IRWCL
             ELSE
                ICOL=LRWCL(1,NUMCLL)
                IROW=LRWCL(2,NUMCLL)
             ENDIF
C
C4Ba----Cell activity
C
               IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
C
C4C----Write attributes of each cell and confining layer cells
C
             IF(IBDCEL.NE.0) THEN
                NCELLS=NCELLS+1
                WRITE(IOUT(5),'(I10,'','',
     &                          4(F16.3,'',''))')
     &                         NCELLS,
     &                         CHDTRM(ICOL,IROW,ILAY),
     &                         FRFTRM(ICOL,IROW,ILAY),
     &                         FFFTRM(ICOL,IROW,ILAY),
     &                         FLFTRM(ICOL,IROW,ILAY)
                 WRITE(OUTUNT,'(10X,4I10,4F17.6)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          CHDTRM(ICOL,IROW,ILAY),
     &                          FRFTRM(ICOL,IROW,ILAY),
     &                          FFFTRM(ICOL,IROW,ILAY),
     &                          FLFTRM(ICOL,IROW,ILAY)
C
              IF(LAYCBD(ILAY).GT.0.0) THEN
                 NUMCON=NUMCON+1
                 WRITE(OUTUNT,'(10X,
     &                   ''CONFINING LAYER CELL          '',
     &                   4I10)')
     &                           IROW,
     &                           ICOL,
     &                           ILAY,
     &                           IBDCEL
                 NCELLS=NCELLS+1
                 CHDIBD=0.0
                 FRFIBD=0.0
                 FFFIBD=0.0
                 FLFIBD=0.0
                 WRITE(IOUT(5),'(I10,'','',
     &                           4(F16.3,'',''))')
     &                          NCELLS,
     &                          CHDIBD,
     &                          FRFIBD,
     &                          FFFIBD,
     &                          FLFIBD
              ENDIF
C
C4Cb----Omit label point attributes of each cell
C
             ELSE
                BADIBD=BADIBD+1
                WRITE(OUTUNT,'(10X,4I10,20X,
     &                         ''INACTIVE CELL                 '')')
     &                         IROW,
     &                         ICOL,
     &                         ILAY,
     &                         IBDCEL
             ENDIF
  410       CONTINUE
  400    CONTINUE
C
C5----Build mapview for single ibound value (iview=5)
C
      ELSE IF(IVIEW.EQ.5) THEN
         BADCEL=0
C
C5A----Create files containing Arc/Info item attribute files
C
         CALL OPEN_FILE (IOUT(5),FILES(5),NSTAT,ITALK,*9999)
C
C5B----Create files containing Arc/Info item attribute files
C
         LAYIBD=IRWCL
C
         DO 500 IROW=1,NROW
            DO 510 ICOL=1,NCOL
               CHDIBD=0.0
               FRFIBD=0.0
               FFFIBD=0.0
               FLFIBD=0.0
               IBDFLG=0
               ICLFLG=0
               NUMLAY=0
               DO 520 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.EQ.LAYIBD) THEN
                   NCELLS=NCELLS+1
                   NUMLAY=NUMLAY+1
                   IBDFLG=1
                   CHDIBD=CHDIBD+CHDTRM(ICOL,IROW,ILAY)
                   FRFIBD=FRFIBD+FRFTRM(ICOL,IROW,ILAY)
                   FFFIBD=FFFIBD+FFFTRM(ICOL,IROW,ILAY)
                 IF(NUMLAY.EQ.ILAY) THEN
                    FLFIBD=FLFTRM(ICOL,IROW,ILAY)
                 ELSE
                    FLFIBD=FLFIBD+FLFTRM(ICOL,IROW,ILAY)
                 ENDIF
                ENDIF
                IF(IBDCEL.NE.LAYIBD .AND. IBDCEL.NE.0) THEN
                   ICLFLG=IBDCEL
                ENDIF
  520          CONTINUE
C
               IF(IBDFLG.GT.0) THEN
                  NCELLS=NCELLS+1
                  SEQNUM=(IROW-1)*NCOL+ICOL
                  NUMCLL=(IROW-1)*NCOL+ICOL
                  ROWCOL=IROW*(10**LENCOL)+ICOL
                  WRITE(CHRWCL,'(I10)') ROWCOL
                  FINCHR=10
                  STRCHR=FINCHR-(LENROW+LENCOL-1)
                  WRITE(IOUT(5),'(
     &                         I10,'','',
     &                         A,'','',
     &                         3(I10,'',''),
     &                         4(F16.3,'',''))')
     &                   SEQNUM,
     &                   CHRWCL (STRCHR:FINCHR),
     &                   IROW,
     &                   ICOL,
     &                   LAYIBD,
     &                   CHDIBD,
     &                   FRFIBD,
     &                   FFFIBD,
     &                   FLFIBD
                  WRITE(OUTUNT,'(10X,4I10,4F17.6)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          CHDIBD,
     &                          FRFIBD,
     &                          FFFIBD,
     &                          FLFIBD
               ELSE IF(IBDFLG.EQ.0 .AND. ICLFLG.NE.0) THEN
                  ILAY=-999
                  IBDCEL=-999
                  BADCEL=BADCEL+1
                  WRITE(OUTUNT,'(10X,4I10,20X,
     &                             ''           IBOUND = '',I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          LAYIBD
               ELSE
                  BADIBD=BADIBD+1
                  CHRWCL='   NONE   '
                  WRITE(OUTUNT,'(10X,2I10,2A10,20X,
     &                           ''INACTIVE CELL                 '')')
     &                           IROW,
     &                           ICOL,
     &                           CHRWCL,
     &                           CHRWCL
               ENDIF
  510       CONTINUE
  500    CONTINUE
C
C6----Build mapview for uppermost active layer
C
      ELSE
         CALL OPEN_FILE (IOUT(5),FILES(5),NSTAT,ITALK,*9999)
C
C6B----Grid label attributes
C
         DO 600 IROW=1,NROW
            DO 610 ICOL=1,NCOL
               DO 620 ILAY=1,NLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.NE.0) THEN
                   NCELLS=NCELLS+1
                   SEQNUM=(IROW-1)*NCOL+ICOL
                   ROWCOL=IROW*(10**LENCOL)+ICOL
                   WRITE(CHRWCL,'(I10)') ROWCOL
                   FINCHR=10
                   STRCHR=FINCHR-(LENROW+LENCOL-1)
                   WRITE(IOUT(5),'(
     &                             I10,'','',
     &                             A,'','',
     &                             4(I10,'',''),
     &                             4(F16.3,'',''))')
     &                       SEQNUM,
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       ILAY,
     &                       IBDCEL,
     &                       CHDTRM(ICOL,IROW,ILAY),
     &                       FRFTRM(ICOL,IROW,ILAY),
     &                       FFFTRM(ICOL,IROW,ILAY),
     &                       FLFTRM(ICOL,IROW,ILAY)
                   GO TO 612
                ENDIF
  620          CONTINUE
  612          IF(IBDCEL.NE.0) THEN
                  WRITE(OUTUNT,'(10X,4I10,4F17.6)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          CHDTRM(ICOL,IROW,ILAY),
     &                          FRFTRM(ICOL,IROW,ILAY),
     &                          FFFTRM(ICOL,IROW,ILAY),
     &                          FLFTRM(ICOL,IROW,ILAY)
               ELSE
                  BADIBD=BADIBD+1
                  CHRWCL='   NONE   '
                  WRITE(OUTUNT,'(10X,2I10,A10,I10,20X,
     &                           ''INACTIVE CELL                 '')')
     &                           IROW,
     &                           ICOL,
     &                           CHRWCL,
     &                           IBDCEL
               ENDIF
  610      CONTINUE
  600    CONTINUE
      ENDIF
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(//)')
C
C7----Close-out attribute, label and arc coordinate files
C
      CALL CLOSE_FILE (FILES(5),*9999)
      IF(IVIEW.EQ.0 .OR. IVIEW.EQ.4) THEN
         CALL CLOSE_FILE (FILES(6),*9999)
         CALL CLOSE_FILE (FILES(7),*9999)
         CALL CLOSE_FILE (FILES(8),*9999)
      ENDIF
C
C8----Messages
C
C
C8A----Message to user
C
      WRITE(*,'(''      ..Cell-by-cell flow terms file '',A,
     &          '' consists of '')')
     &        CBCFIL (1:INDEX(CBCFIL,' ')-1)
      WRITE(*,'(''      ..'',I10,'' cells'')') NCELLS
      WRITE(*,'(''      ..'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      IF(IVIEW.EQ.5) 
     &   WRITE(*,'(''      ..'',I10,'' cells were '',
     &          ''ommited (wrong ibound)'')') BADCEL
      WRITE(*,'(''      ..'',I10,'' cells were '',
     &          ''in confining layer'')') NUMCON
C
C8B----Message to operational output file
C
      WRITE(OUTUNT,'(''      ..Cell-by-cell flow terms file '',A,
     &          '' consists of '')')
     &        CBCFIL (1:INDEX(CBCFIL,' ')-1)
      WRITE(OUTUNT,'(''      ..'',I10,'' cells'')') NCELLS
      WRITE(OUTUNT,'(''      ..'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      IF(IVIEW.EQ.5) 
     &   WRITE(OUTUNT,'(''      ..'',I10,'' cells were '',
     &          ''ommited (wrong ibound)'')') BADCEL
      WRITE(OUTUNT,'(''      ..'',I10,'' cells were '',
     &          ''in confining layer'')') NUMCON
C
C9----Arc Operations to coverage and attribute files
C
      NSTAT=2
      ITALK=1
      CALL DELETE_FILE (FILES(MAXFIL),*9999)
      CALL OPEN_FILE (IOUT(MAXFIL),FILES(MAXFIL),NSTAT,ITALK,*9999)
C
      IF(IVIEW.EQ.1 .OR. IVIEW.EQ.2 .OR. IVIEW.EQ.3) THEN
         NSTAT=1
         ITALK=1
         TMPFIL='arccrs'
         CALL OPEN_FILE (IOUT(1),TMPFIL,NSTAT,ITALK,*9999)
C
 902     READ(IOUT(1),'(A)',END=904) BUFFER
         WRITE(IOUT(MAXFIL),'(A)') BUFFER
         GO TO 902
C
 904     CALL CLOSE_FILE (TMPFIL,*9999)
         CALL DELETE_FILE (TMPFIL,*9999)
      ENDIF
C
C9A----Create attribute files and "add from" attributes
C
      IF(IVIEW.EQ.0 .OR. IVIEW.EQ.4) THEN
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(5) (1:INDEX(FILES(5),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
         ARCCMD='tables'
         TMPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
         ADDFIL=FILES(6) (1:INDEX(FILES(6),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
         ARCCMD='tables'
         TMPFIL=FILES(3) (1:INDEX(FILES(3),' ')-1)
         ADDFIL=FILES(7) (1:INDEX(FILES(7),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
         ARCCMD='tables'
         TMPFIL=FILES(4) (1:INDEX(FILES(4),' ')-1)
         ADDFIL=FILES(8) (1:INDEX(FILES(8),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
      ELSE IF(IVIEW.EQ.6 .OR. IVIEW.EQ.5) THEN
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(5) (1:INDEX(FILES(5),' ')-1)
         WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
C
C9F---Join attribute file label to Coverage CBCFIL PAT file
C
      ELSE
         ARCCMD='tables'
         TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
         ADDFIL=FILES(5) (1:INDEX(FILES(5),' ')-1)
         ACTION='kill'
         WRITE(IOUT(MAXFIL),'(A,3(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
         ARCCMD='joinitem'
         LABFIL=CBCFIL (1:INDEX(CBCFIL,' ')-1)//'.pat'
         INPFIL=FILES(5) (1:INDEX(FILES(5),' ')-1)
         RELITM=CBCFIL (1:INDEX(CBCFIL,' ')-1)//'-id'
         STRITM='thick'
         JONTYP='linear'
         WRITE(IOUT(MAXFIL),'(A,6(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             INPFIL (1:INDEX(INPFIL,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ENDIF
      CALL CLOSE_FILE (FILES(9),*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &    '(//,'' Abnormal termination of Pthcbc_Subroutine'')')
      WRITE(OUTUNT,
     &    '(//,'' Abnormal termination of Pthcbc_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
