C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Disall version 6.1                **************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 DISALL 18MARCH2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR MODTOOLS OUTPUT LSTING FILE
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCBL    : NUMBER OF QUASI-3D LAYERS
C     IOUT    : FORTRAN UNIT NUMBER
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C             => 1 BEGINNING
C
C    Outputs:
C
C     LCCVPR  : STARTING ARRAY INDEX FOR COVERAGE PARAMETERS
C     LCLAYC  : STARTING ARRAY INDEX FOR LAYER TYPE CODES
C     LCCONF  : STARTING ARRAY INDEX FOR LAYER TYPE CODES
C     LCDELX  : STARTING ARRAY INDEX FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE
C                DIRECTION
C     LCZCON  : STARTING ARRAY INDEX FOR CONFINING LAYERS/UNITS
C     LCIBOU  : STARTING ARRAY INDEX FOR IBOUND ARRAY
C     LCRWCL  : STARTING ARRAY INDEX FOR THE CELL'S ROW OR COLUMN
C     LCZBOT  : STARTING ARRAY INDEX FOR CELL BOTTOM ELEVATIONS
C     LCZTOP  : STARTING ARRAY INDEX FOR CELL TOP ELEVATIONS
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-----------------------------------------------------------------------
      SUBROUTINE DISALL (
     I                   OUTUNT,
     I                   LENX,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,NCBL,NPER,
     B                   ISUM,
     O                   LCCVPR,LAYCON,LAYCBD,LCDELX,
     O                   LCIBOU,LCRWCL,LCZBOT,LCZTOP,LCZCON,
     E                   *)
C-------------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      REAL
     &     HNOFLO,HDRY
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        NLAY,NROW,NCOL,NCBL,NPER,
     &        ISUM,
     O        LCCVPR,LAYCON,LAYCBD,LCDELX,
     O        LCIBOU,LCRWCL,LCZBOT,LCZTOP,LCZCON
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(/)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          MODTOOLS GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''              VERSION 6.0 18MARCH2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(/)')
C
C1----Calculate storage parameters NRCL
C
      NRCL=NROW*NCOL*NLAY
C
C2----Allocate space for arrays
C
      LAYCON=ISUM
      ISUM=ISUM+NLAY
      LAYCBD=ISUM
      ISUM=ISUM+NLAY
      LCDELX=ISUM
      ISUM=ISUM+NCOL*NROW
      LCIBOU=ISUM
      ISUM=ISUM+NRCL
      LCRWCL=ISUM
      ISUM=ISUM+2*NCOL*NROW
      LCZBOT=ISUM
      ISUM=ISUM+NRCL
      LCZTOP=ISUM
      ISUM=ISUM+NRCL
      LCZCON=ISUM
      ISUM=ISUM+NRCL
      ISUM1=ISUM-1
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(/)')
      WRITE(OUTUNT,'(1H1,36X,
     &             ''MODTOOLS OPERATION PARAMETERS LIST'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(/)')
C
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY    '',
     &             10X,I15,'' LENX'')')
     &          LENX
      WRITE(OUTUNT,'(10X,
     &             ''ELEMENTS IN THE COMMON BLOCK ARRAY USED        '',
     &             10X,I15,'' ISUM'')')
     &          ISUM1
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF COVERAGE PARAMETERS          '',
     &             10X,I15,'' MAXPMR'')')
     &          MAXPMR
C
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF MODEL LAYERS                         '',
     &             10X,I15,'' NLAY'')')
     &          NLAY
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF MODEL ROWS                           '',
     &             10X,I15,'' NROW'')')
     &          NROW 
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF MODEL COLUMNS                        '',
     &             10X,I15,'' NCOL'')')
     &          NCOL
      WRITE(OUTUNT,'(10X,
     &             ''QUASI-3D CONFINING FLAG                        '',
     &             10X,I15,'' NCBL'')')
     &          NCBL
      IF(NPER.EQ.0) THEN
         WRITE(OUTUNT,'(10X,
     &             ''STEADY-STATE GROUND-WATER SIMULATION           '',
     &             10X,I15,'' NPER'')')
     &          NPER
      ELSE
         WRITE(OUTUNT,'(10X,
     &             ''TRANSIENT GROUND-WATER SIMULATION              '',
     &             10X,I15,'' NPER'')')
     &          NPER
      ENDIF
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(/)')
C
C4----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Disall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Disall_Subroutine'')')
         RETURN 1
      ENDIF
C
C5----Print Modtools common block listing
C
      WRITE(OUTUNT,'(/)')
      WRITE(OUTUNT,'(1H1,36X,
     &             ''MODTOOLS COMMON BLOCK INDEX LISTING          '')')
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR COVERAGE PARAMETERS   '',
     &             10X,I15,'' LCCVPR'')')
     &          LCCVPR
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR LAYER TYPE CODES      '',
     &             10X,I15,'' LAYCON'')')
     &          LAYCON
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CONFINING LAYER CODES '',
     &             10X,I15,'' LAYCBD'')')
     &          LAYCBD
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CELL WIDTHS           '',
     &             10X,I15,'' LCDELX'')')
     &          LCDELX
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR IBOUND ARRAY          '',
     &             10X,I15,'' LCIBOU'')')
     &          LCIBOU
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CELL ROW AND COLUMN   '',
     &             10X,I15,'' LCRWCL'')')
     &          LCRWCL
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CELL BOTTOM ELEVATION '',
     &             10X,I15,'' LCZBOT'')')
     &          LCZBOT
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CELL TOP ELEVATION    '',
     &             10X,I15,'' LCZTOP'')')
     &          LCZTOP
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CONFINING BOTTOMS     '',
     &             10X,I15,'' LCZCON'')')
     &          LCZCON
      WRITE(OUTUNT,'(10X,
     &             ''TOTAL NUMBER OF ELEMENTS USED IN COMMON BLOCK  '',
     &             10X,I15,'' ISUM1'')')
     &          ISUM1
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(/)')
C
C6----Return
C
      RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Dispre version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 DISPRE 26MARCH2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads the discretization information file for model problem information
C      and array information.  The arrays needed by Modtools are the same 
C      ones input to MODFLOW 2000:
C       NCOL,NROW, etc (Data item 1), 
C       LAYCBD line (Data item 2), 
C       DELR or DELC (Data item 3 and 4), 
C       TOP for layer 1 (Data item 5)
C       BOT (Data item 6)
C       BOT for confining layers if present (Data item 6)
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT FILENAME MODTOOLS.LIST
C     FILOUT  : FILENAME OF THE MODTOOLS.LIST
C     DISFIL  : FILE NAME OF MODFLOW 2000 DISCRETIZATION MODEL DATA
C     DISUNT  : FORTRAN UNIT NUMBER FOR INPUT DATA FILE
C
C    Outputs:
C
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NPER    : NUMBER OF MODFLOW STRESS PERIODS
C     NCBL    : NUMBER OF QUASI-3D LAYERS
C     ITMUNI  : TIME UNITS GROUND-WATER SIMULATION
C     LENUNI  : LENGTH UNITS GROUND-WATER SIMULATION
C
C     Program:
C
C     VARNAM  : DUMMY CHARACTER STRING HOLDING VARIABLE NAME
C
C-------------------------------------------------------------------------
      SUBROUTINE DISPRE (
     I                   OUTFIL,OUTUNT,
     I                   DISFIL,DISUNT,
     O                   NLAY,NROW,NCOL,NPER,ITMUNI,LENUNI,NCBL,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      CHARACTER*(*) OUTFIL,
     &              DISFIL
C
      INTEGER OUTUNT,
     &        DISUNT,
     &        NLAY,
     &        NROW,
     &        NCOL,
     &        NCBL,
     &        NPER,
     &        ITMUNI,
     &        LENUNI
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*128 BUFFER
      CHARACTER*80 VARNAM
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                 Read model problem definition information
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(/)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          MODTOOLS GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''              VERSION 6.1 03JUNE2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''         DISCRETIZATION PREPARATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(/)')
C
C0---Set 
C
      NPER=0
C
C0---Open MODPATH main file
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (DISUNT,DISFIL,NSTAT,ITALK,*9998)
C
C1----Read model problem information
C
      VARNAM='MODFLOW_DISCRETIZATION_DATA'
 102   READ(DISUNT,'(A)',ERR=9990,END=9991) 
     &                                     BUFFER
      IF(BUFFER (1:1) .EQ.'#') GO TO 102
C
      READ(BUFFER,*,ERR=9990,END=9991) 
     &                                               NLAY,
     &                                               NROW,
     &                                               NCOL,
     &                                               NPER,
     &                                               ITMUNI,
     &                                               LENUNI
C
C2----Read model problem information
C
      VARNAM='MODFLOW_LAYCBD_DATA'
       NCBL=0
        NUMLAY=0
 104    READ(DISUNT,'(A)',ERR=9990,END=9991) 
     &                                    BUFFER
      IF(BUFFER (1:1) .EQ.'#') GO TO 104
        NUMSTR=0
 106     NUMSTR=NUMSTR+1
       IF(BUFFER (NUMSTR:NUMSTR).EQ.' ') GO TO 106
        NUMLAY=NUMLAY+1
         NUMFIN=NUMSTR
C
 108     NUMFIN=NUMFIN+1
      IF(BUFFER (NUMFIN:NUMFIN).NE.' ') GO TO 108
C
      READ(BUFFER (NUMSTR:NUMFIN),*,ERR=9990,END=104) 
     &                                                LAYCBD
      NUMSTR=NUMFIN
      IF(LAYCBD.NE.0) NCBL=NCBL+1
C
      IF(NUMLAY.LT.NLAY) GO TO 106
C
      CALL CLOSE_FILE (DISFIL,*9998)
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(/)')
      WRITE(OUTUNT,'(1H1,36X,
     &             ''MODEL DIMENSION AND PARAMETER LIST'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF MODEL LAYERS                         '',
     &             10X,I15,'' NLAY'')')
     &          NLAY
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF MODEL ROWS                           '',
     &             10X,I15,'' NROW'')')
     &          NROW 
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF MODEL COLUMNS                        '',
     &             10X,I15,'' NCOL'')')
     &          NCOL
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF STRESS PERIODS                       '',
     &             10X,I15,'' NPER'')')
     &          NPER
      IF(NPER.EQ.1) THEN
         WRITE(OUTUNT,'(10X,
     &             ''STEADY-STATE GROUND-WATER SIMULATION           '',
     &             10X,I15,'' NPER'')')
     &          NPER
      ELSE
         WRITE(OUTUNT,'(10X,
     &             ''TRANSIENT GROUND-WATER SIMULATION              '',
     &             10X,I15,'' NPER'')')
     &          NPER
      ENDIF
C
      IF(ITMUNI.EQ.0) THEN
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' UNDEFINED'')')
     &          ITMUNI
      ELSE IF(ITMUNI.EQ.1) THEN
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' SECONDS'')')
     &          ITMUNI
      ELSE IF(ITMUNI.EQ.2) THEN
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' MINUTES'')')
     &          ITMUNI
      ELSE IF(ITMUNI.EQ.3) THEN
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' HOURS'')')
     &          ITMUNI
      ELSE IF(ITMUNI.EQ.4) THEN
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' DAYS'')')
     &          ITMUNI
      ELSE IF(ITMUNI.EQ.5) THEN
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' YEARS'')')
     &          ITMUNI
      ELSE
         WRITE(OUTUNT,'(10X,
     &             ''TIME UNITS GROUND-WATER SIMULATION             '',
     &             10X,I15,'' NONE SPECIFIED'')')
     &          ITMUNI
      END IF
C
      IF(LENUNI.EQ.0) THEN
         WRITE(OUTUNT,'(10X,
     &             ''LENGTH UNITS GROUND-WATER SIMULATION           '',
     &             10X,I15,'' UNDEFINED'')')
     &          LENUNI
      ELSE IF(LENUNI.EQ.1) THEN
         WRITE(OUTUNT,'(10X,
     &             ''LENGTH UNITS GROUND-WATER SIMULATION           '',
     &             10X,I15,'' FEET'')')
     &          LENUNI
      ELSE IF(LENUNI.EQ.2) THEN
         WRITE(OUTUNT,'(10X,
     &             ''LENGTH UNITS GROUND-WATER SIMULATION           '',
     &             10X,I15,'' METERS'')')
     &          LENUNI
      ELSE IF(LENUNI.EQ.3) THEN
         WRITE(OUTUNT,'(10X,
     &             ''LENGTH UNITS GROUND-WATER SIMULATION           '',
     &             10X,I15,'' CENTIMETERS'')')
     &          LENUNI
      ELSE
         WRITE(OUTUNT,'(10X,
     &             ''LENGTH UNITS GROUND-WATER SIMULATION           '',
     &             10X,I15,'' NONE SPECIFIED'')')
     &          LENUNI
      END IF
      WRITE(OUTUNT,'(10X,
     &             ''NUMBER OF QUASI-3D CONFINING LAYERS            '',
     &             10X,I15,'' NCBL'')')
     &          NCBL
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(/)')
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,
     &          '' Unable to read '',A,'' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read '',A,'' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,
     &          '' End-of-file; Unable to read '',A,
     &          '' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
      WRITE(*,'(//,
     &          '' End-of-file; Unable to read '',A,
     &          '' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
 9998 CALL CLOSE_FILE (DISFIL,*9999)
 9999 WRITE(*,'(//,'' Abnormal Termination of Dispre_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Disrp1 version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 DISRP1 07MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads the specifed MODFLOW 2000 discretization model cell information.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING DISRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     DISFIL  : FILE NAME OF MODFLOW 2000 DISCRETIZATION MODEL DATA
C     DISUNT  : FORTRAN UNIT NUMBER FOR INPUT DATA FILE
C     ICOVER  : FLAG (TYPE OF COVERAGE)
C     IVIEW   : FLAG (ORIENTATION OF COVERAGE)
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NCBL    : NUMBER OF QUASI-3D CONFINING LAYERS
C
C    Outputs:
C
C     LAYCBD  : ARRAY FOR CONFINING LAYERS TYPE
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION
C     ZTOP    : ARRAY FOR LAYER/UNIT TOP ELEVATIONS
C     ZBOT    : ARRAY FOR LAYER/UNIT BOTTOM ELEVATIONS
C     ZCON    : ARRAY FOR CONFINING LAYERS/UNITS ELEVATIONS
C
C
C-------------------------------------------------------------------------
      SUBROUTINE DISRP1 (
     I                   OUTFIL,OUTUNT,
     I                   DISFIL,DISUNT,
     I                   ICOVER,IVIEW,
     I                   NLAY,NROW,NCOL,NCBL,
     I                   MAXPMR,COVPMR,
     O                   LAYCBD,
     O                   DELX,
     O                   ZBOT,ZTOP,ZCON,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     &          COVPMR(MAXPMR),
     &          LAYCBD(NLAY),
     &          DELX(NCOL*NROW),
     &          ZTOP(NCOL*NROW*NLAY),
     &          ZBOT(NCOL*NROW*NLAY),
     &          ZCON(NCOL*NROW*NLAY)
C
      CHARACTER*(*) OUTFIL,DISFIL
C
      INTEGER DISUNT,OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*132 BUFFER
      CHARACTER*80 VARNAM
      CHARACTER*16 TEXT
      CHARACTER*24 ANAME(5)
C
      INTEGER HEDFLG
C
      LOGICAL BOUND,COLUMN,ELEV,ROW,THICK,GRID,FLUX,HEAD,BUDGET,POR
C
      DATA ANAME(1) /'                    DELR'/
      DATA ANAME(2) /'                    DELC'/
      DATA ANAME(3) /'    TOP LAYER ELEVATIONS'/
      DATA ANAME(4) /' BOTTOM LAYER ELEVATIONS'/
      DATA ANAME(5) /'  BOTTOM CONFINING LAYER'/
C
      DATA POR /.FALSE./
      DATA BOUND /.TRUE./
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                 Reading basic model problem information
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CL*********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''       MODTOOLS GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          VERSION 6.1 03JUNE2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''    DISCRETIZATION READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                       Discretization File Operations
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C1*********************************************************************C
      HNOFLO=COVPMR(15)
      HDRY=COVPMR(16)
C
CH----Open MODFLOW discretization file
C
      IF(DISUNT.LE.0) THEN
         WRITE(*,'(
     &             ''      .'',
     &             ''No MODFLOW discretization file specified Name '',
     &             ''file'')')
         WRITE(OUTUNT,'(
     &             ''      .'',
     &             ''No MODFLOW discretization file specified Name '',
     &             ''file'')')
         GO TO 9999
      ENDIF
C
C0----MODFLOW discretization file
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (DISUNT,DISFIL,NSTAT,ITALK,*9998)
C
C1----Read MODFLOW discretization file [Item 1]
C
      VARNAM='MODFLOW discretization ITEM 0'
 102   READ(DISUNT,'(A132)',ERR=9990,END=9991) BUFFER 
C
      IF(BUFFER (1:1) .EQ."#") GO TO 102
C
C2----Read MODFLOW confining layer type codes
C
      VARNAM='MODFLOW confining layer ITEM 2'
 202   READ(DISUNT,'(A132)',ERR=9990,END=9991) BUFFER 
C
      IF(BUFFER (1:1) .EQ."#") GO TO 202
        READ(BUFFER,*,ERR=9990,END=9991) (LAYCBD(ILAY), ILAY=1,NLAY)
        DO 200 ILAY=1,NLAY
           LAYCBD(ILAY)=-1*LAYCBD(ILAY)
  200   CONTINUE          
C
C3=---Read for row width (MODFLOW x-direction)
C
      WRITE(*,'(''      .Processing for cell widths along '',
     &          ''rows (x-direction)'')')
      CALL U1DREL (DELX(1),ANAME(1),NCOL,DISUNT,OUTUNT)
C
C4----Read for column width (MODFLOW y-direction)
C
      WRITE(*,'(''      .Processing for cell widths along '',
     &          ''columns (y-direction)'')')
      CALL U1DREL (DELX(NCOL+1),ANAME(2),NROW,DISUNT,OUTUNT)
C
C5---Read Top and Bottom Elevations.
C
      WRITE(*,'(''      .Processing Elevations....'')')
      DO 500 K=1,NLAY
         KK=K
         LOC=1+(KK-1)*NCOL*NROW
C
C5A--Read Top Elevations for layer 1.
C
       IF(KK.EQ.1) THEN
          WRITE(*,'(''      ..top of layer '',I10)')
     &                                                           KK
          CALL U2DREL (ZTOP(LOC),ANAME(3),NROW,NCOL,KK,DISUNT,
     &                 OUTUNT)
       ENDIF
C
C5B--Read Bottom Elevations for layer KK.
C
         WRITE(*,'(''      ..bottom of layer '',I10)')
     &                                                            KK
         CALL U2DREL (ZBOT(LOC),ANAME(4),NROW,NCOL,KK,DISUNT,
     &                OUTUNT)
C
C5B--Read Bottom Elevations for confining layer KK.
C
       IF(LAYCBD(KK).EQ.1) THEN
          WRITE(*,'(''      ..bottom of confining layer '',I10)')
     &                                                            KK
          CALL U2DREL (ZCON(LOC),ANAME(5),NROW,NCOL,KK,DISUNT,
     &                 OUTUNT)
       ELSE
          DO 510 I=LOC,LOC+NCOL*NROW
             ZCON(I)=ZBOT(I)
 510      CONTINUE
       ENDIF
C
C5C--Load Top Elevations for layer KK-1.
C
       IF(KK.LE.NLAY) THEN
          KKK=KK
          WRITE(*,'(''      ..top of layer '',I10)')
     &                                                           KK
          LOCTOP=1+(KKK)*NCOL*NROW
          LOCBOT=1+(KK-1)*NCOL*NROW
          DO 520 I=1,NROW*NCOL
             ZTOP(LOCTOP)=ZCON(LOCBOT)
             LOCTOP=LOCTOP+1
             LOCBOT=LOCBOT+1
 520      CONTINUE
       ENDIF
  500 CONTINUE
C
C6----Close MODFLOW discretization file
C
      CALL CLOSE_FILE (DISFIL,*9998)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,'' Unable to read '',A,'' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
      WRITE(OUTUNT,'(//,'' Unable to read '',A,'' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,'' End-of-file; Unable to read '',A,
     &          '' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
      WRITE(OUTUNT,'(//,'' End-of-file; Unable to read '',A,
     &          '' from file '',A)') 
     &          VARNAM,DISFIL (1:INDEX(DISFIL,' ')-1)
 9998 CALL CLOSE_FILE (DISFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Disrp1_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Disrp1_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Disbas version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 DISBAS 18MARCH2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads the specifed MODFLOW 2000 discretization model cell information.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING DISBAS OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     BASFIL  : FILE NAME OF MODFLOW 2000 DISCRETIZATION MODEL DATA
C     BASUNT  : FORTRAN UNIT NUMBER FOR INPUT DATA FILE
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     COVPMR  : STARTING ARRAY INDEX FOR COVERAGE PARAMETERS
C
C    Outputs:
C
C     IBOUND  : STARTING ARRAY INDEX FOR IBOUND ARRAY
C
C
C-------------------------------------------------------------------------
      SUBROUTINE DISBAS (
     I                   OUTFIL,OUTUNT,
     I                   BASFIL,BASUNT,
     I                   NLAY,NROW,NCOL,
     I                   MAXPMR,COVPMR,
     O                   IBOUND,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     &          COVPMR(MAXPMR),
     &          IBOUND(NCOL*NROW*NLAY)
C
      CHARACTER*(*) OUTFIL,BASFIL
C
      INTEGER BASUNT,OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*132 BUFFER,OPTIONS
      CHARACTER*80 VARNAM
      CHARACTER*16 TEXT
      CHARACTER*24 ANAME
C
      DATA ANAME /'          BOUNDARY ARRAY'/
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                 Reading basic model problem information
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CL*********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''       MODTOOLS GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          VERSION 6.0 25MARCH2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''    BASIC PACKAGE READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                       Block-Centered Operations
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C1*********************************************************************C
C
C0----Open MODFLOW discretization file
C
      IF(BASUNT.LE.0) THEN
         WRITE(*,'(
     &             ''      .'',
     &             ''No MODFLOW Basic file specified Name '',
     &             ''file'')')
         WRITE(OUTUNT,'(
     &             ''      .'',
     &             ''No MODFLOW Basic file specified Name '',
     &             ''file'')')
         GO TO 9999
      ENDIF
C
      NSTAT=1
       ITALK=1
        CALL OPEN_FILE (BASUNT,BASFIL,NSTAT,ITALK,*9998)
         WRITE(*,'(
     &             ''      ..'',
     &             ''MODFLOW Basic file specified '',A,
     &             '' file'')')
     &                                 BASFIL (1:INDEX(BASFIL,' ')-1)         
C
C1----Read MODFLOW Basic file [Item 0]
C
      VARNAM='MODFLOW Basic Package'
 102   READ(BASUNT,'(A132)',ERR=9990,END=9991) BUFFER 
C
        WRITE(*,'('' Buffer '',A)') BUFFER
      IF(BUFFER (1:1) .EQ.'#') GO TO 102
C
C2----Read MODFLOW Basic file [Item 1]
C
      VARNAM='MODFLOW Options ITEM 1'
       READ(BUFFER,'(A132)',ERR=9990,END=9991) OPTIONS 
        WRITE(*,'('' Options '',A)') OPTIONS
C
C3----Read MODFLOW layer type codes
C
      VARNAM='MODFLOW Ibound ITEM 2'
      DO 300 ILAY=1,NLAY
         KK=ILAY
         LOC=1+(KK-1)*NCOL*NROW
         WRITE(*,'(''      ..ibound array for layer '',I10)')
     &                                                            KK
         CALL U2DINT (IBOUND(LOC),ANAME,NROW,NCOL,KK,BASUNT,
     &                OUTUNT)
  300   CONTINUE          
C
C2----Read MODFLOW No Flow [Item 3]
C
      VARNAM='MODFLOW Options ITEM 3'
       READ(BASUNT,'(A132)',ERR=9990,END=9991) BUFFER 
        READ(BUFFER,*,ERR=9990,END=9991) HNOFLO
         WRITE(*,'('' HNOFLO '',F20.6)') HNOFLO
          COVPMR(15)=HNOFLO
C
C6----Close MODFLOW discretization file
C
      CALL CLOSE_FILE (BASFIL,*9998)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,'' Unable to read '',A,'' from file '',A)') 
     &          VARNAM,BASFIL (1:INDEX(BASFIL,' ')-1)
      WRITE(OUTUNT,'(//,'' Unable to read '',A,'' from file '',A)') 
     &          VARNAM,BASFIL (1:INDEX(BASFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,'' End-of-file; Unable to read '',A,
     &          '' from file '',A)') 
     &          VARNAM,BASFIL (1:INDEX(BASFIL,' ')-1)
      WRITE(OUTUNT,'(//,'' End-of-file; Unable to read '',A,
     &          '' from file '',A)') 
     &          VARNAM,BASFIL (1:INDEX(BASFIL,' ')-1)
 9998 CALL CLOSE_FILE (BASFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Disbas_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Disbas_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
