C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Endall version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 ENDALL 15MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for starting and finishing
C      point coverages and attributes.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     MAXATB  : MAXIMUM NUMBER OF ATTRIBUTES IN PARTICLE ATTRIBUTE TABLE
C     MAXPTS  : MAXIMUM NUMBER OF PARTICLE POSITIONS
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     OUTUNT    : FORTRAN UNIT NUMBER
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCATRB  : STARTING ARRAY INDEX FOR PARTICLE ATTRIBUTE TABLE
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE ENDALL (
     I                   OUTUNT,
     I                   LENX,
     I                   MAXATB,MAXPTS,
     B                   ISUM,
     O                   LCATRB,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        MAXATB,
     &        MAXPTS,
     &        ISUM,
     &        LCATRB
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*9 UNLIMIT
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          ENDPOINT GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''              VERSION 6.1 15MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Allocate space for arrays
C
      LCATRB=ISUM
      ISUM=ISUM+MAXATB
      ISUM1=ISUM-1
C
C2----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          STARTING/ENDING POINT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          VERSION 6.1 15MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      UNLIMIT='UNLIMITED'
      WRITE(OUTUNT,'(10X,
     &             A9,1X,
     &             ''MAXIMUM NUMBER OF PARTICLES '',
     &             ''IN ENDPOINT COVERAGE'')')
     &          UNLIMIT
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''MAXIMUM NUMBER OF ATTRIBUTES FOR PARTICLE '',
     &             ''IN ENDPOINT COVERAGE'')')
     &          MAXATB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR POINT ATTRIBUTES '',
     &             10X,I15,'' LCATRB'')')
     &          LCATRB
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      ISP=MAXATB
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR STARTING/ENDING POINT PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I8)')
     &          ISUM1,LENX
C
C3----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Endall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Endall_Subroutine'')')
         RETURN 1
      ENDIF
C
C4----Return
C
      RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthend version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 PTHEND 15MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a point coverage from results of the particle
C      tracking program MODPATH using the endpoint file ENDPOINT.
C      Attributes (items) associated with each location/point are (these
C      can be selected by the user for different location symbols or other
C      user defined functions or actions) :
C        Attributes (items) associated with each particle are
C      (these can be selected by the user for different pathlines or other
C      user defined functions or actions) :
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     ENDFIL  : MODPATH OUTPUT ENDPOINT PARTICLE FILE NAME
C     ENDUNT  : FORTRAN UNIT NUMBER TO ENDPOINT PARTICLE FILE
C     PRTCOV  : USER DEFINED PARTICLE COVERAGE NAME
C     MAXATB  : MAXIMUM DIMENSION OF THE NUMBER OF PARTICLE ATTRIBUTES
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     MAXPTS  : MAXIMUM DIMENSION OF THE NUMBER OF LABEL (MAXARC*MAXPTS)
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR)
C     IBOUND  : ARRAY FOR IBOUND ARRAY
C
C    Outputs:
C
C     None
C
C-------------------------------------------------------------------------
C
      SUBROUTINE PTHEND (
     I                   OUTUNT,
     I                   ENDFIL,ENDUNT,
     I                   PRTCOV,
     I                   MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   DELX,COVPMR,
     I                   IBOUND,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C::::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::
C
      DIMENSION 
     &          COVPMR(MAXPMR),
     &          DELX(NCOL*NROW),
     &          IBOUND(NCOL,NROW,NLAY)
C
      CHARACTER*(*) PRTCOV,ENDFIL
C
      INTEGER 
     &        OUTUNT,
     &        ENDUNT
C
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER 
     &          (MAXFIL=6,
     &           MAXITM=33,
     &           MXITEM=2)
C
      CHARACTER*80 BUFFER
      CHARACTER*128 LABFIL
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL
      CHARACTER*16 ARCCMD,ACTION,STRITM,RELITM,JONTYP,
     &             COVTYP,ITEM,ITEMS(MAXITM)
      CHARACTER*1 TYPITM,TYPES(MAXITM)
C
      REAL
     &     XCOORD,YCOORD,TREF,
     &     STRXGC,STRXLC,FINXGC,FINXLC,STRXCN,FINXCN,
     &     STRYGC,STRYLC,FINYGC,FINYLC,STRYCN,FINYCN,
     &     STRZGC,STRZLC,FINZGC,FINZLC,STRZCN,FINZCN,
     &     XGCXVL,YGCYVL,ZGCZVL,PRTSPD,PRTDIS
C
      INTEGER DECIML,DECIMLS(MAXITM),
     &        IOUT(MAXFIL),ITMSTR(MXITEM),ITMFIN(MXITEM),
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM)
C
      INTEGER 
     &        BUFDEX,COMPCT,BINARY,
     &        IDPART,IPCODE,IDCODE,ROW,
     &        STRCOL,STRROW,STRLAY,STRIBD,STRNOD,
     &        FINCOL,FINROW,FINLAY,FINIBD,FINNOD
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA FILES /'startitems','finalitems','start','final',
     &            'points','arccmd'/
      DATA IOUT /90,91,92,93,94,95/
C
      DATA ITMSTR /1,14/
      DATA ITMFIN /13,33/
      DATA ITEMS /'PARTID','PINDEX',
     &            'STRROW','STRCOL','STRLAY','STRIBD',
     &            'STRXGC','STRXLC',
     &            'STRYGC','STRYLC',
     &            'STRZGC','STRZLC',
     &            'STRTIM',
     &            'FINROW','FINCOL','FINLAY','FINIBD',
     &            'FINXGC','FINXLC',
     &            'FINYGC','FINYLC',
     &            'FINZGC','FINZLC',
     &            'FINTIM',
     &            'FINXVL','FINYVL','FINZVL','FINSPD','FINDIS',
     &            'MODSTP','IPCODE','IDCODE','RESTIM'/
      DATA DECIMLS / 0, 5*0,  7*3,  4*0,  7*3,  4*6,  1*3,  3*0, 1*3/
      DATA TYPES   /'B',5*'I',7*'F',4*'I',7*'F',4*'F',1*'F',3*'I',1*'F'/
      DATA WIDTHS  / 4, 5*10, 7*8,  4*10, 7*8,  4*8,  1*8,  3*10, 1*8/
      DATA OUTPUTS / 5, 5*10, 7*16, 4*10, 7*16, 4*16, 1*16, 3*10, 1*16/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          ENDPOINT GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''              VERSION 6.1 15MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 GIS OUTPUT MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C0----Endpoint file
C
      IF(ENDUNT.LE.0) THEN    
         WRITE(*,'(
     &             ''      .No endpoint listed '',
     &             ''in MODPATH name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No endpoint file listed '',
     &             ''in MODPATH name file'')')
         GO TO 9999
      ELSE
         WRITE(*,'(
     &             ''      .Processing particles from Endpoint file '',
     &             A,
     &             '' on unit number'',I5)')
     &       ENDFIL (1:INDEX(ENDFIL,' ')-1),
     &       ENDUNT
         WRITE(OUTUNT,'(
     &             ''      .Processing particles from Endpoint file '',
     &             A,
     &             '' on unit number'',I5)')
     &       ENDFIL (1:INDEX(ENDFIL,' ')-1),
     &       ENDUNT
      ENDIF
C
C1----Set local variables
C
      MODFLG=INT(COVPMR(1))
      ICOVER=INT(COVPMR(2))
      XYSCALE=COVPMR(3)
      ZSCALE=COVPMR(4)
      ANGLE=COVPMR(5)
      XOGRID=COVPMR(6)
      YOGRID=COVPMR(7)
      IF(COVPMR(9).LE.0.0) COVPMR(9)=1.0
      HSCALE=12.0/COVPMR(9)
      IF(COVPMR(8).LE.0.0) COVPMR(8)=1.0
      VSCALE=HSCALE*COVPMR(8)
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
      MODFMT=INT(COVPMR(MAXPMR-4))
      MODFMT=MODFMT/100
      YMIN=COVPMR(MAXPMR)
C
C2----Transform from Modpath origin (lower left) to Modular Model
C      origin (upper left)
C
      YORIGN=0.0
C
C2A----Calculate upper left corner distance from DELC array (y-axis)
C
      DO 200 ROW=NCOL+1,NCOL+NROW
         YORIGN=YORIGN+DELX(ROW)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
      WIDTHS(5)=LENLAY
      OUTPUTS(5)=LENLAY
C
      WIDTHS(14)=LENROW
      OUTPUTS(14)=LENROW
      WIDTHS(15)=LENCOL
      OUTPUTS(15)=LENCOL
      WIDTHS(16)=LENLAY
      OUTPUTS(16)=LENLAY
C
C3----Delete and create files for attribute definitions, attribute value, and
C     coverage point data and tolerance/dangle values
C
      NSTAT=2
      ITALK=1
C
C3A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      DO 300 N=1,MAXFIL-1
         CALL DELETE_FILE (FILES(N),*9999)
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999)
  300 CONTINUE
C
C3B----Write attribute definitions
C
      DO 310 N=1,MXITEM
         DO 320 K=ITMSTR(N),ITMFIN(N)
            DECIML=DECIMLS(K)
            ITEM=ITEMS(K)
            OUTPUT=OUTPUTS(K)
            TYPITM=TYPES(K)
            WIDTH=WIDTHS(K)
            WRITE(IOUT(N),'(A'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                      ITEM (1:INDEX(ITEM,' ')-1),
     &                                       WIDTH,
     &                                       OUTPUT,
     &                                       TYPITM,
     &                                       DECIML
  320    CONTINUE
  310 CONTINUE
       CALL CLOSE_FILE (FILES(1),*9999)
       CALL CLOSE_FILE (FILES(2),*9999)
C
C4----Endpoint file
C
      NUMPRT=0
      NUMUNR=0
      NUMDRY=0
      NUMACT=0
      NUMDIS=0
      NUMZON=0
C
C4A----Determine particle format
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (ENDUNT,ENDFIL,NSTAT,ITALK,*9999)
      MODFMT=0
C
      READ(ENDUNT,'(A)',ERR=9990,END=9991) BUFFER
       MODPTH=INDEX(BUFFER,'Version 3.00')
        IF(MODPTH.LE.0) MODPTH=INDEX(BUFFER,'Version 4.')
         COMPCT=INDEX(BUFFER,'(COMPACT)')
          BUFDEX=INDEX(BUFFER,'TREF=')
           CALL CLOSE_FILE (ENDFIL,*9999)
C
      IF(COMPCT.GT.0) MODFMT=1
C
      IF(MODFMT.LT.1) THEN    
         WRITE(*,'(7X,
     &             ''..Using standard text format'')')
         WRITE(OUTUNT,'(7X,
     &             ''..Using standard text format'')')
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (ENDUNT,ENDFIL,NSTAT,ITALK,*9999)
      ELSEIF(MODFMT.EQ.1) THEN    
         WRITE(*,'(7X,
     &             '' Using compact text format'')')
         WRITE(OUTUNT,'(7X,
     &             '' Using compact text format'')')
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (ENDUNT,ENDFIL,NSTAT,ITALK,*9999)
      ELSE
         WRITE(*,'(7X,
     &             '' Using binary text format'')')
         WRITE(OUTUNT,'(7X,
     &             '' Using binary text format'')')
         NSTAT=-4
         ITALK=1
         CALL OPEN_FILE (-ABS(ENDUNT),ENDFIL,NSTAT,ITALK,*9999)
      ENDIF
C
C4----Read header line from endpoint file
C
      IF(MODFMT.LT.2) THEN    
         READ(ENDUNT,'(A)',ERR=9990,END=9991) BUFFER
         MODPTH=INDEX(BUFFER,'Version 3.')
       IF(MODPTH.LE.0) MODPTH=INDEX(BUFFER,'Version 4.')
         COMPCT=INDEX(BUFFER,'(COMPACT)')
         BUFDEX=INDEX(BUFFER,'TREF=')
       IF(BUFDEX.GT.0) THEN
          BUFFER=BUFFER (BUFDEX+5:LEN(BUFFER))
          BUFDEX=INDEX(BUFFER,')')
        IF(BUFDEX.LE.0) BUFDEX=LEN(BUFFER)
          READ(BUFFER (1:BUFDEX-1),*) TREF
       ELSE
          TREF=0.0
       ENDIF
      ELSE
         READ(ENDUNT,ERR=9990,END=9991) BUFFER,TREF
         MODPTH=INDEX(BUFFER,'Version 3.')
       IF(MODPTH.LE.0) MODPTH=INDEX(BUFFER,'Version 4.')
         BINARY=INDEX(BUFFER,'(BINARY)')
      ENDIF
C
C4C----Check format of endpoint file
C
      IF(MODFMT.EQ.0 .AND. MODPTH.LE.0) THEN    
         WRITE(*,'(''      .Format of particle records is '',
     &             ''not in MODPATH version 3.0 text format '')')
         WRITE(*,'(''      ..Using standard text format '',
     &             ''for MODPATH version 2.0 '')')
         WRITE(OUTUNT,'(''      .Format of particle records is '',
     &             ''not in MODPATH version 3.0 text format '')')
         WRITE(OUTUNT,'(''      ..Using standard text format '',
     &             ''for MODPATH version 2.0 '')')
         MODFMT=9
         CALL CLOSE_FILE (ENDFIL,*9999)
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (ENDUNT,ENDFIL,NSTAT,ITALK,*9999)
      ENDIF
C
      IF(MODFMT.EQ.1 .AND. COMPCT.LE.0) THEN    
         WRITE(*,'(''      .Format of particle records is '',
     &             ''not in compact format '')')
         GO TO 9999
      ENDIF
C
      IF(MODFMT.EQ.2 .AND. BINARY.LE.0) THEN    
         WRITE(*,'(''      .Format of particle records is '',
     &             ''not in binary format '')')
         GO TO 9999
      ENDIF
C
C4----Loop to read particles from endpoint file
C
  402 IF(MODFMT.LT.1) THEN    
         READ(ENDUNT,*,ERR=9992,END=502)
     &             FINIBD,FINCOL,FINROW,FINLAY,
     &             FINXGC,FINYGC,FINZGC,FINZLC,
     &             FINTIM,
     &             STRXGC,STRYGC,STRZLC,
     &             STRCOL,STRROW,STRLAY,STRIBD,
     &             MODSTP,IPCODE,RESTIM
      ELSE IF(MODFMT.EQ.1) THEN
         READ(ENDUNT,*,ERR=9992,END=502)
     &             FINIBD,FINNOD,
     &             FINXGC,FINYGC,FINZLC,
     &             FINTIM,
     &             STRXGC,STRYGC,STRZLC,
     &             STRNOD,STRIBD,
     &             MODSTP,IPCODE,RESTIM
         CALL ND2IJK (
     &                STRNOD,
     &                STRROW,
     &                STRCOL,
     &                STRLAY,
     &                NROW,
     &                NCOL
     &                )
         CALL ND2IJK (
     &                FINNOD,
     &                FINROW,
     &                FINCOL,
     &                FINLAY,
     &                NROW,
     &                NCOL
     &                )
      ELSE IF(MODFMT.EQ.2) THEN
         READ(ENDUNT,ERR=9992,END=502)
     &             FINIBD,FINNOD,
     &             FINXGC,FINYGC,FINZLC,
     &             FINTIM,
     &             STRXGC,STRYGC,STRZLC,
     &             STRNOD,STRIBD,
     &             MODSTP,IPCODE,RESTIM
         CALL ND2IJK (
     &                STRNOD,
     &                STRROW,
     &                STRCOL,
     &                STRLAY,
     &                NROW,
     &                NCOL
     &                )
         CALL ND2IJK (
     &                FINNOD,
     &                FINROW,
     &                FINCOL,
     &                FINLAY,
     &                NROW,
     &                NCOL
     &                )
      ELSE
         READ(ENDUNT,*,ERR=9992,END=502)
     &             FINIBD,FINCOL,FINROW,FINLAY,
     &             FINXGC,FINYGC,FINZGC,FINZLC,
     &             FINTIM,
     &             STRXGC,STRYGC,STRZLC,
     &             STRCOL,STRROW,STRLAY,STRIBD
         MODSTP=0
         IPCODE=0
         RESTIM=0.0
      ENDIF
C
C4Aa----Compute local x-,y-coordinates
C
      IF(IPCODE.LT.0) THEN
         NSTEP=0
         IDCODE=IPCODE
       IF(IDCODE.EQ.-1) NUMUNR=NUMUNR+1
       IF(IDCODE.EQ.-2) NUMDRY=NUMDRY+1
      ELSE
         NSTEP=IPCODE/10
         IDCODE=MOD(IPCODE,10)
       IF(IDCODE.EQ.0) NUMACT=NUMACT+1
       IF(IDCODE.EQ.1) NUMDIS=NUMDIS+1
       IF(IDCODE.EQ.2) NUMZON=NUMZON+1
      ENDIF
C
C4Aa----Compute local x-,y-coordinates
C
      CALL LOCAL_COORDINATES (
     I                        NCOL,NROW,
     I                        DELX,
     I                        STRCOL,STRROW,
     I                        STRXGC,STRYGC,YORIGN,
     O                        STRXLC,STRYLC)
      CALL LOCAL_COORDINATES (
     I                        NCOL,NROW,
     I                        DELX,
     I                        FINCOL,FINROW,
     I                        FINXGC,FINYGC,YORIGN,
     O                        FINXLC,FINYLC)
C
C4Aa----Compute arc length, transform and assign coordinates,
C--------and obtain particle-time
C
      STRXGC=STRXGC*XYSCALE
       FINXGC=FINXGC*XYSCALE
        STRYGC=(STRYGC-YORIGN)*XYSCALE
         FINYGC=(FINYGC-YORIGN)*XYSCALE
          STRZLC=STRZLC
           FINZGC=FINZGC*ZSCALE
         IF(IVIEW.LE.0) THEN
          IF(ANGLE.NE.0.0) THEN
             CALL ROTATE (
     I                    ANGLE,
     B                    STRXGC,STRYGC)
             CALL ROTATE (
     I                    ANGLE,
     B                    FINXGC,FINYGC)
          ENDIF
            STRXGC=STRXGC+XOGRID
            FINXGC=FINXGC+XOGRID
            STRYGC=STRYGC+YOGRID
            FINYGC=FINYGC+YOGRID
         ELSE IF(IVIEW.EQ.1) THEN
            STRXCN=STRXGC*HSCALE
            FINXCN=FINXGC*HSCALE
            STRYCN=STRZLC*VSCALE
            FINYCN=FINZGC*VSCALE
         ELSE
            STRXCN=-1.0*STRYGC*HSCALE
            FINXCN=-1.0*FINYGC*HSCALE
            STRYCN=STRZLC*VSCALE
            FINYCN=FINZGC*VSCALE
         ENDIF
C
C4Ab---Fix for negative times (Modpath makes intermittent
C-------times negative)
C
      FINTIM=ABS(FINTIM)
C
C4B----Set label information for attribute table
C
      NUMPRT=NUMPRT+1
       IDLAB=NUMPRT
C
C4C----Test for final or start locations
C
       IF(MODFLG.LT.2) THEN
          XCOORD=STRXGC
          YCOORD=STRYGC
       ELSE
          XCOORD=FINXGC
          YCOORD=FINYGC
       ENDIF
C
C4E----Write label attributes for LAB file
C
        TRVTIM=FINTIM-RESTIM
      IF(TRVTIM.GT.0.0) THEN
         XGCXVL=(FINXGC-STRXGC)/TRVTIM
         YGCYVL=(FINYGC-STRYGC)/TRVTIM
         ZGCZVL=((FLOAT(STRLAY)-STRZGC)-
     &           (FLOAT(FINLAY)-FINZLC))/TRVTIM
         PRTSPD=(XGCXVL**2.0+YGCYVL**2.0+ZGCZVL**2.0)**0.5
         PRTDIS=((FINXGC-STRXGC)**2.0+
     &           (FINYGC-STRYGC)**2.0+
     &           (FINZGC-STRZGC)**2.0)**0.5
       IF(ABS(XGCVEL).LE.0.00000001) XGCVEL=SIGN(0.00000001,XGCVEL)
       IF(ABS(YGCVEL).LE.0.00000001) YGCVEL=SIGN(0.00000001,YGCVEL)
       IF(ABS(ZGCVEL).LE.0.00000001) ZGCVEL=SIGN(0.00000001,ZGCVEL)
       IF(PRTSPD.LE.0.00000001) PRTSPD=0.00000001
       IF(PRTDIS.LE.0.00000001) PRTDIS=0.00000001
      ELSE
         XGCVEL=0.0
         YGCVEL=0.0
         ZGCVEL=0.0
         PRTSPD=0.0
         PRTDIS=0.0
      ENDIF
C
C4F----Write label attributes for LAB file
C
         IDPART=NUMPRT
C
C4Fa----Write start attributes
C
         WRITE(IOUT(3),'(6(I10,'',''),
     &                   6(G16.6,'',''),
     &                   1(G16.3,'',''))')
     &                                 IDPART,
     &                                 IDLAB,
     &                                 STRROW,
     &                                 STRCOL,
     &                                 STRLAY,
     &                                 STRIBD,
     &                                 STRXGC,
     &                                 STRXLC,
     &                                 STRYGC,
     &                                 STRYLC,
     &                                 STRZGC,
     &                                 STRZLC,
     &                                 STRTIM
C
C4Fb----Write final attributes
C
         WRITE(IOUT(4),'(4(I5,'',''),
     &                   7(G16.6,'',''),
     &                   4(G16.8,'',''),
     &                   1(G16.3,'',''),
     &                   3(I10,'',''),
     &                   1(G16.3,'',''))')
     &                                 FINROW,
     &                                 FINCOL,
     &                                 FINLAY,
     &                                 FINIBD,
     &                                 FINXGC,
     &                                 FINXLC,
     &                                 FINYGC,
     &                                 FINYLC,
     &                                 FINZGC,
     &                                 FINZLC,
     &                                 FINTIM,
     &                                 XGCXVL,
     &                                 YGCYVL,
     &                                 ZGCZVL,
     &                                 PRTSPD,
     &                                 PRTDIS,
     &                                 MODSTP,
     &                                 IPCODE,
     &                                 IDCODE,
     &                                 RESTIM
C
C4E----Write labels to LAB file
C
         WRITE(IOUT(5),'(I10,'','',F20.10,'','',F20.10)')
     &                IDLAB,
     &                XCOORD,
     &                YCOORD
C
C4F----Read another label from ENDPOINT file
C
       GO TO 402
C
C5----Close ENDPOINT and attribute files
C
  502 CALL CLOSE_FILE (ENDFIL,*9999)
      CALL CLOSE_FILE (FILES(3),*9999)
      CALL CLOSE_FILE (FILES(4),*9999)
C
C6----Close generate label coordinate file
C
      WRITE(IOUT(5),'(''END'')')
      CALL CLOSE_FILE (FILES(5),*9999)
C
C7----Messages
C
C
C7A----Message to user
C
      WRITE(*,'(''       .Endpoint coverage '',A,
     &          '' consists of '')')
     &        PRTCOV (1:INDEX(PRTCOV,' ')-1)
      WRITE(*,'(''       .'',I10,'' particles'')')
     &       NUMPRT
      WRITE(*,'(''       .'',I10,'' particles '',
     &          ''unreleased'')')
     &       NUMUNR
      WRITE(*,'(''       .'',I10,'' particles '',
     &          ''stranded in active [dry] cells'')')
     &       NUMDRY
      WRITE(*,'(''       .'',I10,'' particles '',
     &          ''remaining active'')')
     &       NUMACT
      WRITE(*,'(''       .'',I10,'' particles '',
     &          ''discharged'')')
     &       NUMDIS
      WRITE(*,'(''       .'',I10,'' particles '',
     &          ''stopped in a specified zone'')')
     &       NUMZON
      WRITE(*,'(''       .'',G16.7,
     &          '' simulation reference time '')')
     &       TREF
C
C7B----Message to operational output file
C
      WRITE(OUTUNT,'(//,
     &               ''       .Endpoint coverage '',
     &               A,'' consists of '')')
     &            PRTCOV (1:INDEX(PRTCOV,' ')-1)
      WRITE(OUTUNT,'(''       .'',I10,'' particles'')')
     &       NUMPRT
      WRITE(OUTUNT,'(''       .'',I10,'' particles '',
     &          ''unreleased'')')
     &       NUMUNR
      WRITE(OUTUNT,'(''       .'',I10,'' particles '',
     &          ''stranded in active [dry] cells'')')
     &       NUMDRY
      WRITE(OUTUNT,'(''       .'',I10,'' particles '',
     &          ''remaining active'')')
     &       NUMACT
      WRITE(OUTUNT,'(''       .'',I10,'' particles '',
     &          ''discharged'')')
     &       NUMDIS
      WRITE(OUTUNT,'(''       .'',I10,'' particles '',
     &          ''stopped in a specified zone'')')
     &       NUMZON
      WRITE(OUTUNT,'(''       .'',G16.7,
     &          '' simulation reference time '')')
     &       TREF
C
C8----Arc Operations to coverage and attribute files
C
      IF(NUMPRT.LE.0) GO TO 1000 
        NSTAT=2
        ITALK=1
        CALL DELETE_FILE (FILES(MAXFIL),*9999)
        CALL OPEN_FILE (IOUT(MAXFIL),FILES(MAXFIL),NSTAT,ITALK,*9999)
C
C8----Arc Operations to coverage and attribute files
C
      ARCCMD='create'
      WRITE(IOUT(MAXFIL),'(A,1X,A)') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             PRTCOV (1:INDEX(PRTCOV,' ')-1)
      ARCCMD='generate'
      COVTYP='point'
      INPFIL=FILES(5) (1:INDEX(FILES(5),' ')-1)
      WRITE(IOUT(MAXFIL),'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             PRTCOV (1:INDEX(PRTCOV,' ')-1),
     &                             INPFIL (1:INDEX(INPFIL,' ')-1),
     &                             COVTYP (1:INDEX(COVTYP,' ')-1)
      ARCCMD='build'
      COVTYP='point'
      WRITE(IOUT(MAXFIL),'(A,2(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             PRTCOV (1:INDEX(PRTCOV,' ')-1),
     &                             COVTYP (1:INDEX(COVTYP,' ')-1)
      ARCCMD='tables'
      TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
      ADDFIL=FILES(3) (1:INDEX(FILES(3),' ')-1)
      ACTION='kill'
      WRITE(IOUT(MAXFIL),'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
      ARCCMD='tables'
      TMPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
      ADDFIL=FILES(4) (1:INDEX(FILES(4),' ')-1)
      WRITE(IOUT(MAXFIL),'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
      ARCCMD='joinitem'
      LABFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      INPFIL=FILES(4) (1:INDEX(FILES(4),' ')-1)
      RELITM='$recno'
      STRITM=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'-ID'
      JONTYP='link'      
      WRITE(IOUT(MAXFIL),'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             INPFIL (1:INDEX(INPFIL,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ARCCMD='joinitem'
      LABFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      INPFIL=FILES(3) (1:INDEX(FILES(3),' ')-1)
      RELITM='$recno'
      STRITM=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'-ID'
      JONTYP='link'      
      WRITE(IOUT(MAXFIL),'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             INPFIL (1:INDEX(INPFIL,' ')-1),
     &                             LABFIL (1:INDEX(LABFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      CALL CLOSE_FILE (FILES(MAXFIL),*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,'' Unable to read first line from Endpoint file'')')
      CALL CLOSE_FILE (ENDFIL,*9999)
      GO TO 9999
 9991 WRITE(*,'(//,'' Premature end-of-file error in Endpoint file'')')
      CALL CLOSE_FILE (ENDFIL,*9999)
      GO TO 9999
 9992 WRITE(*,'(//,'' Wrong format in Endpoint file'')')
      CALL CLOSE_FILE (ENDFIL,*9999)
 9999 WRITE(*,'(//,'' Abnormal termination of Endpoint_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
