C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthcrs version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event 
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270 
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470 
C*   10615 SE. Cherry Blossom Drive 
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov 
C* 
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C* 
C* Disclaimer:  
C           Although this program has been used by the U.S. Geological Survey, 
C            no warranty, expressed or implied, is made by the USGS as to the 
C            accuracy and functioning of the program and related program 
C            material nor shall the fact of distribution constitute any such 
C            warranty, and no responsibility is assumed by the USGS in 
C            connection therewith. 
C* 
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C 
C 
C---VERSION 6.0 PTHCRS 26MARCH2002
C 
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: 
C 
C .. Purpose: 
C 
C     Generates a cross-sectional view coverage (polygon topology) along 
C      a row or a column or a single particle from results of the particle 
C      tracking program MODPATH (Pollock, 1989).  Attributes (items)  
C      associated with each cell are (these can be selected by the user to 
C      shade each cell or other user defined functions or actions) : 
C      POLYGON ATTRIBUTE ARRAY: 
C              => (n,1) CELL POLYGON NUMBER 
C              => (n,2) CELL ROW 
C              => (n,3) CELL COLUMN 
C              => (n,4) CELL LAYER 
C              => (n,5) MODEL IBOUND VALUE 
C              => (n,6) CELL TOP ELEVATION 
C              => (n,7) CELL BOTTOM ELEVATION 
C              => (n,8) CELL THICKNESS 
C 
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: 
C 
C .. Language: FORTRAN 
C 
C 
C    Inputs: 
C 
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED 
C     XSCOV   : COVERAGE ROOT NAME 
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS 
C     NLAY    : NUMBER OF LAYERS IN MODEL 
C     NROW    : NUMBER OF ROWS IN GRID 
C     NCOL    : NUMBER OF COLUMNS IN GRID 
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION 
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR) 
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NCOL*NROW) 
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY) 
C     ZTOP    : ARRAY FOR LAYER/UNIT TOP ELEVATIONS 
C     ZBOT    : ARRAY FOR LAYER/UNIT BOTTOM ELEVATIONS 
C     ZCON    : ARRAY FOR CONFINING LAYER THICKNESS 
C 
C    Outputs: None 
C 
C----------------------------------------------------------------------- 
      SUBROUTINE PTHCRS ( 
     I                   OUTUNT, 
     I                   XSCOV, 
     I                   MAXPMR, 
     I                   NLAY,NROW,NCOL, 
     I                   DELX,COVPMR, 
     I                   LRWCL,IBOUND,LAYCBD, 
     I                   ZTOP,ZBOT,ZCON, 
     E                   *) 
C----------------------------------------------------------------------- 
C 
C        SPECIFICATIONS: 
C     ------------------------------------------------------------------ 
C 
C:::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::: 
C 
      DIMENSION  
     &          COVPMR(MAXPMR), 
     &          LAYCBD(NLAY), 
     &          DELX(NCOL*NROW), 
     &          LRWCL(2,NCOL*NROW), 
     &          IBOUND(NCOL,NROW,NLAY), 
     &          ZTOP(NCOL,NROW,NLAY), 
     &          ZBOT(NCOL,NROW,NLAY),
     &          ZCON(NCOL,NROW,NLAY) 
C 
      CHARACTER*(*) XSCOV 
C 
      INTEGER OUTUNT 
C 
C::::::::::::::::::::::::  PROGRAM_VARIABLES  :::::::::::::::::::::::::: 
C 
      PARAMETER (MAXFIL=4) 
      PARAMETER (MAXITM=8) 
C 
      CHARACTER*128 TMPCOV,LABFIL 
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL 
      CHARACTER*16 ARCCMD,ACTION,STRITM,RELITM,JONTYP, 
     &             COVTYP,ITEM,ITEMS(MAXITM) 
      CHARACTER*1 TYPITM,TYPES(MAXITM) 
C 
      INTEGER  
     &        IVIEW,IRWCL, 
     &        DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN, 
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM), 
     &        ROW,COLUMN,CELL,BADARC,BADIBD,BADTOP,NUMDRY,NUMFLW,COVDEX, 
     &        LAYER, 
     &        ICOL,IROW,ILAY, 
     &        STRCLL,FINCLL 
C 
      LOGICAL CLLFLG 
C 
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE 
C 
      DATA FILES /'cellitems','cellitem','arcscrs','arccrs'/ 
      DATA IOUT /90,91,92,93/ 
      DATA ITEMS /'CELL','ROW','COLUMN','LAYER','IBOUND', 
     &            'TOPELEV','BOTELEV','THICK'/ 
      DATA DECIMLS /5*0,3*3/ 
      DATA TYPES /'B',4*'I',3*'F'/ 
      DATA WIDTHS /4,4*10,3*8/ 
      DATA OUTPUTS /5,4*10,3*16/ 
C 
C     ------------------------------------------------------------------ 
C 
CH----Print Modtools header 
C 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''          MODEL GRID GIS OUTPUT PACKAGE'')') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''               VERSION 6.0 26MARCH2002 '')') 
      WRITE(OUTUNT,'(10X,85(''*''))') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''              TRUE CROSS-SECTION MODULE '')') 
      WRITE(OUTUNT,'(10X,85(''=''))') 
      WRITE(OUTUNT,'(//)') 
C 
C1----Set local variables 
C 
      ITMSTR=1 
      ITMFIN=8 
      NARCS=0 
      NLABS=0 
      TMPCOV='tmp' 
      ZSCALE=COVPMR(4) 
      IF(COVPMR(9).LE.0.0) COVPMR(9)=1.0 
      HSCALE=12.0/COVPMR(9) 
      IF(COVPMR(8).LE.0.0) COVPMR(8)=1.0 
      VSCALE=HSCALE*COVPMR(8) 
      IVIEW=INT(COVPMR(10)) 
      IRWCL=INT(COVPMR(11)) 
      HNOFLO=COVPMR(16) 
C 
C1B---Set local variables 
C 
      IF(IVIEW.LE.1) THEN 
         WRITE(*,'(''       .Processing vertical profile Information '', 
     &             ''along row '',I10)') 
     &          IRWCL 
      ELSE IF(IVIEW.EQ.2) THEN 
         WRITE(*,'(''       .Processing vertical profile Information '', 
     &             ''along column '',I10)') 
     &          IRWCL 
      ELSE 
         WRITE(*,'(''       .Processing vertical profile Information '', 
     &             ''along particle '',I10)') 
     &          IRWCL 
      ENDIF 
C 
C2----Delete and create files for attribute definitions, attribute value, and 
C     coverage arc data and tolerance/dangle values 
C 
      NSTAT=2 
      ITALK=1 
C 
C2A----Delete/create files containing Arc/Info item definitions for 
C      attribute files 
C 
      DO 200 N=1,MAXFIL 
         CALL DELETE_FILE (FILES(N),*9999) 
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999) 
  200 CONTINUE 
C 
C2B----Calculate rowcol attribute definition 
C 
      IF(NROW.LT.10) THEN 
         LENROW=1 
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN 
         LENROW=2 
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN 
         LENROW=3 
      ELSE 
         LENROW=4 
      ENDIF 
C 
      IF(NCOL.LT.10) THEN 
         LENCOL=1 
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN 
         LENCOL=2 
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN 
         LENCOL=3 
      ELSE 
         LENCOL=4 
      ENDIF 
C 
      IF(NLAY.LT.10) THEN 
         LENLAY=1 
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN 
         LENLAY=2 
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN 
         LENLAY=3 
      ELSE 
         LENLAY=4 
      ENDIF 
C 
      WIDTHS(2)=LENROW 
      OUTPUTS(2)=LENROW 
      WIDTHS(3)=LENCOL 
      OUTPUTS(3)=LENCOL 
      WIDTHS(4)=LENLAY+1 
      OUTPUTS(4)=LENLAY+1 
C 
C2C----Write attribute definitions 
C 
      ITEMS(1)=XSCOV (1:INDEX(XSCOV,' ')-1)//'-id' 
      DO 210 K=ITMSTR,ITMFIN 
         DECIML=DECIMLS(K) 
         ITEM=ITEMS(K) 
         OUTPUT=OUTPUTS(K) 
         TYPITM=TYPES(K) 
         WIDTH=WIDTHS(K) 
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','', 
     &                   I2,'',,,,,,,,,,'')') 
     &                                    ITEM, 
     &                                    WIDTH, 
     &                                    OUTPUT, 
     &                                    TYPITM, 
     &                                    DECIML 
  210 CONTINUE 
       CALL CLOSE_FILE (FILES(1),*9999) 
C 
C3----Build arcs from elevation and row/column data 
C 
      WRITE(*,'(''      .Processing coordinates for '', 
     &             ''Vertical Cell profile now...'')') 
C 
C3A----Set cell min/max and x-direction sum variables 
C 
      BADARC=0 
      BADIBD=0 
      BADTOP=0 
      NUMFLW=0 
      CLLMAX=-9999999999.99 
      CLLMIN=-CLLMAX 
      TOPMAX=-9999999999.99 
      BOTMIN=-CLLMAX 
      NUMCLL=0 
      NUMCON=0 
      NCELLS=0 
      WRITE(OUTUNT,'(//,1H1,15X, 
     &               '' CELLS WITH INVALID VALUES '', 
     &               ''WILL BE OMITTED FROM '', 
     &               ''CROSS-SECTION COVERAGE'', 
     &               /10X,85(''-''))') 
      WRITE(OUTUNT,'(40X, 
     &               ''       ROW'', 
     &               ''    COLUMN'', 
     &               ''     LAYER'', 
     &               ''    IBOUND'', 
     &               /10X,85(''-''))') 
C 
C3B----Set Loop index numcll 
C 
      IF(IVIEW.EQ.1) THEN 
         STRCLL=1 
         FINCLL=NCOL 
      ELSE IF(IVIEW.EQ.2) THEN 
         STRCLL=NCOL+1 
         FINCLL=NCOL+NROW 
      ELSE 
         STRCLL=1 
         FINCLL=IRWCL 
      ENDIF 
C 
C3C----Loop to find minimum and maximum elevations 
C 
      DO 300 N=STRCLL,FINCLL 
         NUMCLL=NUMCLL+1 
       IF(IVIEW.LE.1) THEN 
          IROW=IRWCL 
          ICOL=NUMCLL 
       ELSE IF(IVIEW.EQ.2) THEN 
          IROW=NUMCLL 
          ICOL=IRWCL 
       ELSE 
          ICOL=LRWCL(1,NUMCLL) 
          IROW=LRWCL(2,NUMCLL) 
       ENDIF 
         DO 310 ILAY=1,NLAY 
            IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY)) 
            IBDCON=ABS(LAYCBD(ILAY)) 
          IF(IBDCEL.NE.0) THEN 
             NCELLS=NCELLS+1 
             YTOP=ZTOP(ICOL,IROW,ILAY) 
             YBOT=ZBOT(ICOL,IROW,ILAY) 
           IF(YTOP.EQ.HNOFLO) THEN 
              NUMFLW=NUMFLW+1 
              IBOUND(ICOL,IROW,ILAY)=0 
              WRITE(OUTUNT,'(10X, 
     &                      ''NOFLOW CELL RESET IBOUND TO 0 '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
C 
           ELSE IF(YTOP.EQ.YBOT) THEN 
              BADTOP=BADTOP+1 
              IBOUND(ICOL,IROW,ILAY)=0 
              WRITE(OUTUNT,'(10X, 
     &                      ''CELL ELEVATION (TOP=BOTTOM)   '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
C 
           ELSE IF(YTOP.LT.YBOT) THEN 
              BADTOP=BADTOP+1 
              IBOUND(ICOL,IROW,ILAY)=0 
              WRITE(OUTUNT,'(10X, 
     &                      ''CELL ELEVATION (TOP<BOTTOM)   '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
C 
           ELSE 
            IF(YTOP.GT.CLLMAX) CLLMAX=YTOP 
            IF(YBOT.LT.CLLMIN) CLLMIN=YBOT 
            IF(YTOP.GT.TOPMAX) TOPMAX=YTOP 
            IF(YBOT.LT.BOTMIN) BOTMIN=YBOT 
           ENDIF 
          ELSE 
             BADIBD=BADIBD+1 
             WRITE(OUTUNT,'(10X, 
     &                      ''INACTIVE CELL (IBOUND=0)      '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
          ENDIF 
          IF(IBDCON.NE.0) THEN 
             YTOP=ZBOT(ICOL,IROW,ILAY) 
             YBOT=ZCON(ICOL,IROW,ILAY)
            IF(YTOP.GT.CLLMAX) CLLMAX=YTOP 
            IF(YBOT.LT.CLLMIN) CLLMIN=YBOT 
            IF(YTOP.GT.TOPMAX) TOPMAX=YTOP 
            IF(YBOT.LT.BOTMIN) BOTMIN=YBOT 
          ENDIF
  310    CONTINUE 
  300 CONTINUE 
      COVPMR(MAXPMR)=CLLMIN*VSCALE 
      CLLMAX=CLLMAX*VSCALE 
C 
C4----Loop to build arcs from elevation and row/column data 
C 
      XLEFT=0.0 
      ISIDES=-99 
      IBOTTM=-9 
      NUMCLL=0 
      WRITE(OUTUNT,'(//,1H1,40X, 
     &               '' TRUE '', 
     &               ''CROSS-SECTION COVERAGE'', 
     &               /10X,85(''-''))') 
      WRITE(OUTUNT,'(40X, 
     &               ''       ROW'', 
     &               ''    COLUMN'', 
     &               ''     LAYER'', 
     &               ''         TOP'',1X, 
     &               ''      BOTTOM'')') 
      WRITE(OUTUNT,'(70X, 
     &               ''  ELEVATIONS'',1X, 
     &               ''  ELEVATIONS'')') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
C 
      DO 400 N=STRCLL,FINCLL 
         NUMCLL=NUMCLL+1 
         CLLFLG=.FALSE. 
         XWIDTH=DELX(N)*HSCALE 
         XRIGHT=XLEFT+XWIDTH 
         HWIDTH=XWIDTH/2.0 
C
       IF(IVIEW.LE.1) THEN 
          IROW=IRWCL 
          ICOL=NUMCLL 
       ELSE IF(IVIEW.EQ.2) THEN 
          IROW=NUMCLL 
          ICOL=IRWCL 
       ELSE 
          ICOL=LRWCL(1,NUMCLL) 
          IROW=LRWCL(2,NUMCLL) 
       ENDIF 
         YMAX=-99999999999.99 
         YMIN=-YMAX 
C 
C4A---Cell with no width 
C 
       IF(XLEFT.EQ.XRIGHT) THEN 
          BADARC=BADARC+1 
          WRITE(OUTUNT,'(10X, 
     &                   ''ZERO LENGTH TOP ARC           '', 
     &                   4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
          GO TO 400 
       ENDIF 
C 
C4B----Set correct row/column 
C 
         DO 410 ILAY=1,NLAY 
            IBDCEL=IBOUND(ICOL,IROW,ILAY) 
            IBDCON=ABS(LAYCBD(ILAY))
          IF(IBDCEL.EQ.0) THEN 
             WRITE(OUTUNT,'(10X, 
     &                      ''INACTIVE CELL (IBOUND=0)      '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
          ELSE 
             CLLFLG=.TRUE. 
             NARCS=NARCS+1 
             TOP=ZTOP(ICOL,IROW,ILAY) 
             BOT=ZBOT(ICOL,IROW,ILAY) 
             YTOP=(TOP-CLLMIN)*VSCALE 
             YBOT=(BOT-CLLMIN)*VSCALE 
           IF(YTOP.GT.YMAX) YMAX=YTOP 
           IF(YBOT.LT.YMIN) YMIN=YBOT 
C 
C4Ba---Cell with no thickness 
C 
           IF(YTOP.EQ.YBOT) THEN 
              WRITE(OUTUNT,'(10X, 
     &                       ''CELL WITH NO THICKNESS       '', 
     &                       3I10,2(F12.5,1X))') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               TOP, 
     &               BOT 
C 
C4Bb---Cell [active] 
C 
           ELSE 
              WRITE(OUTUNT,'(10X, 
     &                       ''ACTIVE CELL                   '', 
     &                       3I10,2(F12.5,1X))') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               TOP, 
     &               BOT 
C 
C4Bc---Cell label 
C 
              NLABS=NLABS+1 
              XLABEL=XLEFT+HWIDTH 
              YLABEL=YBOT+ABS(YTOP-YBOT)/2.0 
              WRITE(IOUT(3),'(I10,'','',F20.10,'','',F20.10)') 
     &                    NLABS, 
     &                    XLABEL, 
     &                    YLABEL 
C 
C4Bd---Cell walls 
C 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XLEFT, 
     &                                               YTOP 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XRIGHT, 
     &                                               YTOP 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XRIGHT, 
     &                                               YBOT 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XLEFT, 
     &                                               YBOT 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XLEFT, 
     &                                               YTOP 
              WRITE(IOUT(3),'(''END'')') 
C 
C4Ac---Write cell attributes 
C 
              THKLAY=TOP-BOT 
              WRITE(IOUT(2),'(5(I10,'',''), 
     &                        3(F16.3,'',''))') 
     &                    NLABS, 
     &                    IROW, 
     &                    ICOL, 
     &                    ILAY, 
     &                    IBDCEL, 
     &                    TOP, 
     &                    BOT, 
     &                    THKLAY 
            ENDIF
C
C4B---Confining Layer
C
            IF(IBDCON.NE.0) THEN 
               TOPCON=ZBOT(ICOL,IROW,ILAY)
               BOTCON=ZCON(ICOL,IROW,ILAY)
               NARCS=NARCS+1
               NUMCON=NUMCON+1
               YTOP=(TOPCON-CLLMIN)*VSCALE
               YBOT=(BOTCON-CLLMIN)*VSCALE
C
               WRITE(OUTUNT,'(10X,
     &                       ''CONFINING CELL                '',
     &                       3I10,2(F12.5,1X))')
     &                IROW,
     &                ICOL,
     &                ILAY,
     &                TOPCON,
     &                BOTCON
C
C4Bb---Write confining cell IBOUND
C
             IF(NLAY.LT.10) THEN
                IBDCON=90+ILAY
             ELSE IF(NLAY.GE.10 .AND. NLAY.LT.100) THEN
                IBDCON=900+ILAY
             ELSE
                IBDCON=9000+ILAY
             ENDIF
C
C4Bb---Write cell label for confining layer
C
               NLABS=NLABS+1
               XLABEL=XLEFT+HWIDTH
               YLABEL=YBOT+ABS(YTOP-YBOT)/2.0
               WRITE(IOUT(3),'(I10,'','',F20.10,'','',F20.10)')
     &                      NLABS,
     &                      XLABEL,
     &                      YLABEL
C
C4Ba---Write cell walls for confining layer
C
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XLEFT, 
     &                                                YTOP 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XRIGHT, 
     &                                                YTOP 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XRIGHT, 
     &                                                YBOT 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XLEFT, 
     &                                                YBOT 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XLEFT, 
     &                                                YTOP 
               WRITE(IOUT(3),'(''END'')') 
C
C4Bc---Write cell attributes for confining layer
C
               THKLAY=TOP-BOT
               WRITE(IOUT(2),'(5(I10,'',''),
     &                         3(F16.3,'',''))')
     &                     NLABS,
     &                     IROW,
     &                     ICOL,
     &                     IBDCON,
     &                     IBDCEL,
     &                     TOP,
     &                     BOT,
     &                     THKLAY
           ENDIF 
          ENDIF 
  410    CONTINUE 
         XLEFT=XRIGHT 
  400 CONTINUE 
C 
C5----Close-out arc coordinate file 
C 
      WRITE(OUTUNT,'(10X,85(''-''))') 
C 
C5A----Close-out arc coordinate file 
C 
      WRITE(IOUT(3),'(''END'')') 
      CALL CLOSE_FILE (FILES(3),*9999) 
C 
C6----Close-out model cell attributes to label file 
C 
      CALL CLOSE_FILE (FILES(2),*9999) 
C 
C7----Messages 
C 
C 
C7A----Message to user 
C 
      WRITE(*,'(''      ..Vertical profile polygon coverage '', 
     &               A,'' consists of '')') 
     &            XSCOV (1:INDEX(XSCOV,' ')-1) 
      WRITE(*,'(''      ..'',I10,'' cells'')') NCELLS 
      WRITE(*,'(''      ..'',I10,'' confining cells'')') NUMCON 
      WRITE(*,'(''      ..'',I10,'' inactive cells'')') BADIBD 
      WRITE(*,'(''      ..'',I10,'' noflow cells'')') NUMFLW 
      WRITE(*,'(''      ..'',I10,'' cells omitted '', 
     &          ''tops equals bottoms'')') BADTOP 
      WRITE(*,'(''      ..           cells had '', 
     &          ''maximum elevation of '',E15.6)') TOPMAX 
      WRITE(*,'(''      ..           cells had '', 
     &          ''minimum elevation of '',E15.6)') BOTMIN 
      WRITE(*,'(''      ..'',I10,'' arcs'')') NARCS 
      WRITE(*,'(''      ..'',I10,'' labels'')') NLABS 
      WRITE(*,'(''      ..'',I10,'' arcs having '', 
     &          ''zero length'')') BADARC 
C 
C7B----Message to operational output file 
C 
      WRITE(OUTUNT,'(//,''      ..Vertical profile polygon coverage '', 
     &               A,'' consists of '')') 
     &            XSCOV (1:INDEX(XSCOV,' ')-1) 
      WRITE(OUTUNT,'(''      ..'',I10,'' cells'')') NCELLS 
      WRITE(OUTUNT,'(''      ..'',I10,'' confining cells'')') NUMCON 
      WRITE(OUTUNT,'(''      ..'',I10,'' inactive cells'')') BADIBD 
      WRITE(OUTUNT,'(''      ..'',I10,'' noflow cells'')') NUMFLW 
      WRITE(OUTUNT,'(''      ..'',I10,'' cells omitted '', 
     &          ''tops equals bottoms'')') BADTOP 
      WRITE(OUTUNT,'(''      ..           cells had '', 
     &          ''maximum elevation of '',E15.6)') TOPMAX 
      WRITE(OUTUNT,'(''      ..           cells had '', 
     &          ''minimum elevation of '',E15.6)') BOTMIN 
      WRITE(OUTUNT,'(''      ..'',I10,'' arcs'')') NARCS 
      WRITE(OUTUNT,'(''      ..'',I10,'' labels'')') NLABS 
      WRITE(OUTUNT,'(''      ..'',I10,'' arcs having '', 
     &          ''zero length'')') BADARC 
C 
C8----Arc Operations to coverage and attribute files 
C 
      ARCCMD='create' 
      WRITE(IOUT(4),'(A,1X,A)')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1) 
C 
C8A----Generate Coverage temporary TMPCOV (polygon only) 
C 
      ARCCMD='generate' 
      COVTYP='polygon' 
      INPFIL=FILES(3) (1:INDEX(FILES(3),' ')-1) 
      WRITE(IOUT(4),'(A,3(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1), 
     &                             INPFIL (1:INDEX(INPFIL,' ')-1), 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
C 
C8B----Create attribute files and "add from" attributes 
C 
      ARCCMD='tables' 
      TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1) 
      ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1) 
      WRITE(IOUT(4),'(A,2(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1), 
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1) 
C 
C8E---Clean Coverage TMPCOV to produce XSCOV coverage (for polygon topology) 
C 
      ARCCMD='clean' 
      COVDAN=ABS(CLLMAX-CLLMIN)/1000000000.0 
      COVTOL=COVDAN 
      COVTYP='poly' 
      WRITE(IOUT(4),'(A,1X,A,1X,''#'',2(1X,F20.10),1X,A)')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1), 
     &                             COVDAN, 
     &                             COVTOL, 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
C 
C8F---Join attribute file label to Coverage XSCOV PAT file 
C 
      ARCCMD='joinitem' 
      LABFIL=XSCOV (1:INDEX(XSCOV,' ')-1)//'.PAT' 
      INPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1) 
      RELITM=XSCOV (1:INDEX(XSCOV,' ')-1)//'-id' 
      STRITM=XSCOV (1:INDEX(XSCOV,' ')-1)//'-id' 
      JONTYP='ordered'       
      WRITE(IOUT(4),'(A,6(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             LABFIL (1:INDEX(LABFIL,' ')-1), 
     &                             INPFIL (1:INDEX(INPFIL,' ')-1), 
     &                             LABFIL (1:INDEX(LABFIL,' ')-1), 
     &                             RELITM (1:INDEX(RELITM,' ')-1), 
     &                             STRITM (1:INDEX(STRITM,' ')-1), 
     &                             JONTYP (1:INDEX(JONTYP,' ')-1) 
C 
C8C----Build Coverage XSCOV 
C 
      ARCCMD='build' 
      COVTYP='line' 
      WRITE(IOUT(4),'(A,2(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1), 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
      CALL CLOSE_FILE (FILES(4),*9999) 
       GO TO 1000 
C**********************************************************************C 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C                        Program Errors 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
CE*********************************************************************C 
 9999 WRITE(*, 
     &    '(//,'' Abnormal termination of Cross_section_Subroutine'')') 
      WRITE(OUTUNT, 
     &    '(//,'' Abnormal termination of Cross_section_Subroutine'')') 
      RETURN 1 
C**********************************************************************C 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C                  Return to Calling Program 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C**********************************************************************C 
 1000 RETURN 
      END 
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthnor version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event 
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270 
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470 
C*   10615 SE. Cherry Blossom Drive 
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov 
C* 
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C* 
C* Disclaimer:  
C           Although this program has been used by the U.S. Geological Survey, 
C            no warranty, expressed or implied, is made by the USGS as to the 
C            accuracy and functioning of the program and related program 
C            material nor shall the fact of distribution constitute any such 
C            warranty, and no responsibility is assumed by the USGS in 
C            connection therewith. 
C* 
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C 
C 
C---VERSION 6.0 PTHNOR 26MARCH2002 
C 
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: 
C 
C .. Purpose: 
C 
C     Generates a normalized cross-sectional view coverage (polygon topology) 
C      along a row or column (or a particle pathline) using input to the 
C      particle tracking program MODPATH (Pollock, 1989).  Attributes (items)  
C      associated with each cell are (these can be selected by the user to 
C      shade each cell or other user defined functions or actions) : 
C      POLYGON ATTRIBUTE ARRAY: 
C              => (n,1) CELL POLYGON NUMBER 
C              => (n,2) CELL ROW 
C              => (n,3) CELL COLUMN 
C              => (n,4) CELL LAYER 
C              => (n,5) MODEL IBOUND VALUE 
C              => (n,6) CELL TOP ELEVATION 
C              => (n,7) CELL BOTTOM ELEVATION 
C              => (n,8) CELL THICKNESS 
C 
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: 
C 
C .. Language: FORTRAN 
C 
C 
C    Inputs: 
C 
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED 
C     XSCOV   : COVERAGE ROOT NAME 
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS 
C     NLAY    : NUMBER OF LAYERS IN MODEL 
C     NROW    : NUMBER OF ROWS IN GRID 
C     NCOL    : NUMBER OF COLUMNS IN GRID 
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION 
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR) 
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NCOL*NROW) 
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY) 
C     ZTOP    : ARRAY FOR LAYER/UNIT TOP ELEVATIONS 
C     ZBOT    : ARRAY FOR LAYER/UNIT BOTTOM ELEVATIONS 
C     ZCON    : ARRAY FOR CONFINING LAYER THICKNESS 
C 
C    Outputs: None 
C 
C----------------------------------------------------------------------- 
      SUBROUTINE PTHNOR ( 
     I                   OUTUNT, 
     I                   XSCOV, 
     I                   MAXPMR, 
     I                   NLAY,NROW,NCOL, 
     I                   DELX,COVPMR, 
     I                   LRWCL,IBOUND,LAYCBD, 
     I                   ZTOP,ZBOT,ZCON, 
     E                   *) 
C----------------------------------------------------------------------- 
C 
C        SPECIFICATIONS: 
C     ------------------------------------------------------------------ 
C 
C:::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::: 
C 
      DIMENSION  
     &          COVPMR(MAXPMR), 
     &          LAYCBD(NLAY), 
     &          DELX(NCOL*NROW), 
     &          LRWCL(2,NCOL*NROW), 
     &          IBOUND(NCOL,NROW,NLAY), 
     &          ZTOP(NCOL,NROW,NLAY), 
     &          ZBOT(NCOL,NROW,NLAY),
     &          ZCON(NCOL,NROW,NLAY) 
C 
      CHARACTER*(*) XSCOV 
C 
      INTEGER OUTUNT 
C 
C::::::::::::::::::::::::  PROGRAM_VARIABLES  :::::::::::::::::::::::::: 
C 
      PARAMETER (MAXFIL=4) 
      PARAMETER (MAXITM=8) 
C 
      CHARACTER*128 TMPCOV,LABFIL 
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL 
      CHARACTER*16 ARCCMD,ACTION,STRITM,RELITM,JONTYP, 
     &             COVTYP,ITEM,ITEMS(MAXITM) 
      CHARACTER*1 TYPITM,TYPES(MAXITM) 
C 
      INTEGER  
     &        IVIEW,IRWCL, 
     &        DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN, 
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM), 
     &        ROW,COLUMN,CELL,BADARC,BADIBD,BADTOP,NUMDRY,NUMFLW,COVDEX, 
     &        LAYER, 
     &        ICOL,IROW,ILAY, 
     &        STRCLL,FINCLL 
C 
      LOGICAL CLLFLG 
C 
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE 
C 
      DATA FILES /'cellitems','cellitem','arcscrs','arccrs'/ 
      DATA IOUT /90,91,92,93/ 
      DATA ITEMS /'CELL','ROW','COLUMN','LAYER','IBOUND', 
     &            'TOPELEV','BOTELEV','THICK'/ 
      DATA DECIMLS /5*0,3*3/ 
      DATA TYPES /'B',4*'I',3*'F'/ 
      DATA WIDTHS /4,4*10,3*8/ 
      DATA OUTPUTS /5,4*10,3*16/ 
C 
C     ------------------------------------------------------------------ 
C 
CH----Print Modtools header 
C 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''          MODEL GRID GIS OUTPUT PACKAGE'')') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''               VERSION 6.0 26MARCH2002 '')') 
      WRITE(OUTUNT,'(10X,85(''*''))') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''           NORMALIZED CROSS-SECTION MODULE '')') 
      WRITE(OUTUNT,'(10X,85(''=''))') 
      WRITE(OUTUNT,'(//)') 
C 
C1----Set local variables 
C 
      ITMSTR=1 
      ITMFIN=8 
      NARCS=0 
      NLABS=0 
      TMPCOV='tmp' 
      ZSCALE=COVPMR(4) 
      IF(COVPMR(9).LE.0.0) COVPMR(9)=1.0 
      HSCALE=12.0/COVPMR(9) 
      IF(COVPMR(8).LE.0.0) COVPMR(8)=1.0 
      VSCALE=HSCALE*COVPMR(8) 
      IVIEW=INT(COVPMR(10)) 
      IRWCL=INT(COVPMR(11)) 
      HNOFLO=COVPMR(16) 
C 
C1B---Set local variables 
C 
      IF(IVIEW.LE.1) THEN 
         WRITE(*,'(''       .Constructing normalized vertical profile '', 
     &             ''along row '',I10)') 
     &          IRWCL 
      ELSE IF(IVIEW.EQ.2) THEN 
         WRITE(*,'(''       .Constructing normalized vertical profile '', 
     &             ''along column '',I10)') 
     &          IRWCL 
      ELSE 
         WRITE(*,'(''       .Constructing normalized vertical profile '', 
     &             ''along particle '',I10)') 
     &          IRWCL 
      ENDIF 
C 
C2----Delete and create files for attribute definitions, attribute value, and 
C     coverage arc data and tolerance/dangle values 
C 
      NSTAT=2 
      ITALK=1 
C 
C2A----Delete/create files containing Arc/Info item definitions for 
C      attribute files 
C 
      DO 200 N=1,MAXFIL 
         CALL DELETE_FILE (FILES(N),*9999) 
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999) 
  200 CONTINUE 
C 
C2B----Calculate rowcol attribute definition 
C 
      IF(NROW.LT.10) THEN 
         LENROW=1 
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN 
         LENROW=2 
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN 
         LENROW=3 
      ELSE 
         LENROW=4 
      ENDIF 
C 
      IF(NCOL.LT.10) THEN 
         LENCOL=1 
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN 
         LENCOL=2 
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN 
         LENCOL=3 
      ELSE 
         LENCOL=4 
      ENDIF 
C 
      IF(NLAY.LT.10) THEN 
         LENLAY=1 
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN 
         LENLAY=2 
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN 
         LENLAY=3 
      ELSE 
         LENLAY=4 
      ENDIF 
C 
      WIDTHS(2)=LENROW 
      OUTPUTS(2)=LENROW 
      WIDTHS(3)=LENCOL 
      OUTPUTS(3)=LENCOL 
      WIDTHS(4)=LENLAY+1 
      OUTPUTS(4)=LENLAY+1 
C 
C2C----Write attribute definitions 
C 
      ITEMS(1)=XSCOV (1:INDEX(XSCOV,' ')-1)//'-id' 
      DO 210 K=ITMSTR,ITMFIN 
         DECIML=DECIMLS(K) 
         ITEM=ITEMS(K) 
         OUTPUT=OUTPUTS(K) 
         TYPITM=TYPES(K) 
         WIDTH=WIDTHS(K) 
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','', 
     &                   I2,'',,,,,,,,,,'')') 
     &                                    ITEM, 
     &                                    WIDTH, 
     &                                    OUTPUT, 
     &                                    TYPITM, 
     &                                    DECIML 
  210 CONTINUE 
       CALL CLOSE_FILE (FILES(1),*9999) 
C 
C3----Build arcs from elevation and row/column data 
C 
      WRITE(*,'(''      .Processing coordinates for '', 
     &             ''Vertical Cell profile now...'')') 
C 
C3A----Set cell min/max and x-direction sum variables 
C 
      BADARC=0 
      BADIBD=0 
      BADTOP=0 
      NUMDRY=0 
      NUMFLW=0 
      CLLMAX=-9999999999.99 
      CLLMIN=-CLLMAX 
      TOPMAX=-9999999999.99 
      BOTMIN=-CLLMAX 
      NUMCLL=0 
      NUMCON=0 
      NCELLS=0 
      WRITE(OUTUNT,'(//,1H1,15X, 
     &               '' CELLS WITH INVALID VALUES '', 
     &               ''WILL BE OMITTED FROM '', 
     &               ''CROSS-SECTION COVERAGE'', 
     &               /10X,85(''-''))') 
      WRITE(OUTUNT,'(40X, 
     &               ''       ROW'', 
     &               ''    COLUMN'', 
     &               ''     LAYER'', 
     &               ''    IBOUND'', 
     &               /10X,85(''-''))') 
C 
C3B----Set Loop index numcll 
C 
      IF(IVIEW.EQ.1) THEN 
         STRCLL=1 
         FINCLL=NCOL 
      ELSE IF(IVIEW.EQ.2) THEN 
         STRCLL=NCOL+1 
         FINCLL=NCOL+NROW 
      ELSE 
         STRCLL=1 
         FINCLL=IRWCL 
      ENDIF 
C 
C3C----Loop to find minimum and maximum elevations 
C 
      DO 300 ILAY=1,NLAY 
         TOPELE=0.0 
         BOTELE=0.0 
         TOPCON=0.0 
         BOTCON=0.0 
         NCELLS=0 
         NUMCLL=0 
         DO 310 N=STRCLL,FINCLL 
            NUMCLL=NUMCLL+1 
          IF(IVIEW.LE.1) THEN 
             IROW=IRWCL 
             ICOL=NUMCLL 
          ELSE IF(IVIEW.EQ.2) THEN 
             IROW=NUMCLL 
             ICOL=IRWCL 
          ELSE 
             ICOL=LRWCL(1,NUMCLL) 
             IROW=LRWCL(2,NUMCLL) 
          ENDIF 
C 
            IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY)) 
            IBDCON=ABS(LAYCBD(ILAY)) 
          IF(IBDCEL.NE.0) THEN 
             YTOP=ZTOP(ICOL,IROW,ILAY) 
             YBOT=ZBOT(ICOL,IROW,ILAY) 
           IF(YTOP.EQ.HNOFLO) THEN 
              NUMFLW=NUMFLW+1 
              IBOUND(ICOL,IROW,ILAY)=0 
              WRITE(OUTUNT,'(10X, 
     &                      ''NOFLOW CELL RESET IBOUND TO 0 '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
C 
           ELSE IF(YTOP.EQ.YBOT) THEN 
              BADTOP=BADTOP+1 
              IBOUND(ICOL,IROW,ILAY)=0 
              WRITE(OUTUNT,'(10X, 
     &                      ''CELL ELEVATION (TOP=BOTTOM)   '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
C 
           ELSE IF(YTOP.LT.YBOT) THEN 
              BADTOP=BADTOP+1 
              IBOUND(ICOL,IROW,ILAY)=0 
              WRITE(OUTUNT,'(10X, 
     &                      ''CELL ELEVATION (TOP<BOTTOM)   '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
C 
           ELSE 
              NCELLS=NCELLS+1 
              TOPELE=TOPELE+YTOP 
              BOTELE=BOTELE+YBOT 
           ENDIF 
          ELSE 
             BADIBD=BADIBD+1 
             WRITE(OUTUNT,'(10X, 
     &                      ''INACTIVE CELL (IBOUND=0)      '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
          ENDIF 
C
          IF(IBDCON.NE.0) THEN 
             YTOP=ZBOT(ICOL,IROW,ILAY) 
             YBOT=ZCON(ICOL,IROW,ILAY)
             TOPCON=TOPCON+YTOP
             BOTCON=BOTCON+YBOT
          ENDIF 
  310    CONTINUE 
C 
         TOPAVG=TOPELE/NCELLS 
         BOTAVG=BOTELE/NCELLS 
       IF(TOPAVG.GT.CLLMAX) CLLMAX=TOPAVG 
       IF(BOTAVG.LT.CLLMIN) CLLMIN=BOTAVG 
       IF(TOPAVG.GT.TOPMAX) TOPMAX=TOPAVG 
       IF(BOTAVG.LT.BOTMIN) BOTMIN=BOTAVG 
          WRITE(OUTUNT,'(10X, 
     &                   ''AVERAGE CELL ELEVATIONS LAYER'', 
     &                   20X,I10,2(F12.5,1X)))') 
     &               ILAY, 
     &               TOPAVG, 
     &               BOTAVG 
         TOPCON=TOPCON/NCELLS 
         BOTCON=BOTCON/NCELLS 
       IF(TOPCON.GT.CLLMAX) CLLMAX=TOPAVG 
       IF(BOTCON.LT.CLLMIN) CLLMIN=BOTAVG 
       IF(TOPCON.GT.TOPMAX) TOPMAX=TOPAVG 
       IF(BOTCON.LT.BOTMIN) BOTMIN=BOTAVG 
          WRITE(OUTUNT,'(10X, 
     &                   ''AVERAGE CONFINING ELEVATIONS  LAYER'', 
     &                   20X,I10,2(F12.5,1X)))') 
     &               ILAY, 
     &               TOPCON, 
     &               BOTCON 
C 
         NUMCLL=0 
         DO 320 N=STRCLL,FINCLL 
            NUMCLL=NUMCLL+1 
          IF(IVIEW.LE.1) THEN 
             IROW=IRWCL 
             ICOL=NUMCLL 
          ELSE IF(IVIEW.EQ.2) THEN 
             IROW=NUMCLL 
             ICOL=IRWCL 
          ELSE 
             ICOL=LRWCL(1,NUMCLL) 
             IROW=LRWCL(2,NUMCLL) 
          ENDIF 
C 
            ZTOP(ICOL,IROW,ILAY)=TOPAVG 
            ZBOT(ICOL,IROW,ILAY)=BOTAVG 
            ZCON(ICOL,IROW,ILAY)=BOTCON
  320    CONTINUE 
  300 CONTINUE 
      COVPMR(MAXPMR)=CLLMIN*VSCALE 
      COVPMR(MAXPMR-1)=CLLMAX*VSCALE 
      CLLMAX=CLLMAX*VSCALE 
C 
C4----Build arcs from elevation and row/column data 
C 
      XLEFT=0.0 
      ISIDES=-99 
      IBOTTM=-9 
      NUMCLL=0 
      WRITE(OUTUNT,'(//,1H1,35X, 
     &               '' NORMALIZED '', 
     &               ''CROSS-SECTION COVERAGE'', 
     &               /10X,85(''-''))') 
      WRITE(OUTUNT,'(40X, 
     &               ''       ROW'', 
     &               ''    COLUMN'', 
     &               ''     LAYER'', 
     &               ''         TOP'',1X, 
     &               ''      BOTTOM'')') 
      WRITE(OUTUNT,'(70X, 
     &               ''  ELEVATIONS'',1X, 
     &               ''  ELEVATIONS'')') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
C 
C4----Loop to build arcs 
C 
      DO 400 N=STRCLL,FINCLL 
         NUMCLL=NUMCLL+1 
         CLLFLG=.FALSE. 
         XWIDTH=DELX(N)*HSCALE 
         XRIGHT=XLEFT+XWIDTH 
         HWIDTH=XWIDTH/2.0 
C 
       IF(IVIEW.LE.1) THEN 
          IROW=IRWCL 
          ICOL=NUMCLL 
       ELSE IF(IVIEW.EQ.2) THEN 
          IROW=NUMCLL 
          ICOL=IRWCL 
       ELSE 
          ICOL=LRWCL(1,NUMCLL) 
          IROW=LRWCL(2,NUMCLL) 
       ENDIF 
         YMAX=-99999999999.99 
         YMIN=-YMAX 
C 
C4A---Cell with no width 
C 
       IF(XLEFT.EQ.XRIGHT) THEN 
          BADARC=BADARC+1 
          WRITE(OUTUNT,'(10X, 
     &                   ''ZERO LENGTH TOP ARC           '', 
     &                   4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
          GO TO 400 
       ENDIF 
C 
C4B----Set correct row/column 
C 
         DO 410 ILAY=1,NLAY 
            IBDCEL=IBOUND(ICOL,IROW,ILAY) 
            IBDCON=ABS(LAYCBD(ILAY)) 
          IF(IBDCEL.EQ.0) THEN 
             WRITE(OUTUNT,'(10X, 
     &                      ''INACTIVE CELL (IBOUND=0)      '', 
     &                      4I10)') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               IBDCEL 
          ELSE 
             CLLFLG=.TRUE. 
             NARCS=NARCS+1 
             TOP=ZTOP(ICOL,IROW,ILAY) 
             BOT=ZBOT(ICOL,IROW,ILAY) 
             YTOP=(TOP-CLLMIN)*VSCALE 
             YBOT=(BOT-CLLMIN)*VSCALE 
           IF(YTOP.GT.YMAX) YMAX=YTOP 
           IF(YBOT.LT.YMIN) YMIN=YBOT 
C 
C4Ba---Cell with no thickness 
C 
           IF(YTOP.EQ.YBOT) THEN 
              WRITE(OUTUNT,'(10X, 
     &                       ''CELL WITH NO THICKNESS       '', 
     &                       3I10,2(F12.5,1X))') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               TOP, 
     &               BOT 
C 
C4Bb---Cell [active] 
C 
           ELSE 
              WRITE(OUTUNT,'(10X, 
     &                       ''ACTIVE CELL                   '', 
     &                       3I10,2(F12.5,1X))') 
     &               IROW, 
     &               ICOL, 
     &               ILAY, 
     &               TOP, 
     &               BOT 
C 
C4Bc---Cell label 
C 
              NLABS=NLABS+1 
              XLABEL=XLEFT+HWIDTH 
              YLABEL=YBOT+ABS(YTOP-YBOT)/2.0 
              WRITE(IOUT(3),'(I10,'','',F20.10,'','',F20.10)') 
     &                    NLABS, 
     &                    XLABEL, 
     &                    YLABEL 
C 
C4Bd---Cell walls 
C 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XLEFT, 
     &                                               YTOP 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XRIGHT, 
     &                                               YTOP 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XRIGHT, 
     &                                               YBOT 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XLEFT, 
     &                                               YBOT 
              WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                               XLEFT, 
     &                                               YTOP 
              WRITE(IOUT(3),'(''END'')') 
C 
C4Ac---Write cell attributes 
C 
              THKLAY=TOP-BOT 
              WRITE(IOUT(2),'(5(I10,'',''), 
     &                        3(F16.3,'',''))') 
     &                    NLABS, 
     &                    IROW, 
     &                    ICOL, 
     &                    ILAY, 
     &                    IBDCEL, 
     &                    TOP, 
     &                    BOT, 
     &                    THKLAY 
C
C4B---Confining Layer
C
            IF(IBDCON.NE.0) THEN 
               TOPCON=ZBOT(ICOL,IROW,ILAY)
               BOTCON=ZCON(ICOL,IROW,ILAY)
               NARCS=NARCS+1
               NUMCON=NUMCON+1
               YTOP=(TOPCON-CLLMIN)*VSCALE
               YBOT=(BOTCON-CLLMIN)*VSCALE
C
               WRITE(OUTUNT,'(10X,
     &                       ''CONFINING CELL                '',
     &                       3I10,2(F12.5,1X))')
     &                IROW,
     &                ICOL,
     &                ILAY,
     &                TOPCON,
     &                BOTCON
C
C4Bb---Write confining cell IBOUND
C
             IF(NLAY.LT.10) THEN
                IBDCON=90+ILAY
             ELSE IF(NLAY.GE.10 .AND. NLAY.LT.100) THEN
                IBDCON=900+ILAY
             ELSE
                IBDCON=9000+ILAY
             ENDIF
C
C4Bb---Write cell label for confining layer
C
               NLABS=NLABS+1
               XLABEL=XLEFT+HWIDTH
               YLABEL=YBOT+ABS(YTOP-YBOT)/2.0
               WRITE(IOUT(3),'(I10,'','',F20.10,'','',F20.10)')
     &                      NLABS,
     &                      XLABEL,
     &                      YLABEL
C
C4Ba---Write cell walls for confining layer
C
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XLEFT, 
     &                                                YTOP 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XRIGHT, 
     &                                                YTOP 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XRIGHT, 
     &                                                YBOT 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XLEFT, 
     &                                                YBOT 
               WRITE(IOUT(3),'(F20.10,'','',F20.10)') 
     &                                                XLEFT, 
     &                                                YTOP 
               WRITE(IOUT(3),'(''END'')') 
C
C4Bc---Write cell attributes for confining layer
C
               THKLAY=TOP-BOT
               WRITE(IOUT(2),'(5(I10,'',''),
     &                         3(F16.3,'',''))')
     &                     NLABS,
     &                     IROW,
     &                     ICOL,
     &                     IBDCON,
     &                     IBDCEL,
     &                     TOP,
     &                     BOT,
     &                     THKLAY
            ENDIF
           ENDIF 
          ENDIF 
  410    CONTINUE 
         XLEFT=XRIGHT 
  400 CONTINUE 
      COVPMR(MAXPMR-2)=XLEFT 
C 
C5----Close-out arc coordinate file 
C 
      WRITE(OUTUNT,'(10X,85(''-''))') 
C 
C5A----Close-out arc coordinate file 
C 
      WRITE(IOUT(3),'(''END'')') 
      CALL CLOSE_FILE (FILES(3),*9999) 
C 
C6----Close-out model cell attributes to label file 
C 
      CALL CLOSE_FILE (FILES(2),*9999) 
C 
C7----Messages 
C 
C 
C7A----Message to user 
C 
      WRITE(*,'(''      ..Vertical profile polygon coverage '', 
     &               A,'' consists of '')') 
     &            XSCOV (1:INDEX(XSCOV,' ')-1) 
      WRITE(*,'(''      ..'',I10,'' cells'')') NCELLS 
      WRITE(*,'(''      ..'',I10,'' confining cells'')') NUMCON 
      WRITE(*,'(''      ..'',I10,'' inactive cells'')') BADIBD 
      WRITE(*,'(''      ..'',I10,'' noflow cells'')') NUMFLW 
      WRITE(*,'(''      ..'',I10,'' cells omitted '', 
     &          ''tops equals bottoms'')') BADTOP 
      WRITE(*,'(''      ..           cells had '', 
     &          ''maximum elevation of '',F15.6)') TOPMAX 
      WRITE(*,'(''      ..           cells had '', 
     &          ''minimum elevation of '',F15.6)') BOTMIN 
      WRITE(*,'(''      ..'',I10,'' arcs'')') NARCS 
      WRITE(*,'(''      ..'',I10,'' labels'')') NLABS 
      WRITE(*,'(''      ..'',I10,'' arcs having '', 
     &          ''zero length'')') BADARC 
C 
C7B----Message to operational output file 
C 
      WRITE(OUTUNT,'(//,''      ..Vertical profile polygon coverage '', 
     &               A,'' consists of '')') 
     &            XSCOV (1:INDEX(XSCOV,' ')-1) 
      WRITE(OUTUNT,'(''      ..'',I10,'' cells'')') NCELLS 
      WRITE(OUTUNT,'(''      ..'',I10,'' confining cells'')') NUMCON 
      WRITE(OUTUNT,'(''      ..'',I10,'' inactive cells'')') BADIBD 
      WRITE(OUTUNT,'(''      ..'',I10,'' noflow cells'')') NUMFLW 
      WRITE(OUTUNT,'(''      ..'',I10,'' cells omitted '', 
     &          ''tops equals bottoms'')') BADTOP 
      WRITE(OUTUNT,'(''      ..           cells had '', 
     &          ''maximum elevation of '',F15.6)') TOPMAX 
      WRITE(OUTUNT,'(''      ..           cells had '', 
     &          ''minimum elevation of '',F15.6)') BOTMIN 
      WRITE(OUTUNT,'(''      ..'',I10,'' arcs'')') NARCS 
      WRITE(OUTUNT,'(''      ..'',I10,'' labels'')') NLABS 
      WRITE(OUTUNT,'(''      ..'',I10,'' arcs having '', 
     &          ''zero length'')') BADARC 
C 
C8----Arc Operations to coverage and attribute files 
C 
      ARCCMD='create' 
      WRITE(IOUT(4),'(A,1X,A)')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1) 
C 
C8A----Generate Coverage temporary TMPCOV (polygon only) 
C 
      ARCCMD='generate' 
      COVTYP='polygon' 
      INPFIL=FILES(3) (1:INDEX(FILES(3),' ')-1) 
      WRITE(IOUT(4),'(A,3(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1), 
     &                             INPFIL (1:INDEX(INPFIL,' ')-1), 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
C 
C8B----Create attribute files and "add from" attributes 
C 
      ARCCMD='tables' 
      TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1) 
      ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1) 
      WRITE(IOUT(4),'(A,2(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1), 
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1) 
C 
C8E---Clean Coverage TMPCOV to produce XSCOV coverage (for polygon topology) 
C 
      ARCCMD='clean' 
      COVDAN=ABS(CLLMAX-CLLMIN)/1000000000.0 
      COVTOL=COVDAN 
      COVTYP='poly' 
      WRITE(IOUT(4),'(A,1X,A,1X,''#'',2(1X,F20.10),1X,A)')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1), 
     &                             COVDAN, 
     &                             COVTOL, 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
C 
C8F---Join attribute file label to Coverage XSCOV PAT file 
C 
      ARCCMD='joinitem' 
      LABFIL=XSCOV (1:INDEX(XSCOV,' ')-1)//'.PAT' 
      INPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1) 
      RELITM=XSCOV (1:INDEX(XSCOV,' ')-1)//'-id' 
      STRITM=XSCOV (1:INDEX(XSCOV,' ')-1)//'-id' 
      JONTYP='ordered'       
      WRITE(IOUT(4),'(A,6(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             LABFIL (1:INDEX(LABFIL,' ')-1), 
     &                             INPFIL (1:INDEX(INPFIL,' ')-1), 
     &                             LABFIL (1:INDEX(LABFIL,' ')-1), 
     &                             RELITM (1:INDEX(RELITM,' ')-1), 
     &                             STRITM (1:INDEX(STRITM,' ')-1), 
     &                             JONTYP (1:INDEX(JONTYP,' ')-1) 
C 
C8C----Build Coverage XSCOV 
C 
      ARCCMD='build' 
      COVTYP='line' 
      WRITE(IOUT(4),'(A,2(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             XSCOV (1:INDEX(XSCOV,' ')-1), 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
      CALL CLOSE_FILE (FILES(4),*9999) 
       GO TO 1000 
C**********************************************************************C 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C                        Program Errors 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
CE*********************************************************************C 
 9999 WRITE(*, 
     &    '(//,'' Abnormal termination of Cross_section_Subroutine'')') 
      WRITE(OUTUNT, 
     &    '(//,'' Abnormal termination of Cross_section_Subroutine'')') 
      RETURN 1 
C**********************************************************************C 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C                  Return to Calling Program 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C**********************************************************************C 
 1000 RETURN 
      END 
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthgrd version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event 
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270 
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470 
C*   10615 SE. Cherry Blossom Drive 
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov 
C* 
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C* 
C* Disclaimer:  
C           Although this program has been used by the U.S. Geological Survey, 
C            no warranty, expressed or implied, is made by the USGS as to the 
C            accuracy and functioning of the program and related program 
C            material nor shall the fact of distribution constitute any such 
C            warranty, and no responsibility is assumed by the USGS in 
C            connection therewith. 
C* 
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C 
C---VERSION 6.1 PTHGRD 15APRIL2003
C 
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: 
C 
C .. Purpose: 
C 
C     Generates a mapview coverage (polygon topology) of the model grid 
C      used in MODFLOW (McDonald and Harbaugh, 1988).  Attributes (items)  
C      associated with each cell are (these can be selected by the user to 
C      shade each cell or other user defined functions or actions) : 
C      POLYGON ATTRIBUTE ARRAY: 
C              => (n,1) CELL POLYGON NUMBER 
C              => (n,2) CELL ROWCOL 
C              => (n,3) CELL ROW 
C              => (n,4) CELL COLUMN 
C              => (n,5) CELL SEQNUM 
C 
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: 
C 
C .. Language: FORTRAN 
C 
C 
C    Inputs: 
C 
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED 
C     GRDCOV  : COVERAGE ROOT NAME 
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS 
C     NLAY    : NUMBER OF LAYERS IN MODEL 
C     NROW    : NUMBER OF ROWS IN GRID 
C     NCOL    : NUMBER OF COLUMNS IN GRID 
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION 
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR) 
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY) 
C 
C    Outputs: None 
C 
C----------------------------------------------------------------------- 
      SUBROUTINE PTHGRD ( 
     I                   OUTUNT, 
     I                   GRDCOV, 
     I                   MAXPMR, 
     I                   NLAY,NROW,NCOL, 
     I                   DELX,COVPMR, 
     I                   IBOUND, 
     E                   *) 
C----------------------------------------------------------------------- 
C 
C        SPECIFICATIONS: 
C     ------------------------------------------------------------------ 
C 
C:::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::: 
C 
      DIMENSION  
     &          COVPMR(MAXPMR), 
     &          DELX(NCOL*NROW), 
     &          IBOUND(NCOL,NROW,NLAY) 
C 
      INTEGER  
     &        OUTUNT 
C 
      CHARACTER*(*) GRDCOV 
C 
C::::::::::::::::::::::::  PROGRAM_VARIABLES  :::::::::::::::::::::::::: 
C 
      PARAMETER (MAXFIL=4) 
      PARAMETER (MAXITM=5) 
C 
      CHARACTER*128 TMPCOV,LABFIL 
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL 
      CHARACTER*16 ARCCMD,ACTION,STRITM,RELITM,JONTYP, 
     &             COVTYP,ITEM,ITEMS(MAXITM) 
      CHARACTER*10 CHRLAY,CHRWCL,LAYFMT,ITMFMT 
      CHARACTER*1 TYPITM,TYPES(MAXITM) 
C 
      INTEGER  
     &        IVIEW,IRWCL, 
     &        DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN, 
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM), 
     &        ROWCOL, 
     &        ROW,COL,COLUMN,SEQNUM,NUMCLL, 
     &        CELL,BADARC,BADIBD,BADTOP,NUMDRY,NUMFLW,COVDEX, 
     &        LAYER, 
     &        ICOL,IROW,ILAY, 
     &        STRCHR,FINCHR,STRCLL,FINCLL 
C 
      LOGICAL CLLFLG 
C 
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE 
C 
      DATA FILES /'cellitems','cellitem','arcscrs','arccrs'/ 
      DATA IOUT /90,91,92,93/ 
      DATA ITEMS /'CELL','ROWCOL','ROW','COLUMN','SEQNUM'/ 
      DATA DECIMLS /5*0/ 
      DATA TYPES /'B','C',3*'I'/ 
      DATA WIDTHS /4,4*10/ 
      DATA OUTPUTS /5,4*10/ 
C 
C     ------------------------------------------------------------------ 
C 
CH----Print Modtools header 
C 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''          MODEL GRID GIS OUTPUT PACKAGE'')') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''               VERSION 6.1 15APRIL2003 '')') 
      WRITE(OUTUNT,'(10X,85(''*''))') 
      WRITE(OUTUNT,'(10X,85(''-''))') 
      WRITE(OUTUNT,'(30X, 
     &               ''         MAPVIEW GRID GENERATION MODULE '')') 
      WRITE(OUTUNT,'(10X,85(''=''))') 
      WRITE(OUTUNT,'(//)') 
C 
C1----Set local variables 
C 
      ITMSTR=1 
      ITMFIN=8 
      NARCS=0 
      NLABS=0 
      TMPCOV='tmp' 
      XYSCALE=COVPMR(3) 
      ZSCALE=COVPMR(4) 
      ANGLE=COVPMR(5) 
      XOGRID=COVPMR(6) 
      YOGRID=COVPMR(7) 
      IF(COVPMR(9).LE.0.0) COVPMR(9)=1.0 
      HSCALE=12.0/COVPMR(9) 
      IF(COVPMR(8).LE.0.0) COVPMR(8)=1.0 
      VSCALE=HSCALE*COVPMR(8) 
      IVIEW=INT(COVPMR(10)) 
      ILAY=INT(COVPMR(11)) 
C 
C1B---Set local variables 
C 
      WRITE(*,'(''       ..Processing Information '', 
     &          ''for model grid '')') 
C 
C2----Delete and create files for attribute definitions, attribute value, and 
C     coverage arc data and tolerance/dangle values 
C 
      NSTAT=2 
      ITALK=1 
C 
C2A----Delete/create files containing Arc/Info item definitions for 
C      attribute files 
C 
      DO 200 N=1,MAXFIL 
         CALL DELETE_FILE (FILES(N),*9999) 
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999) 
  200 CONTINUE 
C 
C2B----Calculate rowcol attribute definition 
C 
      IF(NROW.LT.10) THEN 
         LENROW=1 
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN 
         LENROW=2 
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN 
         LENROW=3 
      ELSE 
         LENROW=4 
      ENDIF 
C 
      IF(NCOL.LT.10) THEN 
         LENCOL=1 
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN 
         LENCOL=2 
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN 
         LENCOL=3 
      ELSE 
         LENCOL=4 
      ENDIF 
C 
      IF(NLAY.LT.10) THEN 
         LENLAY=1 
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN 
         LENLAY=2 
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN 
         LENLAY=3 
      ELSE 
         LENLAY=4 
      ENDIF 
C 
      WIDTHS(2)=LENROW+LENCOL 
      OUTPUTS(2)=LENROW+LENCOL 
      WIDTHS(3)=LENROW 
      OUTPUTS(3)=LENROW 
      WIDTHS(4)=LENCOL 
      OUTPUTS(4)=LENCOL 
C 
C2C----Write attribute definitions 
C 
      ITEMS(1)=GRDCOV (1:INDEX(GRDCOV,' ')-1)//'-id' 
      DO 210 K=1,MAXITM 
         DECIML=DECIMLS(K) 
         ITEM=ITEMS(K) 
         OUTPUT=OUTPUTS(K) 
         TYPITM=TYPES(K) 
         WIDTH=WIDTHS(K) 
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','', 
     &                   I2,'',,,,,,,,,,'')') 
     &                                    ITEM, 
     &                                    WIDTH, 
     &                                    OUTPUT, 
     &                                    TYPITM, 
     &                                    DECIML 
  210 CONTINUE 
       CALL CLOSE_FILE (FILES(1),*9999) 
C 
C3----MODFLOW grid 
C 
      NUMCLL=0 
      NLABS=0 
      BADIBD=0 
      ROWMAX=-9999999999.99 
      ROWMIN=-ROWMAX 
      COLMAX=-9999999999.99 
      COLMIN=-COLMAX 
      WRITE(*,'(''       ..Processing cell coordinates for '', 
     &             ''Model grid now...'')') 
C 
C3A---Loop to build labels from row/column data 
C 
      XLEFT=0.0
      XRGHT=0.0 
      DO 300 COL=1,NCOL 
         ICOL=COL 
         XRGHT=XLEFT+DELX(COL)*XYSCALE 
         XLABEL=(XRGHT+XLEFT)*0.5 
         YTOP=0.0 
         IROW=0 
       IF(DELX(COL).LT.COLMIN) COLMIN=DELX(COL) 
       IF(DELX(COL).GT.COLMAX) COLMAX=DELX(COL) 
         DO 320 ROW=NCOL+1,NCOL+NROW 
            NLABS=NLABS+1 
          IF(DELX(ROW).LT.ROWMIN) ROWMIN=DELX(ROW) 
          IF(DELX(ROW).GT.ROWMAX) ROWMAX=DELX(ROW) 
            IROW=IROW+1 
            ROWCOL=IROW*(10**LENCOL)+ICOL 
            WRITE(CHRWCL,'(I10)') ROWCOL 
            FINCHR=10 
            STRCHR=FINCHR-(LENROW+LENCOL-1) 
C
            YBOT=YTOP-1.0*DELX(ROW)*XYSCALE 
            YLABEL=(YTOP+YBOT)*0.5 
C 
C3B---Cell center
C 
            XCOORD=XLABEL
            YCOORD=YLABEL
          IF(ANGLE.NE.0.0) THEN 
              CALL ROTATE ( 
     I                     ANGLE, 
     B                     XCOORD,YCOORD) 
          ENDIF 
            XCOORD=XCOORD+XOGRID 
            YCOORD=YCOORD+YOGRID 
            WRITE(IOUT(3),'(I10,'','',F20.5,'','',F20.5)') 
     &                                              NLABS, 
     &                                              XCOORD, 
     &                                              YCOORD 
C 
C3B---Cell upper left
C 
            XCOORD=XLEFT
            YCOORD=YTOP
          IF(ANGLE.NE.0.0) THEN 
              CALL ROTATE ( 
     I                     ANGLE, 
     B                     XCOORD,YCOORD) 
          ENDIF 
            XCOORD=XCOORD+XOGRID 
            YCOORD=YCOORD+YOGRID 
            WRITE(IOUT(3),'(F20.5,'','',F20.5)') 
     &                                              XCOORD, 
     &                                              YCOORD 
            XFIRST=XCOORD
            YFIRST=YCOORD           
C 
C3B---Cell upper right
C 
            XCOORD=XRGHT
            YCOORD=YTOP
          IF(ANGLE.NE.0.0) THEN 
              CALL ROTATE ( 
     I                     ANGLE, 
     B                     XCOORD,YCOORD) 
          ENDIF 
            XCOORD=XCOORD+XOGRID 
            YCOORD=YCOORD+YOGRID 
            WRITE(IOUT(3),'(F20.5,'','',F20.5)') 
     &                                              XCOORD, 
     &                                              YCOORD 
C 
C3B---Cell lower right
C 
            XCOORD=XRGHT
            YCOORD=YBOT
          IF(ANGLE.NE.0.0) THEN 
              CALL ROTATE ( 
     I                     ANGLE, 
     B                     XCOORD,YCOORD) 
          ENDIF 
            XCOORD=XCOORD+XOGRID 
            YCOORD=YCOORD+YOGRID 
            WRITE(IOUT(3),'(F20.5,'','',F20.5)') 
     &                                              XCOORD, 
     &                                              YCOORD 
C 
C3B---Cell lower left
C 
            XCOORD=XLEFT
            YCOORD=YBOT
          IF(ANGLE.NE.0.0) THEN 
              CALL ROTATE ( 
     I                     ANGLE, 
     B                     XCOORD,YCOORD) 
          ENDIF 
            XCOORD=XCOORD+XOGRID 
            YCOORD=YCOORD+YOGRID 
            WRITE(IOUT(3),'(F20.5,'','',F20.5)') 
     &                                              XCOORD, 
     &                                              YCOORD 
C 
C3B---Cell upper left
C 
            WRITE(IOUT(3),'(F20.5,'','',F20.5)') 
     &                                              XFIRST, 
     &                                              YFIRST 
            WRITE(IOUT(3),'(''END'')') 
C 
C3Ca---Cell IBOUND
C 
            IBDCEL=IBOUND(ICOL,IROW,1) 
         IF(IBDCEL.NE.0) THEN 
            NUMCLL=NUMCLL+1 
         ELSE 
            BADIBD=BADIBD+1 
         ENDIF 
C 
            SEQNUM=(IROW-1)*NCOL+ICOL 
C 
C3Ca---Write cell attributes 
C 
            WRITE(IOUT(2),'( 
     &                      I10,'','', 
     &                      A,'','', 
     &                      3(I10,'',''))') 
     &                     NLABS, 
     &                     CHRWCL (STRCHR:FINCHR), 
     &                     IROW, 
     &                     ICOL, 
     &                     SEQNUM 
            YTOP=YBOT
  320    CONTINUE 
         XLEFT=XRGHT
  300 CONTINUE 
C 
C5----Close-out arc coordinate file 
C 
      WRITE(OUTUNT,'(10X,85(''-''))') 
C 
C5B----Close-Out label coordinate file 
C 
      WRITE(IOUT(3),'(''END'')') 
      CALL CLOSE_FILE (FILES(3),*9999) 
C 
C6----Close-out model cell attributes to label file 
C 
      CALL CLOSE_FILE (FILES(2),*9999) 
C 
C7----Messages 
C 
C 
C7A----Message to user 
C 
      WRITE(*,'(''       ..Model grid polygon coverage '',A, 
     &          '' consists of '')') 
     &        GRDCOV (1:INDEX(GRDCOV,' ')-1) 
      WRITE(*,'(''       ..'',I10,'' active cells'')') NUMCLL
      WRITE(*,'(''       ..'',I10,'' inactive cells'')') BADIBD 
      WRITE(*,'(''       ..'',I10,'' labels'')') NLABS 
      WRITE(*,'(''       ..           cells had '', 
     &          ''maximum width in row direction    '',F15.3)') ROWMAX 
      WRITE(*,'(''       ..           cells had '', 
     &          ''minimum width in row direction    '',F15.3)') ROWMIN 
      WRITE(*,'(''       ..           cells had '', 
     &          ''maximum width in column direction '',F15.3)') COLMAX 
      WRITE(*,'(''       ..           cells had '', 
     &          ''minimum width in column direction '',F15.3)') COLMIN 
C 
C7B----Message to operational output file 
C 
      WRITE(OUTUNT,'(''       ..Model grid polygon coverage '',A, 
     &          '' consists of '')') 
     &        GRDCOV (1:INDEX(GRDCOV,' ')-1) 
      WRITE(OUTUNT,'(''       ..'',I10,'' active cells'')') NUMCLL
      WRITE(OUTUNT,'(''       ..'',I10,'' inactive cells'')') BADIBD 
      WRITE(OUTUNT,'(''       ..'',I10,'' labels'')') NLABS 
      WRITE(OUTUNT,'(''       ..           cells had '', 
     &          ''maximum width in row direction    '',F15.3)') ROWMAX 
      WRITE(OUTUNT,'(''       ..           cells had '', 
     &          ''minimum width in row direction    '',F15.3)') ROWMIN 
      WRITE(OUTUNT,'(''       ..           cells had '', 
     &          ''maximum width in column direction '',F15.3)') COLMAX 
      WRITE(OUTUNT,'(''       ..           cells had '', 
     &          ''minimum width in column direction '',F15.3)') COLMIN 
C 
C8----Arc Operations to coverage and attribute files 
C 
      ARCCMD='create' 
      WRITE(IOUT(4),'(A,1X,A)')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             GRDCOV (1:INDEX(GRDCOV,' ')-1) 
C 
C8C---Generate Coverage TMPCOV (arcs for polygons only) 
C 
      ARCCMD='generate' 
      INPFIL=FILES(3) (1:INDEX(FILES(3),' ')-1) 
      COVTYP='polygons' 
      WRITE(IOUT(4),'(A,3(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             GRDCOV (1:INDEX(GRDCOV,' ')-1), 
     &                             INPFIL (1:INDEX(INPFIL,' ')-1), 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
C 
C8A----Create attribute files and "add from" attributes 
C 
      ARCCMD='tables' 
      TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1) 
      ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1) 
      WRITE(IOUT(4),'(A,2(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1), 
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
C 
C8E---Clean Coverage TMPCOV to produce GRDCOV coverage (for polygon topology) 
C 
      ARCCMD='clean' 
      COVDAN=(ABS(ROWMIN-COLMIN)+1.0)/1000000.0 
      IF(COVDAN.LT.0.00001) COVDAN=0.00001 
      COVTOL=COVDAN 
      COVTYP='poly' 
      WRITE(IOUT(4),'(A,1X,A,1X,''#'',2(1X,F20.10),1X,A)')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             GRDCOV (1:INDEX(GRDCOV,' ')-1), 
     &                             COVDAN, 
     &                             COVTOL, 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
C 
C8F---Join attribute file label to Coverage GRDCOV PAT file 
C 
      ARCCMD='joinitem' 
      LABFIL=GRDCOV (1:INDEX(GRDCOV,' ')-1)//'.PAT' 
      INPFIL=FILES(2) (1:INDEX(FILES(2),' ')-1) 
      RELITM=GRDCOV (1:INDEX(GRDCOV,' ')-1)//'-id' 
      STRITM=GRDCOV (1:INDEX(GRDCOV,' ')-1)//'-id' 
      JONTYP='ordered'       
      WRITE(IOUT(4),'(A,6(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             LABFIL (1:INDEX(LABFIL,' ')-1), 
     &                             INPFIL (1:INDEX(INPFIL,' ')-1), 
     &                             LABFIL (1:INDEX(LABFIL,' ')-1), 
     &                             RELITM (1:INDEX(RELITM,' ')-1), 
     &                             STRITM (1:INDEX(STRITM,' ')-1), 
     &                             JONTYP (1:INDEX(JONTYP,' ')-1) 
C 
C8C----Build Coverage GRDCOV 
C 
      ARCCMD='build' 
      COVTYP='line' 
      WRITE(IOUT(4),'(A,2(1X,A))')  
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1), 
     &                             GRDCOV (1:INDEX(GRDCOV,' ')-1), 
     &                             COVTYP (1:INDEX(COVTYP,' ')-1) 
      CALL CLOSE_FILE (FILES(4),*9999) 
       GO TO 1000 
C**********************************************************************C 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C                        Program Errors 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
CE*********************************************************************C 
 9999 WRITE(*, 
     &    '(//,'' Abnormal termination of Pthgrd_Subroutine'')') 
      WRITE(OUTUNT, 
     &    '(//,'' Abnormal termination of Pthgrd_Subroutine'')') 
      RETURN 1 
C**********************************************************************C 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C                  Return to Calling Program 
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C 
C**********************************************************************C 
 1000 RETURN 
      END 
