C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                Pthutl version 6.1                  *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Purpose: Although this program has been used by the U.S. Geological Survey,
C*           no warranty, expressed or implied, is made by the USGS as to the
C*           accuracy and functioning of the program and related program
C*           material nor shall the fact of distribution constitute any such
C*           warranty, and no responsibility is assumed by the USGS in
C*           connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C<><><><><><><><><><><><><><> UTILITY SUBROUTINES <><><><><><><><><><><><><><>C
C-----------------------------------------------------------------------
C
C---VERSION 6.1 PTHUTL 01MAY1998
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     A set of subroutines that deal with MODTOOLS
C
C    Subroutines:
C
C     U2DARY            : Loads array values from files output by MODFLOW
C                          like U2DREL submodule, but searches using 
C                          stress period and time step.
C     U3DARY            : Loads array values from files output by MODFLOW
C                          UBUDSV submodule, but searches using 
C                          stress period and time step.
C     ITEM_FILES        : Builds item definitions template file.
C
C     LOCAL_COORDINATES : Calculates local x-,y-coordinates inside a 
C                          mode-grid cell using the upper left corner of
C                          cell row 1 column 1 as the x-,y-origin.
C
C     CHRSTR            : Checks whether this string fits within numchr 
C                          range and if not truncates so it does.
C
C     ROTATE            : Rotates (x,y) by deg degrees around (0,0) and 
C                          returns in xr,yr.
C     
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 U2dary version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 U2DARY 17MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user specified file for cell_by_cell terms output by
C      the U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C      This program can read 2D data that is formatted in the same manner as 
C      output the ULASAV module of MODFLOW or that is formatted as text
C      in a gridded manner.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     TEXT    : SEARCH STRING FOR A
C     II      : NUMER OF ROWS
C     JJ      : NUMBER OF COLUMNS
C     KK      : NUMBER OF LAYERS
C     ARRFIL  : FILE CONTAINING THE CELL_BY_CELL TERMS
C     ARRUNT  : FORTRAN UNIT NUMBER CONFIL FILE
C     LVLFIL  : FILE CONTAINING THE RECORDS FOR THE UTILITY READERS 
C     LVLUNT  : FORTRAN UNIT NUMBER FOR LVLFIL
C     OUTUNT  : FORTRAN UNIT NUMBER DOCUMENTING U3DARY OPERATIONS.
C
C    Outputs:
C
C     A       : ARRAY FOR CELL-BY-CELL TERMS
C
C-------------------------------------------------------------------------
      SUBROUTINE U2DARY(
     O                  A,
     I                  ARRTXT,
     I                  II,JJ,KK,
     I                  ARRFIL,ARRUNT,CTLFIL,CTLUNT,
     I                  OUTUNT,
     I                  MODPER,MODSTP,
     E                  *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      CHARACTER*(*) ARRFIL,CTLFIL
      CHARACTER*16 ARRTXT
      DIMENSION A(JJ,II,KK)
C
      INTEGER 
     &       OUTUNT,
     &       ARRUNT,
     &       CTLUNT,
     &       MODPER,
     &       MODSTP
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*80 CNTRL
      CHARACTER*80 FNAME
      CHARACTER*24 ANAME
      CHARACTER*20 FMTIN
      CHARACTER*16 TEXT,BUDTXT
C
      INTEGER ARRFLG,OPNFLG,HEDFLG
C
      DATA NUNOPN/99/
      DATA BUDTXT /'MISCELLANEO TERM'/
C
      ANAME=ARRTXT 
      ARRUNT=0
      ARRFLG=0
      HEDFLG=0
      TEXT=ARRTXT
      OPNFLG=0 
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''        MODTOOLS UTILITY PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 17MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                U2DARY READER MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
CO----Open Control file CTLFIL
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (CTLUNT,CTLFIL,NSTAT,ITALK,*9999)
C
C1------READ ARRAY CONTROL RECORD AS CHARACTER DATA.
C
       K=0 
102     K=K+1
      IF(K.GT.KK) GO TO 702
         READ(CTLUNT,'(A)',ERR=9990,END=702) CNTRL
C
C2------LOOK FOR ALPHABETIC WORD THAT INDICATES THAT THE RECORD IS FREE
C2------FORMAT.  SET A FLAG SPECIFYING IF FREE FORMAT OR FIXED FORMAT.
C 
      ICLOSE=0
      IFREE=1
      ICOL=1
      CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,OUTUNT,CTLUNT)
      IF (CNTRL(ISTART:ISTOP).EQ.'CONSTANT') THEN
         LOCAT=0
         OPNFLG=1
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'INTERNAL') THEN
         LOCAT=CTLUNT
         OPNFLG=1
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'EXTERNAL') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,LOCAT,R,OUTUNT,CTLUNT)
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'OPEN/CLOSE') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,0,N,R,OUTUNT,CTLUNT)
         FNAME=CNTRL(ISTART:ISTOP)
         LOCAT=NUNOPN
         WRITE(OUTUNT,15) LOCAT,FNAME
   15    FORMAT(1X,/1X,'OPENING FILE ON UNIT',I4,':',/1X,A)
         ICLOSE=1
      ELSE
C
C2A-----DID NOT FIND A RECOGNIZED WORD, SO NOT USING FREE FORMAT.
C2A-----READ THE CONTROL RECORD THE ORIGINAL WAY.
C
         IFREE=0
         READ(CNTRL,1,ERR=9991,END=9992) LOCAT,CNSTNT,FMTIN,IPRN
    1    FORMAT(I10,G10.0,A20,I10)
      END IF
C
C3------FOR FREE FORMAT CONTROL RECORD, READ REMAINING FIELDS.
C
      IF(IFREE.NE.0) THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,3,N,CNSTNT,OUTUNT,CTLUNT)
         IF(LOCAT.NE.0) THEN
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,OUTUNT,CTLUNT)
            FMTIN=CNTRL(ISTART:ISTOP)
            IF(ICLOSE.NE.0) THEN
               OPNFLG=1
               IF(FMTIN.EQ.'(BINARY)') THEN
C                  OPEN(UNIT=LOCAT,FILE=FNAME,FORM='UNFORMATTED')
               ELSE
                  OPEN(UNIT=LOCAT,FILE=FNAME)
               END IF
            END IF
            IF(LOCAT.GT.0 .AND. FMTIN.EQ.'(BINARY)') LOCAT=-LOCAT
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,IPRN,R,OUTUNT,CTLUNT)
         END IF
      END IF
C
C3------Open array file if needed
C
      IF(OPNFLG.LT.1 .AND. ABS(LOCAT).NE.ABS(CTLUNT) .AND.
     &   ABS(LOCAT).NE.ABS(ARRUNT)) THEN 
         OPNFLG=1 
         NSTAT=1 
         ITALK=1 
       IF(LOCAT.NE.0) ARRUNT=LOCAT 
         CALL OPEN_FILE (ARRUNT,ARRFIL,NSTAT,ITALK,*9999) 
      ENDIF 
C
C4------TEST LOCAT TO SEE HOW TO DEFINE ARRAY VALUES.
C
      IF(LOCAT) 200,50,90
C
C4A-----LOCAT=0; SET ALL ARRAY VALUES EQUAL TO CNSTNT. RETURN.
C
   50 DO 80 I=1,II
      DO 80 J=1,JJ
   80    A(J,I,K)=CNSTNT
C
      WRITE(OUTUNT,2) ARRTXT,CNSTNT,K
    2 FORMAT(1X,/1X,A,' =',G15.7,' FOR LAYER',I10)
      ARRFLG=ARRFLG+1
      ILAY=ARRFLG
      KPER=MODPER
      KSTP=MODSTP
      GO TO 300
C
C4B-----LOCAT>0; READ FORMATTED RECORDS USING FORMAT FMTIN.
C
   90 WRITE(OUTUNT,94) ARRTXT,K,LOCAT,FMTIN
   94 FORMAT(1X,///11X,A,' FOR LAYER',I10,/
     1       1X,'READING ON UNIT',I10,' WITH FORMAT: ',A)
       MM=INDEX(FMTIN,'FREE')
       WRITE(OUTUNT,'('' FREE '',I10,'' INDEX '',I10)') IFREE,MM
C
      DO 100 I=1,II
C       IF(FMTIN.EQ.'(FREE)') THEN
       IF(INDEX(FMTIN,'(FREE)').GT.0) THEN
          READ(LOCAT,*,ERR=9995,END=9996) (A(J,I,K),J=1,JJ)
       ELSE
          READ(LOCAT,FMTIN,ERR=9995,END=9996) (A(J,I,K),J=1,JJ)
       END IF
  100 CONTINUE
      ARRFLG=ARRFLG+1
      ILAY=ARRFLG
      KPER=MODPER
      KSTP=MODSTP
      GO TO 300
C
C4C-----LOCAT<0; READ UNFORMATTED ARRAY VALUES.
C
  200 WRITE(OUTUNT,201) ARRTXT,K,LOCAT
  201 FORMAT(1X,///11X,A,' FOR LAYER ',I10,/
     1    1X,' READING BINARY ON UNIT ',I10)
C
C5------READ UNFORMATTED RECORDS
C
      ARRFLG=0
      CALL READ_ULASAV (
     I                     ARRFIL,
     I                     ARRTXT,
     I                     MODPER,
     I                     MODSTP,
     O                     A(1,1,1),
     E                     ARRFLG)
C
       IF(ARRFLG.EQ.KK) THEN
          WRITE(*,'(
     &              ''      .....loaded cell_by_cell terms '')')
          WRITE(OUTUNT,'(
     &              ''      .....loaded cell_by_cell terms '')')
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR LAYERS '',I4,/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             ARRTXT,KK,ARRUNT
       ELSE IF(ARRFLG.LT.KK) THEN
          WRITE(*,'(
     &              ''      ....Warning:Did not find cell_by_cell '',
     &              A,'' terms for every layer'')')
     &              ARRTXT (1:INDEX(ARRTXT,' ')-1)
          WRITE(*,'(
     &              ''      .        or for proper stress '',
     &              ''period or time step'')')
          WRITE(*,'(
     &              ''      .  in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....Warning:Did not find cell_by_cell '',
     &              A,'' terms for every layer'')')
     &              ARRTXT (1:INDEX(ARRTXT,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      .        or for proper stress '',
     &              ''period or time step'')')
          WRITE(OUTUNT,'(
     &              ''      .  in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          GO TO 9998
       ELSE
          WRITE(*,'(
     &              ''      ....Did not find cell_by_cell '',
     &              A,'' terms in file '',A)')
     &              ARRTXT (1:INDEX(ARRTXT,' ')-1),
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....Did not find cell_by_cell '',
     &              A,'' terms in file '',A)')
     &              ARRTXT (1:INDEX(ARRTXT,' ')-1),
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          GO TO 9998
       ENDIF
C
C5C------Close
C
      CALL CLOSE_FILE(CTLFIL,*9999)
      GO TO 902
C
C6------IF CNSTNT NOT ZERO THEN MULTIPLY ARRAY VALUES BY CNSTNT.
C
  300 IF(ICLOSE.NE.0) CLOSE(UNIT=LOCAT)
      ZERO=0.
      IF(CNSTNT.EQ.ZERO) GO TO 320
      DO 310 I=1,II
      DO 310 J=1,JJ
         A(J,I,K)=A(J,I,K)*CNSTNT
  310 CONTINUE
C
C6------IF PRINT CODE (IPRN) >0 OR =0 THEN PRINT ARRAY VALUES.
C
  320 IF(IPRN.GE.0) CALL ULAPRW(A(1,1,K),ARRTXT,0,0,JJ,II,0,IPRN,OUTUNT)
C
C7----Next layer
C
      GO TO 102
C
C8----Messages
C
 702  CALL CLOSE_FILE(CTLFIL,*9999)
      CALL CLOSE_FILE(ARRFIL,*9999)
C
C9------Check data flag
C
 902  IF(ARRFLG.LE.0) THEN
         WRITE(*,'(
     &            ''      .Did not find cell_by_cell '',
     &            ''terms in file '',A)')
     &            ARRFIL (1:INDEX(ARRFIL,' ')-1)
         WRITE(OUTUNT,'(
     &            ''      .Did not find cell_by_cell  '',
     &            ''terms in file '',A)')
     &            ARRFIL (1:INDEX(ARRFIL,' ')-1)
         GO TO 9999
      ELSE
          WRITE(*,'(''      ..Found header information for '',A)') 
     &         TEXT 
          WRITE(*,'(''      ...for layer '',I15)') 
     &         ILAY 
          WRITE(*,'(''      ...for stress period '',I15)') 
     &         MODPER 
          WRITE(*,'(''      ...for time step     '',I15)') 
     &         MODSTP 
      ENDIF
C
      GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,
     &          '' Unable to read control line from '',
     &          ''array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read control line from '',
     &          ''array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,
     &          '' Unable to read parameters from control line '',
     &          ''from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read parameters from control line '',
     &          ''from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9992 WRITE(*,'(//,
     &          '' End-of-file; Unable to read parameters from '',
     &          ''control line from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read parameters from '',
     &          ''control line from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9993 WRITE(*,'(//,
     &          '' Unable to read header text for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header text for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      GO TO 9998
 9994 WRITE(*,'(//,
     &          '' End-of-file; Unable to read header for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read header for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      GO TO 9998
 9995 WRITE(*,'(//, 
     &          '' Unable to read values from '', 
     &          ''cell-by-cell terms from file '',A)')  
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1) 
      WRITE(OUTUNT,'(//, 
     &          '' Unable to read values from '', 
     &          ''cell-by-cell terms from file '',A)')  
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1) 
      GO TO 9998 
 9996 WRITE(*,'(//, 
     &          '' End-of-file; Unable to read values from '', 
     &          ''cell-by-cell terms from file '',A)')  
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1) 
      WRITE(OUTUNT,'(//, 
     &          '' End-of-file; Unable to read values from '', 
     &          ''cell-by-cell terms from file '',A)')  
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1) 
 9997 CALL CLOSE_FILE (ARRFIL,*9999)
 9998 CALL CLOSE_FILE (CTLFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of U2dary_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of U2dary_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 U3dary version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 U3DARY 19MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user specified file for cell_by_cell terms output by
C      the U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C      This program can read 3D data that is formatted in the same manner as 
C      output the UBUDSV module of MODFLOW or that is formatted as text
C      in a gridded manner.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     TEXT    : SEARCH STRING FOR A
C     II      : NUMER OF ROWS
C     JJ      : NUMBER OF COLUMNS
C     KK      : NUMBER OF LAYERS
C     ARRFIL  : FILE CONTAINING THE CELL_BY_CELL TERMS
C     ARRUNT  : FORTRAN UNIT NUMBER CONFIL FILE
C     LVLFIL  : FILE CONTAINING THE RECORDS FOR THE UTILITY READERS 
C     LVLUNT  : FORTRAN UNIT NUMBER FOR LVLFIL
C     OUTUNT  : FORTRAN UNIT NUMBER DOCUMENTING U3DARY OPERATIONS.
C
C    Outputs:
C
C     A       : ARRAY FOR CELL-BY-CELL TERMS
C
C-------------------------------------------------------------------------
      SUBROUTINE U3DARY(
     O                  A,
     I                  ARRTXT,
     I                  II,JJ,KK,
     I                  ARRFIL,ARRUNT,CTLFIL,CTLUNT,OUTUNT,
     I                  MODPER,MODSTP,
     E                  *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      CHARACTER*(*) ARRFIL,CTLFIL
      CHARACTER*16 ARRTXT
      DIMENSION A(JJ,II,KK)
C
      INTEGER 
     &       OUTUNT,
     &       ARRUNT,
     &       CTLUNT,
     &       MODPER,
     &       MODSTP
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*80 CNTRL
      CHARACTER*80 FNAME
      CHARACTER*24 ANAME
      CHARACTER*20 FMTIN
      CHARACTER*16 TEXT,BUDTXT
C
      INTEGER 
     &       ARRFLG,OPNFLG
C
      DATA NUNOPN/99/
      DATA BUDTXT /'MISCELLANEO TERM'/
C
      ANAME=ARRTXT
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''        MODTOOLS UTILITY PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 24MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                U3DARY READER MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
CO----Open Control file CTLFIL
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (CTLUNT,CTLFIL,NSTAT,ITALK,*9999)
C
C1------READ ARRAY CONTROL RECORD AS CHARACTER DATA. 
C
      READ(CTLUNT,'(A)',ERR=9990,END=9991) CNTRL
C
C2------LOOK FOR ALPHABETIC WORD THAT INDICATES THAT THE RECORD IS FREE
C2------FORMAT.  SET A FLAG SPECIFYING IF FREE FORMAT OR FIXED FORMAT. 
C 
      OPNFLG=0
      ICLOSE=0
      IFREE=1
      ICOL=1
      CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,OUTUNT,CTLUNT)
      IF(CNTRL(ISTART:ISTOP).EQ.'CONSTANT') THEN 
         OPNFLG=1
         LOCAT=0
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'INTERNAL') THEN 
         OPNFLG=1
         LOCAT=CTLUNT
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'EXTERNAL') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,LOCAT,R,OUTUNT,CTLUNT)
      ELSE IF(CNTRL(ISTART:ISTOP).EQ.'OPEN/CLOSE') THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,0,N,R,OUTUNT,CTLUNT)
         FNAME=CNTRL(ISTART:ISTOP)
         LOCAT=NUNOPN
         WRITE(OUTUNT,15) LOCAT,FNAME
   15    FORMAT(1X,/1X,'OPENING FILE ON UNIT',I4,':',/1X,A)
         ICLOSE=1
      ELSE
C
C2A-----DID NOT FIND A RECOGNIZED WORD, SO NOT USING FREE FORMAT.
C2A-----READ THE CONTROL RECORD THE ORIGINAL WAY.
         IFREE=0
         READ(CNTRL,1,ERR=9992,END=9993) LOCAT,CNSTNT,FMTIN,IPRN
    1    FORMAT(I10,F10.0,A20,I10)
      END IF
C
C3------FOR FREE FORMAT CONTROL RECORD, READ REMAINING FIELDS.
      IF(IFREE.NE.0) THEN
         CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,3,N,CNSTNT,OUTUNT,CTLUNT)
         IF(LOCAT.NE.0) THEN
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,1,N,R,OUTUNT,CTLUNT)
            FMTIN=CNTRL(ISTART:ISTOP)
            IF(ICLOSE.NE.0) THEN 
               OPNFLG=1
               IF(FMTIN.EQ.'(BINARY)') THEN
                  OPEN(UNIT=LOCAT,FILE=FNAME,FORM='UNFORMATTED')
               ELSE
                  OPEN(UNIT=LOCAT,FILE=FNAME)
               END IF
            END IF
            IF(LOCAT.GT.0 .AND. FMTIN.EQ.'(BINARY)') LOCAT=-LOCAT
            CALL URWORD(CNTRL,ICOL,ISTART,ISTOP,2,IPRN,R,OUTUNT,CTLUNT)
         END IF
      END IF
C 
C3------Open array file if needed 
C 
      IF(OPNFLG.LT.1 .AND. ABS(LOCAT).NE.ABS(CTLUNT)) THEN 
         OPNFLG=1 
         NSTAT=1 
         ITALK=1 
       IF(LOCAT.LE.0) ARRUNT=LOCAT 
         CALL OPEN_FILE (ARRUNT,ARRFIL,NSTAT,ITALK,*9999) 
      ENDIF 
C
C4------USE LOCAT TO SEE WHERE ARRAY VALUES COME FROM. 
C
      IF(LOCAT) 200,50,90
C
C4A------IF LOCAT=0 THEN SET ALL ARRAY VALUES EQUAL TO CNSTNT. RETURN
   50 DO 80 K=1,KK
      DO 80 I=1,II
      DO 80 J=1,JJ
   80    A(J,I,K)=CNSTNT
      WRITE(OUTUNT,2) ARRTXT,CNSTNT,KK
    2 FORMAT(1H0,52X,A,' =',G15.7,' FOR ',I10,' LAYERS')
      GO TO 1000
C
C4B------IF LOCAT>0 THEN READ FORMATTED RECORDS USING FORMAT FMTIN.
   90 WRITE(OUTUNT,4) ARRTXT,KK,LOCAT,FMTIN
    4 FORMAT(1H0,///30X,A,' FOR ',I10,' LAYERS WILL BE READ ON UNIT',
     1       I10,' USING FORMAT: ',A20,/30X,96('-'))
      DO 100 K=1,KK
      DO 100 I=1,II
       IF(FMTIN.EQ.'(FREE)') THEN
          READ(LOCAT,*,ERR=9994,END=9995) (A(J,I,K), J=1,JJ)
       ELSE
          READ (LOCAT,FMTIN,ERR=9994,END=9995) (A(J,I,K), J=1,JJ)
       END IF
  100 CONTINUE
      GO TO 300
C
C4C------LOCAT<0 THEN READ UNFORMATTED RECORD CONTAINING ARRAY VALUES
  200 WRITE(OUTUNT,201) ARRTXT,KK,LOCAT
  201 FORMAT(1H0,///30X,A,' FOR ',I10,' LAYERS',
     1       ' WILL BE READ UNFORMATTED ON UNIT',I10,
     2       /30X,73('-'))
C
C5------READ UNFORMATTED RECORDS
C
      ARRFLG=0
      TEXT=ARRTXT
      CALL READ_UBUDSV (
     I                  ARRFIL,
     I                  TEXT,
     I                  MODPER,
     I                  MODSTP,
     O                  A(1,1,1),
     E                  ARRFLG)
C
       IF(ARRFLG.EQ.KK) THEN
          WRITE(*,'(
     &              ''      .....loaded cell_by_cell terms '')')
          WRITE(OUTUNT,'(
     &              ''      .....loaded cell_by_cell terms '')')
          WRITE(OUTUNT,('(1X,///11X,A,'' FOR LAYERS '',I4,/,
     &                       1X,'' READING BINARY ON UNIT '',I4)'))
     &             ARRTXT,KK,ARRUNT
       ELSE IF(ARRFLG.LT.KK) THEN
          WRITE(*,'(
     &              ''      ....Warning:Did not find cell_by_cell '',
     &              A,'' terms for every layer'')')
     &              ARRTXT
          WRITE(*,'(
     &              ''      .        or for proper stress '',
     &              ''period or time step'')')
          WRITE(*,'(
     &              ''      .  in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....Warning:Did not find cell_by_cell '',
     &              A,'' terms for every layer'')')
     &              ARRTXT
          WRITE(OUTUNT,'(
     &              ''      .        or for proper stress '',
     &              ''period or time step'')')
          WRITE(OUTUNT,'(
     &              ''      .  in file '',A)')
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          GO TO 9998
       ELSE
          WRITE(*,'(
     &              ''      ....Did not find cell_by_cell '',
     &              A,'' terms in file '',A)')
     &              ARRTXT,
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....Did not find cell_by_cell '',
     &              A,'' terms in file '',A)')
     &              ARRTXT,
     &              ARRFIL (1:INDEX(ARRFIL,' ')-1)
          GO TO 9998
       ENDIF
C
C5C------Close
C
      CALL CLOSE_FILE(CTLFIL,*9999)
      GO TO 902
C
C6------IF CNSTNT NOT ZERO THEN MULTIPLY ARRAY VALUES BY CNSTNT.
  300 IF(CNSTNT.EQ.0.) GO TO 320
      DO 310 K=1,KK
      DO 310 I=1,II
      DO 310 J=1,JJ
      A(J,I,K)=A(J,I,K)*CNSTNT
  310 CONTINUE
C
C7------IF PRINT CODE (IPRN) =>0 THEN PRINT ARRAY VALUES.
  320 IF(IPRN.LT.0) RETURN
      DO 400 K=1,KK
         KKK=K
         CALL ULAPRW(A(1,1,KKK),ARRTXT,0,0,JJ,II,KKK,IPRN,OUTUNT)
  400 CONTINUE
C
C9------Check data flag
C
 902  IF(ARRFLG.LE.0) THEN
         WRITE(*,'(
     &            ''      .Did not find cell_by_cell '',
     &            ''terms in file '',A)')
     &            ARRFIL (1:INDEX(ARRFIL,' ')-1)
         WRITE(OUTUNT,'(
     &            ''      .Did not find cell_by_cell  '',
     &            ''terms in file '',A)')
     &            ARRFIL (1:INDEX(ARRFIL,' ')-1)
         GO TO 9999
      ELSE
          WRITE(*,'(''      ..Found header information for '',A)') 
     &         TEXT 
          WRITE(*,'(''      ...for layer '',I15)') 
     &         ILAY 
          WRITE(*,'(''      ...for stress period '',I15)') 
     &         MODPER 
          WRITE(*,'(''      ...for time step     '',I15)') 
     &         MODSTP 
      ENDIF
C
        GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,
     &          '' Unable to read control line for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read control line for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,
     &          '' End-of-file; Unable to read control '',
     &          ''line from file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read control '',
     &          ''line from file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9992 WRITE(*,'(//,
     &          '' Unable to read parameters from control line '',
     &          ''from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read parameters from control line '',
     &          ''from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9993 WRITE(*,'(//,
     &          '' End-of-file; Unable to read parameters from '',
     &          ''control line from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read parameters from '',
     &          ''control line from array control file '',A)') 
     &          CTLFIL (1:INDEX(CTLFIL,' ')-1)
      GO TO 9998
 9994 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read values for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      GO TO 9998
 9995 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      GO TO 9998
 9996 WRITE(*,'(//,
     &          '' Unable to read header for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header for '',
     &          ''cell-by-cell terms from file '',A)') 
     &          ARRFIL (1:INDEX(ARRFIL,' ')-1)
 9998 CALL CLOSE_FILE (ARRFIL,*9999)
      CALL CLOSE_FILE (CTLFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of U3dary_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of U3dary_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 U3dary version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 U3DARY 13NOVEMBER1997
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user specified file for cell_by_cell terms output by
C      the U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C      This program can read 3D data that is formatted in the same manner as 
C      output the UBUDSV module of MODFLOW or that is formatted as text
C      in a gridded manner.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C       A IS ARRAY TO INPUT
C       TEXT IS 16 SEARCH STRING FOR A
C       II IS NO. OF ROWS
C       JJ IS NO. OF COLS
C       KK IS LAYER NO.
C       IN IS INPUT UNIT
C       IOUT IS OUTPUT UNIT
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING CONRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     CONFIL  : FILE CONTAINING THE CELL_BY_CELL TERMS
C     CONUNT  : FORTRAN UNIT NUMBER CONFIL FILE
C     LVLFIL  : FILE CONTAINING THE RECORDS FOR THE UTILITY READERS 
C     LVLUNT  : FORTRAN UNIT NUMBER FOR LVLFIL
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C
C    Outputs:
C
C     CONARY  : ARRAY FOR CELL-BY-CELL FLOW, BUDGET, HEAD TERMS
C
C-------------------------------------------------------------------------
      SUBROUTINE ITEM_FILES (
     I                       ITMFIL,ITMUNT,
     I                       ITMSTR,ITMFIN,
     I                       MAXITM,
     I                       DECIMLS,
     I                       ITEMS,
     I                       OUTPUTS,
     I                       TYPES,
     I                       WIDTHS,
     E                       *)
C-----------------------------------------------------------------------
C
C---VERSION 4.0 ITEM_FILES 24SEPTEMBER1999
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Builds item definitions template file.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     DECIML  : NUMBER OF DECIMAL PLACES
C             => -1 NO DECIMAL PLACES
C             => ^= -1 VAILD DECIMAL PLACES
C     DECIMLS : INTEGER ARRAY HOLDING NUMBER OF DECIMAL PLACES
C     ITEMS   : CHARACTER ARRAY HOLDING POLYGON ITEM NAMES
C     ITMFIL  : ITEM FILE NAMES
C     ITMUNT  : FORTRAN UNIT NUMBER FOR ITEM FILE
C     MAXITM  : MAXIMUM NUMBER OF ATTRIBUTES IN ARC/INFO FILE LABFIL
C     MXITEM  : MAXIMUM NUMBER OF SETS OF ATTRIBUTES IN ARC/INFO FILE LABFIL
C     OUTPUT  : ITEM OUTPUT WIDTH
C     OUTPUTS : INTEGER ARRAY HOLDING ITEM OUTPUT WIDTH
C     TYPE    : ITEM TYPE NUMBER
C             => 1 D DATE
C             => 2 C CHARACTER
C             => 3 I INTEGER
C             => 4 N NUMBERIC
C             => 5 B BINARY
C             => 6 F FLOATING POINT
C     TYPES   : INTEGER ARRAY HOLDING ITEM TYPE NUMBER
C     WIDTH   : ITEM WIDTH
C     WIDTHS  : INTEGER ARRAY HOLDING ITEM WIDTH
C
C    Outputs:
C
C
C     Program:
C
C     ITALK   : FLAG FOR INTERACTIVE DIALOG
C             => 1 NON-INTERACTIVE DIALOG
C             => 0 INTERACTIVE DIALOG
C     NSTAT   : STATUS OF FILE
C             => 1 OLD FILE
C             => 2 NEW FILE
C             => 3 SCRATCH FILE (DELETED AUTOMATICALLY WHEN RUN ENDS)
C             => 4 UNDETERMINED STATUS (MAY OR MAY NOT EXIST)
C                   IF IT DOES NOT EXIST, IT IS CREATED BY OPEN STATEMENT
C                   IF IT DOES EXIST, IT IS OPENED AS 'OLD' FILE
C
C .. History:
C
C      Leonard L. Orzol      02/10/93      Original Coding.
C                                          USGS WRD Portland, Or
C                                          Phone: 503-251-3270
C
C-------------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C::::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::::::::
C
      CHARACTER*(*) ITMFIL
      CHARACTER*16 ITEM,ITEMS(MAXITM)
      CHARACTER*1  TYPITM,TYPES(MAXITM)
C
      INTEGER 
     &        ITMUNT,
     &        DECIML,DECIMLS(MAXITM),
     &        OUTPUT,OUTPUTS(MAXITM),
     &        WIDTH,WIDTHS(MAXITM)
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  :::::::::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
C1----Delete and create files for attribute definitions, attribute value, and
C     coverage arc data and tolerance/dangle values
C
      NSTAT=2
      ITALK=1
C
C1A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      CALL DELETE_FILE (ITMFIL,*9999)
      CALL OPEN_FILE (ITMUNT,ITMFIL,NSTAT,ITALK,*9999)
C
C1B----Write attribute definitions
C
      DO 100 K=ITMSTR,ITMFIN
         DECIML=DECIMLS(K)
         ITEM=ITEMS(K)
         OUTPUT=OUTPUTS(K)
         TYPITM=TYPES(K)
         WIDTH=WIDTHS(K)
         WRITE(ITMUNT,'(A'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                       ITEM (1:INDEX(ITEM,' ')-1),
     &                                       WIDTH,
     &                                       OUTPUT,
     &                                       TYPITM,
     &                                       DECIML
  100 CONTINUE
C
C1C----Write attribute definitions
C
      CALL CLOSE_FILE (ITMFIL,*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &    '(//,'' Abnormal termination of Item_Files_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
      SUBROUTINE LOCAL_COORDINATES (
     I                              NCOL,NROW,
     I                              DELX,
     I                              COL,ROW,
     I                              XGC,YGC,YORIGN,
     O                              XLC,YLC)
C-----------------------------------------------------------------------
C
C---VERSION 6.0 LOCAL_COORINDATES 12MAY1998
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Calculates local x-,y-coordinates inside a mode-grid cell.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     COL     : COLUMN NUMBER OF PARTICLE IN GRID
C     DELX    : REAL ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NROW    : NUMBER OF ROWS IN GRID
C     ROW     : ROW NUMBER OF PARTICLE IN GRID
C     XGC     : GLOBAL X-DIRECTION COORDINATE
C     YGC     : GLOBAL Y-DIRECTION COORDINATE
C     YORIGN  : MODPATH Y ORIGIN COORDINATE
C
C    Outputs:
C
C     XLC     : LOCAL X-DIRECTION COORDINATE
C     YLC     : LOCAL Y-DIRECTION COORDINATE
C
C     Program:
C
C
C .. History:
C
C      Leonard L. Orzol      02/10/93      Original Coding.
C                                          USGS WRD Portland, Or
C                                          Phone: 503-251-3270
C
C-------------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C::::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::::::::
C
      DIMENSION 
     &          DELX(NCOL*NROW)
C
      REAL 
     &       XGC,XLC,
     &       YGC,YLC,
     &       YORIGN
C
      INTEGER 
     &        COL,ROW
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  :::::::::::::::::::::::::::::::::
C
      REAL 
     &     XLEFT,
     &     XWIDTH,
     &     YTOP,
     &     YWIDTH
C
C     ------------------------------------------------------------------
C
C1----Find local x-coordinate
C
      XLEFT=0.0
      DO 100 N=1,COL-1
         XLEFT=XLEFT+DELX(N)
  100 CONTINUE
C
      XWIDTH=DELX(COL)
       XLC=(XGC-XLEFT)/XWIDTH
C
C2----Find local y-coordinate
C
      YTOP=0.0
      DO 200 N=NCOL+1,NCOL+ROW
         YTOP=YTOP+DELX(N)
  200 CONTINUE
C
      YWIDTH=DELX(NCOL+ROW)
       YLC=(YGC-YORIGN+YTOP)/YWIDTH
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
      SUBROUTINE CHRSTR (
     I                   STRING,
     I                   LENCHR,
     O                   NUMCHR
     &                  )
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C-----------------------------------------------------------------------
C
C---VERSION 2.0 CHRSTR 19FEBRUARY1991
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Checks whether this string fits within numchr range and if not
C      truncates so it does.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C   Inputs:
C
C     LENCHR  : NUMBER OF CHARACTER WANTED IN RETURNED STRING
C     STRING  : CHARACTER STRING TO EVALUATE
C
C   Inputs/Outputs:
C
C     NUMCHR  : STARTING POSITION IN RETURNED STRING
C
C   Outputs: None
C
C .. History:
C
C      Leonard L. Orzol      19/02/91      Original Coding.
C                                          USGS WRD Portland, Or
C                                          Phone: 503-251-3270
C
C-------------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
      CHARACTER*(*) STRING
C
      INTEGER LENSTR,NUMCHR
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                      TEST STRING LENGTH
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
C
C1----Set global variables
C
        LENSTR=INDEX(STRING,' ')-1
      IF(LENSTR.LE.LENCHR) THEN
         NUMCHR=1
         RETURN
      ELSE
         NUMCHR=LENSTR-LENCHR+1
      ENDIF
C
C2----RETURN
C
      RETURN
      END
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
      SUBROUTINE ROTATE (
     I                   DEG,
     B                   X,Y)
C-----------------------------------------------------------------------
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
c
c  Rotates (x,y) by deg degrees around (0,0) and return in xr,yr
c
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN, with ARC/INFO subroutine call version 5.0
C
C
C .. History:
C
C      Leonard L. Orzol      06/06/90      Modified Coding to match with
C                                           use in Modpath_Arc.
C                                          USGS WRD Portland, Or
C                                          Phone: 503-251-3270
C
C      Dan Winkless          04/88         Updated Coding.
C                                          USGS New Mexico
C
C      Dan Winkless          12/87         Original Coding.
C                                          USGS New Mexico
C
C      Mike Kernodle         198?          Base Coding                
C                                          USGS New Mexico
C-------------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
      real x,y
      real*8 xdummy,ydummy
      real   deg
c
      real*8 rads
      real*8 h
      real*8 ang
      intrinsic dsin,dcos
      intrinsic dsqrt
      intrinsic dasin,dacos
      intrinsic datan2
C**********************************************************************C
C
C1----Set local variables
C
      xdummy=x
      ydummy=y
      rads=deg * .1745329d-1
      h = dsqrt (xdummy**2 + ydummy**2)
c
c  check for point rotating about itself.  do not rotate if so.
c
      if(h.eq.0.0)then
         x = 0.0
         y = 0.0
         return
      end if
c
      ang = datan2(ydummy,xdummy) + rads
      xdummy = dcos(ang) * h
      ydummy = dsin(ang) * h
c
      x=xdummy
      y=ydummy
c
c  that was easy.
c
      return
      end
C**********************************************************************C
