C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Strall version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 STRALL 02MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for steam leakage terms of 
C      Streamflow-Routing Package (Prudic, 1989). These terms are output
C      from this package by segement and reach. Currently these data are
C      printed into a listing file if specified by the U.S. Geological Survey 
C      Modular Three-Dimensional Finite-Differences Ground-Water Flow Model
C      MODFLOW (McDonald and Harbaugh. 
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCBUDT  : STARTING ARRAY INDEX FOR CELL_BY_CELL BUDGET TERMS
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE STRALL (
     I                   OUTUNT,
     I                   LENX,
     I                   STRFIL,STRUNT,
     B                   ISUM,
     O                   MAXSTM,LCBUDT,
     E                   *)
C-----------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      CHARACTER*(*) STRFIL
C
      INTEGER
     &        OUTUNT,STRUNT,
     &        LENX,
     &        MAXSTM,
     &        ISUM,
     &        LCBUDT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*128 BUFFER
C
      INTEGER
     &        STRFLG
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''         STREAMFLOW_ROUTING TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 02MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Text format in STRFIL file
C
      IF(STRUNT.GT.0) THEN
          WRITE(*,'(
     &              ''      .Reading text formatted '',
     &              ''Streamflow-Routing terms '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              STRFIL (1:INDEX(STRFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 STRUNT
          WRITE(OUTUNT,'(
     &              ''      .Reading text formatted '',
     &              ''Streamflow-Routing terms '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              STRFIL (1:INDEX(STRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 STRUNT
C
C1C---Error no file containing Streamflow-Routing terms
C
      ELSE
         WRITE(*,'(
     &             ''      .No file containing Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No file containing Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         GO TO 9999
      ENDIF
C
C2----Streamflow-Routing
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (STRUNT,STRFIL,NSTAT,ITALK,*9999)
C
C2A---Loop to read
C
      STRFLG=-1
 302   READ(STRUNT,'(A)',ERR=9991,END=304) BUFFER
C
      IF(INDEX(BUFFER,'STREAM PACKAGE').GT.0) THEN
         STRFLG=1
      ENDIF
C
      IF(INDEX(BUFFER,'STREAM NODES').GT.0 .AND. STRFLG.EQ.1) THEN
         BUFFER = BUFFER(INDEX(BUFFER,'MAXIMUM OF')+10:LEN(BUFFER))
         BUFFER = BUFFER(1:INDEX(BUFFER,'STREAM NODES'))
         READ(BUFFER,*) 
     &                  MAXSTM
         GO TO 304
      ENDIF
C
      GO TO 302
C
C3C---Close text formatted file STRFIL
C
 304  CALL CLOSE_FILE (STRFIL,*9999)
C
C2----Allocate space for arrays
C
      LCBUDT=ISUM
      ISP=MAXSTM*11
      ISUM=ISUM+ISP
      ISUM1=ISUM-1
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF ACTIVE STREAM NODES          '',
     &             10X,I15,'' MAXSTM'')')
     &          MAXSTM
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR STREAM TERMS          '',
     &             10X,I15,'' LCBUDT'')')
     &          LCBUDT
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR STREAMFLOW-ROUTING TERMS PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I8)')
     &          ISUM1,LENX
C
C4----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Strall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Strall_Subroutine'')')
         RETURN 1
      ENDIF
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9991 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''Streamflow-Routing terms from file '',A)') 
     &          STRFIL (1:INDEX(STRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''Streamflow-Routing terms from file '',A)') 
     &          STRFIL (1:INDEX(STRFIL,' ')-1)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Strall_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Strall_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Strrp1 version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 STRRP1 17DECEMBER2001
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user defined files for segment and reach leakage terms.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING HEDRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     STRFIL  : FILE CONTAINING CELL_BY_CELL FLOW TERMS FILE
C     STRUNT  : FORTRAN UNIT NUMBER CELL_BY_CELL FLOW TERMS FILE
C     CTLFIL  : FILE CONTAINING THE CONTROL LINES
C     CTLUNT  : FORTRAN UNIT NUMBER FOR U3DREL CONTROL LINES
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C
C    Outputs:
C
C     BUDARY  : ARRAY FOR SEGMENT AND REACH LEAKAGE TERMS
C
C     Program:
C
C     ANAME   : CHARACTER ARRAY (6,8) CONTAINING ARRAY NAMES
C     K       : LOOP INDEX
C     KK      : PASSED LAYER NUMBER
C     LOC     : STARTING INDEX FOR ARRAY
C
C-------------------------------------------------------------------------
      SUBROUTINE STRRP1 (
     I                   OUTFIL,OUTUNT,
     I                   STRFIL,STRUNT,
     I                   MAXPMR,
     I                   MAXSTM,
     I                   COVPMR,
     O                   STRARY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION COVPMR(MAXPMR),
     &          STRARY(11,MAXSTM)
C
      CHARACTER*(*) OUTFIL,STRFIL
C
      INTEGER OUTUNT,STRUNT,CTLUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*128 BUFFER
      CHARACTER*16 BUDTXT,TEXT
      CHARACTER*24 ANAME
C
      INTEGER ICOL,IROW,ILAY,LAYER,KK,STRFLG
C
      DATA BUDTXT /'STR LEAKAGE TERM'/
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                    Cell_By_Cell Flow Array
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''        STREAMFLOW_ROUTING TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 03MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
CL----Set local variables
C
      MODFLG=INT(COVPMR(1))
      MODFMT=MODFLG-3
      MODFMT=INT(COVPMR(12))/100
      IDCNTR=MOD(INT(COVPMR(12)),100)
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
      FLWCLL=COVPMR(16) 
      LOC=0
C
      IF(NPER.EQ.0) THEN
         MODPER=1
         MODSTP=1
         WRITE(*,'(
     &       ''      .Reading steady-state Streamflow-Routing terms'')')
         WRITE(OUTUNT,'(
     &       ''      .Reading steady-state Streamflow-Routing terms'')')
      ELSE
         WRITE(*,'(
     &          ''      .Reading transient Streamflow-Routing terms'')')
         WRITE(*,'(
     &          ''      .. for Modflow stress period '',I10)')
     &                                                         MODPER
         WRITE(*,'(
     &          ''      .. for Modflow time step '',I10)')
     &                                                         MODSTP
         WRITE(OUTUNT,'(
     &          ''      .Reading transient Streamflow-Routing terms'')')
         WRITE(OUTUNT,'(
     &          ''      .. for Modflow stress period '',I10)')
     &                                                         MODPER
         WRITE(OUTUNT,'(
     &          ''      .. for Modflow time step '',I10)')
     &                                                         MODSTP
      ENDIF
C
C1---Array format in STRFIL file
C
C
C1A---Binary format in STRFIL file
C
      IF(STRUNT.LT.0) THEN
          WRITE(*,'(
     &              ''      .Reading binary formatted '',
     &              ''Streamflow-Routing terms '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              STRFIL (1:INDEX(STRFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 STRUNT 
          WRITE(OUTUNT,'(
     &              ''      .Reading binary formatted '',
     &              ''Streamflow-Routing terms '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              STRFIL (1:INDEX(STRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 STRUNT 
C
C1B---Text format in STRFIL file
C
      ELSE IF(STRUNT.GT.0) THEN
          WRITE(*,'(
     &              ''      .Reading text formatted '',
     &              ''Streamflow-Routing terms '')')
          WRITE(*,'(
     &              ''      ..recorded in file '',A)')
     &              STRFIL (1:INDEX(STRFIL,' ')-1)
          WRITE(*,'(
     &              ''      ..on unit '',I10)')
     &                                                 STRUNT
          WRITE(OUTUNT,'(
     &              ''      .Reading text formatted '',
     &              ''Streamflow-Routing terms '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              STRFIL (1:INDEX(STRFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..on unit '',I10)')
     &                                                 STRUNT
C
C1C---Error no file containing Streamflow-Routing terms
C
      ELSE
         WRITE(*,'(
     &             ''      .No file containing Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No file containing miscellaneous '',
     &             ''terms specified in MODTOOLS name file'')')
         GO TO 9999
      ENDIF
C
C3----Set array
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (STRUNT,STRFIL,NSTAT,ITALK,*9999)
C
C3A---Loop to read
C
      STRFLG=-1
       NUMLIN=0
        NUMSTM=0
 302     READ(STRUNT,'(A)',ERR=9991,END=304) BUFFER
C
C      WRITE(OUTUNT,'(A)') BUFFER
C
      IF(INDEX(BUFFER,'STREAM NODES').GT.0) THEN
         STRFLG=1
      ENDIF
C
      IF(INDEX(BUFFER,'STREAM').GT.0 .AND.
     &   INDEX(BUFFER,'FLOW INTO').GT.0 .AND.
     &   INDEX(BUFFER,'FLOW INTO').GT.0) THEN
         STRFLG=2
      ENDIF
C
      IF(INDEX(BUFFER,'NUMBER').GT.0 .AND.
     &   INDEX(BUFFER,'STREAM REACH').GT.0 .AND.
     &   INDEX(BUFFER,'AQUIFER').GT.0) THEN
         STRFLG=3
      ENDIF
C
      IF(STRFLG.EQ.3) THEN
         NUMLIN=NUMLIN+1
      ENDIF
C
      IF(STRFLG.EQ.3 .AND. NUMLIN.GT.3) THEN
       IF(LEN_TRIM(BUFFER).LE.0 .OR. BUFFER(1:1).EQ.'1') GO TO 304
         NUMSTM=NUMSTM+1
C         WRITE(OUTUNT,'(A)') BUFFER
C         WRITE(OUTUNT,'('' length = '',I10)') LEN_TRIM(BUFFER)
        IF(NUMSTM.GT.MAXSTM) GO TO 304
          READ(BUFFER,*,ERR=9991)
     &                  STRARY(1,NUMSTM),
     &                  STRARY(2,NUMSTM),
     &                  STRARY(3,NUMSTM),
     &                  STRARY(4,NUMSTM),
     &                  STRARY(5,NUMSTM),
     &                  STRARY(6,NUMSTM),
     &                  STRARY(7,NUMSTM),
     &                  STRARY(8,NUMSTM)
      ENDIF
C
      GO TO 302
C
C3C---Close text formatted file STRFIL
C
 304  CALL CLOSE_FILE (STRFIL,*9999)
C
C4----Check streamflow information
C
      IF(NUMSTM.GT.MAXSTM) THEN
         WRITE(*,'(
     &             ''      .Too many records for Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         WRITE(*,'(
     &             ''      ..Streamflow-Routing '',I10)') MAXSTM
         WRITE(*,'(
     &             ''      ..Found records '',I10)') NUMSTM
         WRITE(OUTUNT,'(
     &             ''      .Too many records for Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         WRITE(OUTUNT,'(
     &             ''      ..Streamflow-Routing '',I10)') MAXSTM
         WRITE(OUTUNT,'(
     &             ''      ..Found records '',I10)') NUMSTM
         GO TO 9999
      ENDIF
      IF(STRFLG.LT.3) THEN
         WRITE(*,'(
     &             ''      .No records for Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No records for Streamflow-Routing '',
     &             ''terms specified in MODTOOLS name file'')')
         GO TO 9999
      ENDIF
C
       WRITE(*,'(''      .'',I10,
     &           '' records for Streamflow-Routing '',
     &           ''terms specified in MODTOOLS name file'')')
     &                NUMSTM
       WRITE(OUTUNT,'(''      .'',I10,
     &           '' records for Streamflow-Routing '',
     &           ''terms specified in MODTOOLS name file'')')
     &                NUMSTM
        GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9991 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''segement and reach terms from file '',A)') 
     &          STRFIL (1:INDEX(STRFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''segement and reach terms from file '',A)') 
     &          STRFIL (1:INDEX(STRFIL,' ')-1)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Strrp1_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Strrp1_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthstr version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 PTHSTR 05MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a Info file STRFIL using the miscellaneous gridded
C      terms that could be input to the different packages of the 
C      U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C      IVIEW
C           => 0 Mapview --> all layers
C                 One info files of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 1 Vertical row profile [appended to row coverage]
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) CELL_BY_CELL BUDGET TERMS
C
C           => 2 Vertical column profile
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 3 Vertical particle profile
C                 Cell-by-cell budget terms attached to cell
C              => (n,1) CELL POLYGON NUMBER
C              => (n,2+NLAY) LAYER_xx = CELL_BY_CELL BUDGET TERMS
C
C           => 4 Mapview --> user specified layer
C                 One info file of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) LAYER_xx = CELL_BY_CELL BUDGET TERMS FOR CHOSEN LAYER
C
C           => 5 Mapview --> user specified ibound value
C                 One info file of cell-by-cell budget terms
C                 (this ibound value can be used to differentiate model
C                  layers into geological units.)
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL BUDGET TERMS 
C                       (these terms are computed as a summation of the 
C                         cell-by-cell budget terms for each layer which
C                         match the chosen ibound value.)
C           => 6 Mapview --> uppermost active cell
C                 One info file of cell-by-cell budget terms
C              => (n,1) CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,2) CELL ROW
C              => (n,3) CELL COLUMN
C              => (n,4) CELL LAYER
C              => (n,5) CELL IBOUND VALUE
C              => (n,6) CELL_BY_CELL BUDGET TERMS
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     STRFIL  : COVERAGE ROOT NAME
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR)
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION (NDELX)
C     STRARY  : ARRAY FOR LAYER/UNIT CELL_BY_CELL BUDGETS
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY)
C     IOUTPT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     IRWCL   : WHERE CROSS-SECTION TRANVERSE IS TAKEN
C     IVIEW   : FLAG (ORIENTATION OF COVERAGE)
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NDELX)
C     MAXATB  : MAXIMUM NUMBER OF ATTRIBUTES FOR EACH POLYGON
C                IN THE COVERAGE PAT FILE.
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     MAXPTS  : MAXIMUM NUMBER OF VERTICES IN A SINGLE ARC
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     ZTOP    : ARRAY FOR LAYER/UNIT TOP ELEVATIONS
C     ZBOT    : ARRAY FOR LAYER/UNIT BOTTOM ELEVATIONS
C
C    Outputs:
C
C     PATLAB  : POLYGON ATTRIBUTE ARRAY (MAXPTS,MAXATB)
C             => (n,1) CELL POLYGON NUMBER
C             => (n,2) CELL COLUMN
C             => (n,3) CELL ROW
C             => (n,4) CELL LAYER
C             => (n,5) MODEL IBOUND VALUE
C             => (n,6) CELL TOP ELEVATION
C             => (n,7) CELL BOTTOM ELEVATION
C             => (n,8) to (n,20) Unused
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PTHSTR (
     I                   OUTUNT,
     I                   STRFIL,
     I                   MAXPMR,
     I                   MAXSTM,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,LAYCBD,
     I                   LRWCL,
     I                   IBOUND,
     I                   STRARY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     1          COVPMR(MAXPMR),
     1          LAYCBD(NLAY),
     2          LRWCL(2,NCOL*NROW),
     3          IBOUND(NCOL,NROW,NLAY),
     3          STRARY(11,MAXSTM)
C
      CHARACTER*(*) STRFIL
      INTEGER OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER (MAXFIL=3,
     &           MAXITM=11)
C
      REAL BUDIBD
C
      CHARACTER*128 LABFIL,FMTSTR,BUFFER
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL
      CHARACTER*16 COVTYP,ITEM,ITEMS(MAXITM),ARCCMD,ACTION,
     &             LAYITM,STRITM,RELITM,JONTYP
      CHARACTER*10 CHRLAY,CHRWCL,LAYFMT,ITMFMT
      CHARACTER*6 MODTXT
      CHARACTER*1 TYPITM,TYPES(MAXITM)
C
      INTEGER DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),ITMSTR,ITMFIN,
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM)
C
      INTEGER 
     &        IRWCL,IVIEW,
     &        IDPART,ICOLOR,TIMSTP,OLDSTP,
     &        STRCHR,STRCOL,STRROW,STRLAY,STRIBD,STRDEL,STRCLL,
     &        PRECOL,PREROW,PRELAY,PREIBD,
     &        FINCHR,FINCOL,FINROW,FINLAY,FINIBD,FINDEL,FINCLL,
     &        ROW,COLUMN,CELL,BADIBD,COVDEX,COL,ROWCOL,SEQNUM,
     &        NCELLS,
     &        NUMCOL,NUMROW,NUMLAY,
     &        IRWCOL,SEGNUM,RCHNUM
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA FILES /'budgetitems',
     &            'budgetitem',
     &            'arccmd'/
      DATA IOUT /91,92,93/
      DATA ITEMS /'SEQNUM','ROWCOL','ROW','COLUMN','LAYER','BOUND',
     &            'SEGMENT','REACH','FLOWIN','LEAKAGE','FLOWOUT'/
      DATA DECIMLS /8*0,3*6/
      DATA TYPES /'I','C',6*'I',3*'F'/
      DATA WIDTHS /8*10,3*8/
      DATA OUTPUTS /8*10,3*16/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''         STREAMFLOW_ROUTING TERMS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               VERSION 6.1 05MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                  GIS OUTPUT MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Set local variables
C
      NCELLS=0
      NUMCON=0
      BADIBD=0
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
      MODFLG=INT(COVPMR(1))
      MODFMT=MODFLG-3
      MODFMT=INT(COVPMR(12))/100
      IDCNTR=MOD(INT(COVPMR(12)),100)
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
C
      IF(NPER.EQ.0) THEN
         MODPER=1
         MODSTP=1
      ENDIF
C
         MODTXT='UNKTRM'
         WRITE(*,'(
     &          ''      .Processing Streamflow-Routing Terms '')')
         WRITE(OUTUNT,'(
     &          ''      .Processing Streamflow-Routing Terms '')')
C
C2----Delete and create files for attribute definitions, attribute value, 
C      and coverage arc data and tolerance/dangle values
C
      NSTAT=2
      ITALK=1
      FILES(2) = STRFIL(1:INDEX(STRFIL,' ')-1)
C
C2A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      DO 200 N=1,MAXFIL-1
         CALL DELETE_FILE (FILES(N),*9999)
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      WIDTHS(1)=LENROW+LENCOL
      OUTPUTS(1)=LENROW+LENCOL
      WIDTHS(2)=LENROW+LENCOL
      OUTPUTS(2)=LENROW+LENCOL
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
      WIDTHS(5)=LENLAY
      OUTPUTS(5)=LENLAY
C
C2C----Write standard attribute definitions
C
      DO 210 K=1,MAXITM
         DECIML=DECIMLS(K)
         ITEM=ITEMS(K)
         OUTPUT=OUTPUTS(K)
         TYPITM=TYPES(K)
         WIDTH=WIDTHS(K)
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                      I2,'',,,,,,,,,,'')')
     &                                    ITEM,
     &                                    WIDTH,
     &                                    OUTPUT,
     &                                    TYPITM,
     &                                    DECIML
 210  CONTINUE
      CALL CLOSE_FILE (FILES(1),*9999)
C
C3----Build cell-by-cell terms
C
      BADIBD=0
      BADCEL=0
      NUMCLL=0
      NCELLS=0
      NUMCON=0
C
C3M---Message
C
      WRITE(*,'(
     &             ''      .Processing segment and reach terms '',
     &             ''for all layers '',I10)')
     &         NLAY
         WRITE(OUTUNT,'(
     &             ''      .Processing segment and reach terms '',
     &             ''for all layers '',I10)')
     &         NLAY
C
C3H----Data header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(50X,
     &               '' SEGMENT AND REACH VALUES '')')
C
      WRITE(OUTUNT,'(10X,125(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &            ''    SEQNUM'',
     &            ''       ROW'',
     &            ''    COLUMN'',
     &            ''     LAYER'',
     &            ''    IBOUND'',
     &            ''   SEGMENT'',
     &            ''     REACH'',
     &            ''     FLOW INTO   '',
     &            ''     FLOW INTO   '',
     &            ''    FLOW OUT OF  '')')
      WRITE(OUTUNT,'(10X,
     &            ''          '',
     &            ''          '',
     &            ''          '',
     &            ''          '',
     &            ''          '',
     &            ''          '',
     &            ''          '',
     &            ''   STREAM REACH  '',
     &            ''      AQUIFER    '',
     &            ''   STREAM REACH  '')')
      WRITE(OUTUNT,'(10X,125(''-''))')
C
C3----Build
C
      DO 300 NUMSTM=1,MAXSTM
         ILAY=INT(STRARY(1,NUMSTM))
         IROW=INT(STRARY(2,NUMSTM))
         ICOL=INT(STRARY(3,NUMSTM))
         SEGNUM=INT(STRARY(4,NUMSTM))
         RCHNUM=INT(STRARY(5,NUMSTM))
         SEQNUM=(IROW-1)*NCOL+ICOL
         NUMCLL=(IROW-1)*NCOL+ICOL
         ROWCOL=IROW*(10**LENCOL)+ICOL
         WRITE(CHRWCL,'(I10)') ROWCOL
         FINCHR=10
         STRCHR=FINCHR-(LENROW+LENCOL-1)
         IBDCEL=IBOUND(ICOL,IROW,ILAY)
       IF(IBDCEL.NE.0) THEN
          NCELLS=NCELLS+1
          WRITE(OUTUNT,'(10X,7I10,3F17.6)')
     &                          SEQNUM,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          SEGNUM,
     &                          RCHNUM,
     &                          STRARY(6,NUMSTM),
     &                          STRARY(7,NUMSTM),
     &                          STRARY(8,NUMSTM)
          WRITE(IOUT(2),'(
     &                          I10,'','',
     &                          A,'','',
     &                          6(I10,'',''),
     &                          3(F16.6,'',''))')
     &                       SEQNUM,
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       ILAY,
     &                       IBDCEL,
     &                       SEGNUM,
     &                       RCHNUM,
     &                       STRARY(6,NUMSTM),
     &                       STRARY(7,NUMSTM),
     &                       STRARY(8,NUMSTM)
       ELSE
          BADIBD=BADIBD+1
          WRITE(OUTUNT,'(10X,7I10,
     &                   ''    ====> IBOUND = 0 <====    '')')
     &                          SEQNUM,
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL,
     &                          SEGNUM,
     &                          RCHNUM
       ENDIF
  300 CONTINUE
      WRITE(OUTUNT,'(10X,125(''-''))')
      WRITE(OUTUNT,'(//)')
C
C3----Close attribute files
C
      CALL CLOSE_FILE (FILES(2),*9999)
C
C8----Messages
C
C
C8A----Message to user
C
      WRITE(*,'(''      .Segment and reach term file '',A,
     &          '' consists of '')')
     &        STRFIL (1:INDEX(STRFIL,' ')-1)
      WRITE(*,'(''      .'',I10,'' segments'')') NCELLS
      WRITE(*,'(''      .'',I10,'' segments were '',
     &          ''ommited (ibound = 0)'')') BADIBD
C
C8B----Message to operational output file
C
      WRITE(OUTUNT,'(''      .Segment and reach term file '',A,
     &          '' consists of '')')
     &        STRFIL (1:INDEX(STRFIL,' ')-1)
      WRITE(OUTUNT,'(''      .'',I10,'' segments'')') NCELLS
      WRITE(OUTUNT,'(''      .'',I10,'' segments were '',
     &          ''ommited (ibound = 0)'')') BADIBD
C
C9----Arc Operations to coverage and attribute files
C
      NSTAT=2
      ITALK=1
      CALL DELETE_FILE (FILES(MAXFIL),*9999)
      CALL OPEN_FILE (IOUT(MAXFIL),FILES(MAXFIL),NSTAT,ITALK,*9999)
C
C9A----Create attribute files and "add from" attributes
C
      ARCCMD='tables'
      TMPFIL=FILES(1) (1:INDEX(FILES(1),' ')-1)
      ADDFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
      WRITE(IOUT(MAXFIL),'(A,2(1X,A))')
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1)
      CALL CLOSE_FILE (FILES(MAXFIL),*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &    '(//,'' Abnormal termination of Pthstr_Subroutine'')')
      WRITE(OUTUNT,
     &    '(//,'' Abnormal termination of Pthstr_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
