C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Timall version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 TIMALL 15MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for time-series point
C      coverages and attributes.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     MAXATB  : MAXIMUM NUMBER OF ATTRIBUTES IN PARTICLE ATTRIBUTE TABLE
C     MAXPTS  : MAXIMUM NUMBER OF PARTICLE POSITIONS
C     OUTUNT    : FORTRAN UNIT NUMBER
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCATRB  : STARTING ARRAY INDEX FOR ATTRIBUTE TABLE (ARCS,LABELS)
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE TIMALL (
     I                   OUTUNT,
     I                   LENX,
     I                   MAXATB,MAXPTS,
     B                   ISUM,
     O                   LCATRB,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        MAXATB,
     &        MAXPTS,
     &        ISUM,
     &        LCATRB
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          TIME-SERIES GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''              VERSION 6.1 15MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Allocate space for arrays
C
      LCATRB=ISUM
      ISUM=ISUM+MAXATB*MAXPTS
      ISUM1=ISUM-1
C
C2----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''MAXIMUM NUMBER OF PARTICLES '',
     &             ''IN TIME-SERIES COVERAGE'')')
     &          MAXPTS
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''MAXIMUM NUMBER OF ATTRIBUTES FOR EACH PARTICLE '',
     &             ''IN TIME-SERIES COVERAGE'')')
     &          MAXATB
C
      ISP=MAXATB*MAXPTS
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR TIME-SERIES POINT PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I8)')
     &          ISUM1,LENX
C
      WRITE(OUTUNT,'(10X,85(''-''))')
C
C3----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         GO TO 9999
      ENDIF
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &      '(//,'' Abnormal termination of Timall_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal termination of Timall_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthtim version 6.1                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 PTHTIM 15MAY2002
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a point coverage from results of the particle
C      tracking program MODPATH using the time-series file TIMESERS.
C
C        Attributes (items) associated with each particle time-series are
C      (these can be selected by the user for different time-seriess or other
C      user defined functions or actions) :
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     TIMFIL  : MODPATH OUTPUT TIME-SERIES PARTICLE FILE NAME
C     TIMUNT  : FORTRAN UNIT NUMBER TO TIME-SERIES PARTICLE FILE
C     PRTCOV  : USER SPECIFIED PARTICLE COVERAGE NAME
C     MAXATB  : MAXIMUM DIMENSION OF THE NUMBER OF PARTICLE ATTRIBUTES
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     MAXPTS  : MAXIMUM DIMENSION OF THE NUMBER OF LABEL (MAXARC*MAXPTS)
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR)
C     IBOUND  : ARRAY FOR IBOUND ARRAY
C
C    Outputs:
C
C     PARTLE  : PARTICLE ATTRIBUTE TABLE
C
C-------------------------------------------------------------------------
      SUBROUTINE PTHTIM (
     I                   OUTUNT,
     I                   TIMFIL,TIMUNT,
     I                   PRTCOV,
     I                   MAXATB,MAXPMR,MAXPTS,
     I                   NLAY,NROW,NCOL,
     I                   DELX,COVPMR,
     I                   IBOUND,
     O                   PARTLE,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C::::::::::::::::::::::::  GLOBAL_VARIABLES  :::::::::::::::::::::::::::
C
C
      DIMENSION 
     &          COVPMR(MAXPMR),
     &          DELX(NCOL*NROW),
     &          PARTLE(MAXPTS,MAXATB),
     &          IBOUND(NCOL,NROW,NLAY)
C
      CHARACTER*(*) PRTCOV,TIMFIL
C
      INTEGER OUTUNT,
     &        TIMUNT
C
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER 
     &          (MAXITM=63,
     &           MXITEM=5)
C
      CHARACTER*300 BUFFER
      CHARACTER*128 COVFIL
      CHARACTER*80 STRFIL,LSTFIL,PREFIL,FINFIL,CUMFIL,
     &             GENFIL,ARCFIL,ADDFIL,JONFIL,TMPFIL
      CHARACTER*16 ARCCMD,ACTION,STRITM,RELITM,JONTYP,
     &             COVTYP,ITEMS(MAXITM)
      CHARACTER*1 TYPES(MAXITM)
C
      REAL
     &     XGC,XLC,XCOORD,
     &     YGC,YLC,YCOORD,
     &     ZGC,ZLC,ZCOORD
C
      INTEGER 
     &        STRUNT,LSTUNT,PREUNT,FINUNT,CUMUNT,
     &        GENUNT,ARCUNT,
     &        DECIMLS(MAXITM),
     &        ITMSTR(MXITEM),ITMFIN(MXITEM),
     &        OUTPUTS(MAXITM),
     &        WIDTHS(MAXITM)
C
      INTEGER
     &        COL,ROW,LAY,BUFDEX,COMPCT,BINARY,
     &        ICOVER,MODFLG,MODFMT,IVIEW,IRWCL,
     &        IDPART,PARTID,TIMSTP,OLDSTP,
     &        STRCOL,STRROW,STRLAY,STRIBD,STRSTP,
     &        LSTCOL,LSTROW,LSTLAY,LSTIBD,
     &        PRECOL,PREROW,PRELAY,PREIBD,PRESTP,
     &        FINCOL,FINROW,FINLAY,FINIBD,FINSTP,
     &        TOTPTS,NUMPTS,NEWPTS,OLDPTS,TOTREC
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA STRFIL,STRUNT /'startitems',98/
      DATA LSTFIL,LSTUNT /'lastitems',97/ 
      DATA PREFIL,PREUNT /'presentitems',96/
      DATA FINFIL,FINUNT /'finalitems',95/
      DATA CUMFIL,CUMUNT /'cumitems',94/
      DATA GENFIL,GENUNT /'points',93/
      DATA ARCFIL,ARCUNT /'arccmd',92/
C
      DATA ITMSTR /1, 14,25,42,59/
      DATA ITMFIN /13,24,41,58,63/
      DATA ITEMS /'PARTID','STRSTP',
     &            'STRROW','STRCOL','STRLAY','STRIBD',
     &            'STRXGC','STRXLC',
     &            'STRYGC','STRYLC',
     &            'STRZGC','STRZLC',
     &            'STRTIM',
     &            'LSTROW','LSTCOL','LSTLAY','LSTIBD',
     &            'LSTXGC','LSTXLC',
     &            'LSTYGC','LSTYLC',
     &            'LSTZGC','LSTZLC',
     &            'LSTTIM',
     &            'PRESTP',
     &            'PREROW','PRECOL','PRELAY','PREIBD',
     &            'PREXGC','PREXLC',
     &            'PREYGC','PREYLC',
     &            'PREZGC','PREZLC',
     &            'PRETIM',
     &            'PRETRV',
     &            'PREXVL','PREYVL','PREZVL','PRESPD',
     &            'PARTID','FINSTP',
     &            'FINROW','FINCOL','FINLAY','FINIBD',
     &            'FINXGC','FINXLC',
     &            'FINYGC','FINYLC',
     &            'FINZGC','FINZLC',
     &            'FINTIM',
     &            'FINXVL','FINYVL','FINZVL','FINSPD',
     &            'CUMXVL','CUMYVL','CUMZVL','CUMSPD','MODSTP'/
C
      DATA DECIMLS / 6*0,  7*3,  4*0,  7*3,  5*0,  8*3,  4*8,  6*0,
     &                7*3,  8*8,0/
      DATA TYPES   / 6*'I',7*'F',4*'I',7*'F',5*'I',8*'F',4*'F',6*'I',
     &                11*'F',4*'F','I'/
      DATA WIDTHS  / 6*10, 7*8,  4*10, 7*8,  5*10, 8*8,  4*8,  6*10,
     &                11*8,  4*8,10/
      DATA OUTPUTS / 6*10, 7*16, 4*10, 7*16, 5*10, 8*16, 4*16, 6*10,
     &                11*16, 4*16,10/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          TIME-SERIES GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''              VERSION 6.1 15MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C0----Time-series file
C
      IF(TIMUNT.LE.0) THEN    
         WRITE(*,'(
     &             ''      .No time-series file listed '',
     &             ''in MODPATH name file'')')
         WRITE(OUTUNT,'(
     &             ''      .No time-series file listed '',
     &             ''in MODPATH name file'')')
         GO TO 9999
      ELSE
         WRITE(*,'(
     &          ''      .Processing particles from Time-Series file '',
     &          A,'' on unit number'',I5)')
     &       TIMFIL (1:INDEX(TIMFIL,' ')-1),
     &       TIMUNT
         WRITE(OUTUNT,'(
     &          ''      .Processing particles from Time-Series file '',
     &          A,'' on unit number'',I5)')
     &       TIMFIL (1:INDEX(TIMFIL,' ')-1),
     &       TIMUNT
      ENDIF
C
C1----Set local variables
C
      MODFLG=INT(COVPMR(1))
      ICOVER=INT(COVPMR(2))
      XYSCALE=COVPMR(3)
      ZSCALE=COVPMR(4)
      ANGLE=COVPMR(5)
      XOGRID=COVPMR(6)
      YOGRID=COVPMR(7)
      IF(COVPMR(9).LE.0.0) COVPMR(9)=1.0
      HSCALE=12.0/COVPMR(9)
      IF(COVPMR(8).LE.0.0) COVPMR(8)=1.0
      VSCALE=HSCALE*COVPMR(8)
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
      MODFMT=INT(COVPMR(MAXPMR-4))
      MODFMT=MODFMT/100
      YMIN=COVPMR(MAXPMR)
C
      TOTPTS=0
      NUMPTS=0
C
C2----Transform from Modpath origin (lower left) to Modular Model
C      origin (upper left)
C
      YORIGN=0.0
C
C2A----Calculate upper left corner distance from DELC array (y-axis)
C
      DO 200 ROW=NCOL+1,NCOL+NROW
         YORIGN=YORIGN+DELX(ROW)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
      WIDTHS(5)=LENLAY
      OUTPUTS(5)=LENLAY
C
      WIDTHS(14)=LENROW
      OUTPUTS(14)=LENROW
      WIDTHS(15)=LENCOL
      OUTPUTS(15)=LENCOL
      WIDTHS(16)=LENLAY
      OUTPUTS(16)=LENLAY
C
      WIDTHS(26)=LENROW
      OUTPUTS(26)=LENROW
      WIDTHS(27)=LENCOL
      OUTPUTS(27)=LENCOL
      WIDTHS(28)=LENLAY
      OUTPUTS(28)=LENLAY
C
      WIDTHS(44)=LENROW
      OUTPUTS(44)=LENROW
      WIDTHS(45)=LENCOL
      OUTPUTS(45)=LENCOL
      WIDTHS(46)=LENLAY
      OUTPUTS(46)=LENLAY
C
C3A---Delete and create files for item names of particle attributes
C
      CALL ITEM_FILES (
     I                 STRFIL,STRUNT,
     I                 ITMSTR(1),ITMFIN(1),
     I                 MAXITM,
     I                 DECIMLS,
     I                 ITEMS,
     I                 OUTPUTS,
     I                 TYPES,
     I                 WIDTHS,
     E                 *9999)
      CALL ITEM_FILES (
     I                 LSTFIL,LSTUNT,
     I                 ITMSTR(2),ITMFIN(2),
     I                 MAXITM,
     I                 DECIMLS,
     I                 ITEMS,
     I                 OUTPUTS,
     I                 TYPES,
     I                 WIDTHS,
     E                 *9999)
      CALL ITEM_FILES (
     I                 PREFIL,PREUNT,
     I                 ITMSTR(3),ITMFIN(3),
     I                 MAXITM,
     I                 DECIMLS,
     I                 ITEMS,
     I                 OUTPUTS,
     I                 TYPES,
     I                 WIDTHS,
     E                 *9999)
      CALL ITEM_FILES (
     I                 FINFIL,FINUNT,
     I                 ITMSTR(4),ITMFIN(4),
     I                 MAXITM,
     I                 DECIMLS,
     I                 ITEMS,
     I                 OUTPUTS,
     I                 TYPES,
     I                 WIDTHS,
     E                 *9999)
      CALL ITEM_FILES (
     I                 CUMFIL,CUMUNT,
     I                 ITMSTR(5),ITMFIN(5),
     I                 MAXITM,
     I                 DECIMLS,
     I                 ITEMS,
     I                 OUTPUTS,
     I                 TYPES,
     I                 WIDTHS,
     E                 *9999)
C
C3B---Delete/create files containing particle locations and attributes
C       and Arc command files
C
      NSTAT=2
      ITALK=1
      STRFIL='start'
      CALL DELETE_FILE (STRFIL,*9999)
      CALL OPEN_FILE (STRUNT,STRFIL,NSTAT,ITALK,*9999)
      LSTFIL='last'
      CALL DELETE_FILE (LSTFIL,*9999)
      CALL OPEN_FILE (LSTUNT,LSTFIL,NSTAT,ITALK,*9999)
      PREFIL='present'
      CALL DELETE_FILE (PREFIL,*9999)
      CALL OPEN_FILE (PREUNT,PREFIL,NSTAT,ITALK,*9999)
      CUMFIL='cum'
      CALL DELETE_FILE (CUMFIL,*9999)
      CALL OPEN_FILE (CUMUNT,CUMFIL,NSTAT,ITALK,*9999)
      GENFIL='points'
      CALL DELETE_FILE (GENFIL,*9999)
      CALL OPEN_FILE (GENUNT,GENFIL,NSTAT,ITALK,*9999)
C
C4----Time-Series file
C
      OLDSTP=-999
      NUMPTS=1
      NEWPTS=0
      OLDPTS=0
      NEWPRT=0
      TOTREC=0
      DO 400 N=1,MAXPTS
         PARTLE(N,1)=-99
 400  CONTINUE
C
C4A----Determine particle time-series format
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (TIMUNT,TIMFIL,NSTAT,ITALK,*9999)
      MODFMT=0
C
      READ(TIMUNT,'(A)',ERR=9990,END=9991) BUFFER
       MODPTH=INDEX(BUFFER,'Version 3.00')
        IF(MODPTH.LE.0) MODPTH=INDEX(BUFFER,'Version 4.')
         COMPCT=INDEX(BUFFER,'(COMPACT)')
          BUFDEX=INDEX(BUFFER,'TREF=')
           CALL CLOSE_FILE (TIMFIL,*9999)
C
      IF(COMPCT.GT.0) MODFMT=1
C
      IF(MODFMT.LT.1) THEN    
         WRITE(*,'(7X,
     &             ''..Using standard text format'')')
         WRITE(OUTUNT,'(7X,
     &             ''..Using standard text format'')')
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (TIMUNT,TIMFIL,NSTAT,ITALK,*9999)
      ELSEIF(MODFMT.EQ.1) THEN    
         WRITE(*,'(7X,
     &             '' Using compact text format'')')
         WRITE(OUTUNT,'(7X,
     &             '' Using compact text format'')')
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (TIMUNT,TIMFIL,NSTAT,ITALK,*9999)
      ELSEIF(MODFMT.EQ.2) THEN
         WRITE(*,'(7X,
     &             '' Using binary text format'')')
         WRITE(OUTUNT,'(7X,
     &             '' Using binary text format'')')
         NSTAT=-4
         ITALK=1
         CALL OPEN_FILE (-ABS(TIMUNT),TIMFIL,NSTAT,ITALK,*9999)
      ELSE
         WRITE(*,'(7X,
     &             ''..Using standard text format '',
     &             ''for MODPATH version 2.0 '')')
         WRITE(OUTUNT,'(7X,
     &             ''..Using standard text format '',
     &             ''for MODPATH version 2.0 '')')
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (TIMUNT,TIMFIL,NSTAT,ITALK,*9999)
      ENDIF
C
C4----Read header line from time-series file
C
      IF(MODFMT.LT.2) THEN    
         READ(TIMUNT,'(A)',ERR=9990,END=9991) BUFFER
         MODPTH=INDEX(BUFFER,'Version 3.')
       IF(MODPTH.LE.0) MODPTH=INDEX(BUFFER,'Version 4.')
         COMPCT=INDEX(BUFFER,'(COMPACT)')
         BUFDEX=INDEX(BUFFER,'TREF=')
       IF(BUFDEX.GT.0) THEN
          BUFFER=BUFFER (BUFDEX+5:LEN(BUFFER))
          BUFDEX=INDEX(BUFFER,')')
        IF(BUFDEX.LE.0) BUFDEX=LEN(BUFFER)
          READ(BUFFER (1:BUFDEX-1),*) TREF
       ELSE
          TREF=0.0
       ENDIF
      ELSE
         READ(TIMUNT,ERR=9990,END=9991) BUFFER,TREF
         MODPTH=INDEX(BUFFER,'Version 3.')
       IF(MODPTH.LE.0) MODPTH=INDEX(BUFFER,'Version 4.')
         BINARY=INDEX(BUFFER,'(BINARY)')
      ENDIF
C
C4C----Check format of time-series file
C
      IF(MODFMT.EQ.0 .AND. MODPTH.LE.0) THEN    
         WRITE(*,'(''      .Format of particle records is '',
     &             ''not in MODPATH version 3.0 text format '')')
         WRITE(*,'(''      ..Using standard text format '',
     &             ''for MODPATH version 2.0 '')')
         WRITE(OUTUNT,'(''      .Format of particle records is '',
     &             ''not in MODPATH version 3.0 text format '')')
         WRITE(OUTUNT,'(''      ..Using standard text format '',
     &             ''for MODPATH version 2.0 '')')
         MODFMT=9
         CALL CLOSE_FILE (TIMFIL,*9999)
         NSTAT=1
         ITALK=1
         CALL OPEN_FILE (TIMUNT,TIMFIL,NSTAT,ITALK,*9999)
      ENDIF
C
      IF(MODFMT.EQ.1 .AND. COMPCT.LE.0) THEN    
         WRITE(*,'(''      .Format of particle records is '',
     &             ''not in compact format '')')
         GO TO 9999
      ENDIF
C
      IF(MODFMT.EQ.2 .AND. BINARY.LE.0) THEN    
         WRITE(*,'(''      .Format of particle records is '',
     &             ''not in binary format '')')
         GO TO 9999
      ENDIF
C
C4----Print output header line
C
      WRITE(*,'(//,1H1,23X,
     &               ''  TIME-SERIES PARTICLE SUMMARY'',
     &               /10X,80(''-''))')
      WRITE(*,'(1X,
     &               ''           TIME'',
     &               ''          NEWLY'',
     &               ''     PREVIOUSLY'',
     &               ''          TOTAL'',
     &               ''          TOTAL'')')
      WRITE(*,'(1X,
     &               ''           STEP'',
     &               ''       RELEASED'',
     &               ''       RELEASED'',
     &               ''         ACTIVE'',
     &               ''        RECORDS'',
     &               /10X,80(''-''))')
      WRITE(OUTUNT,'(//,1H1,32X,
     &               ''  TIME-SERIES PARTICLE SUMMARY'',
     &               /10X,80(''-''))')
      WRITE(OUTUNT,'(10X,
     &               ''           TIME'',
     &               ''          NEWLY'',
     &               ''     PREVIOUSLY'',
     &               ''          TOTAL'',
     &               ''          TOTAL'')')
      WRITE(OUTUNT,'(10X,
     &               ''           STEP'',
     &               ''       RELEASED'',
     &               ''       RELEASED'',
     &               ''         ACTIVE'',
     &               ''        RECORDS'',
     &               /10X,80(''-''))')
C
C4A---Loop to read arcs from time-series file
C
  402 IF(MODFMT.LT.1) THEN    
         READ(TIMUNT,*,ERR=9992,END=502)
     &                                  TIMSTP,
     &                                  IDPART,
     &                                  COL,
     &                                  ROW,
     &                                  LAY,
     &                                  XGC,
     &                                  YGC,
     &                                  ZGC,
     &                                  ZLC,
     &                                  TIME,
     &                                  MODSTP
      ELSE IF(MODFMT.EQ.1) THEN
         READ(TIMUNT,*,ERR=9992,END=502)
     &                                  TIMSTP,
     &                                  IDPART,
     &                                  NODE,
     &                                  XGC,
     &                                  YGC,
     &                                  ZGC,
     &                                  ZLC,
     &                                  TIME,
     &                                  MODSTP
         CALL ND2IJK (
     &                NODE,
     &                ROW,
     &                COL,
     &                LAY,
     &                NROW,
     &                NCOL
     &                )
      ELSE IF(MODFMT.EQ.2) THEN
         READ(TIMUNT,ERR=9992,END=502)
     &                                  TIMSTP,
     &                                  IDPART,
     &                                  NODE,
     &                                  XGC,
     &                                  YGC,
     &                                  ZGC,
     &                                  ZLC,
     &                                  TIME,
     &                                  MODSTP
         CALL ND2IJK (
     &                NODE,
     &                ROW,
     &                COL,
     &                LAY,
     &                NROW,
     &                NCOL
     &                )
      ELSE
         READ(TIMUNT,*,ERR=9992,END=502)
     &                                  TIMSTP,
     &                                  IDPART,
     &                                  COL,
     &                                  ROW,
     &                                  LAY,
     &                                  XGC,
     &                                  YGC,
     &                                  ZGC,
     &                                  ZLC,
     &                                  TIME
         MODSTP=0
      ENDIF
C
C4Aa----Compute local x-,y-coordinates
C
      CALL LOCAL_COORDINATES (
     I                        NCOL,NROW,
     I                        DELX,
     I                        COL,ROW,
     I                        XGC,YGC,YORIGN,
     O                        XLC,YLC)
C
C4Ab---Transform and assign coordinates
C
      XGC=XGC*XYSCALE
       YGC=(YGC-YORIGN)*XYSCALE
        ZGC=ZGC*ZSCALE
      IF(IVIEW.LE.0) THEN
       IF(ANGLE.NE.0.0) THEN
          CALL ROTATE (
     I                 ANGLE,
     B                 XGC,YGC)
       ENDIF
         XGC=XGC+XOGRID
         YGC=YGC+YOGRID
         XCOORD=XGC
         YCOORD=YGC
      ELSE IF(IVIEW.EQ.1) THEN
         XCOORD=XGC*HSCALE
         YCOORD=ZGC*VSCALE-YMIN
      ELSE
         XCOORD=-1.0*YGC*HSCALE
         YCOORD=ZGC*VSCALE-YMIN
      ENDIF
C
      IF(TIME.LT.0.000000001) TIME=0.0
C
C4----Test time step index number and particle ids
C
        TOTREC=TOTREC+1
      IF(IDPART.GT.MAXPTS) GO TO 9993
C
       IF(TIMSTP.NE.OLDSTP) THEN
        IF(TOTREC.GT.1) THEN
           TOTPTS=NEWPTS+OLDPTS
           WRITE(*,'(1X,
     &               I15,
     &               I15,
     &               I15,
     &               I15,
     &               I15)')
     &             TIMSTP,
     &             NEWPTS,
     &             OLDPTS,
     &             TOTPTS,
     &             TOTREC
           WRITE(OUTUNT,'(10X,
     &               I15,
     &               I15,
     &               I15,
     &               I15,
     &               I15)')
     &             TIMSTP,
     &             NEWPTS,
     &             OLDPTS,
     &             TOTPTS,
     &             TOTREC
        ENDIF
          NEWPTS=0
          OLDPTS=0
          NUMSTP=NUMSTP+1
          OLDSTP=TIMSTP
       ENDIF
C
C4D----Load particle_id and time_step attributes
C
         PARTID=PARTLE(IDPART,1)
C
C4Da----Load start and print attributes
C
      IF(PARTID.LT.1) THEN
         NEWPTS=NEWPTS+1
         NEWPRT=NEWPRT+1
         PARTLE(IDPART,1)=IDPART
         PARTLE(IDPART,2)=TIMSTP
         PARTLE(IDPART,3)=ROW
         PARTLE(IDPART,4)=COL
         PARTLE(IDPART,5)=LAY
         PARTLE(IDPART,6)=IBOUND(COL,ROW,LAY)
         PARTLE(IDPART,7)=XGC
         PARTLE(IDPART,8)=XLC
         PARTLE(IDPART,9)=YGC
         PARTLE(IDPART,10)=YLC
         PARTLE(IDPART,11)=ZGC
         PARTLE(IDPART,12)=ZLC
         PARTLE(IDPART,13)=TIME
      ELSE
         OLDPTS=OLDPTS+1
      ENDIF
C
      IDPART=PARTLE(IDPART,1)
      STRSTP=PARTLE(IDPART,2)
      STRROW=PARTLE(IDPART,3)
      STRCOL=PARTLE(IDPART,4)
      STRLAY=PARTLE(IDPART,5)
      STRIBD=PARTLE(IDPART,6)
      WRITE(STRUNT,'(6(I10,'',''),
     &               6(G16.6,'',''),
     &               1(G16.3,'',''))')
     &                                   IDPART,
     &                                   STRSTP,
     &                                   STRROW,
     &                                   STRCOL,
     &                                   STRLAY,
     &                                   STRIBD,
     &                                   (PARTLE(IDPART,M), M=7,13)
C
C4Da----Load present attributes
C
      PARTLE(IDPART,25)=TIMSTP
      PARTLE(IDPART,26)=ROW
      PARTLE(IDPART,27)=COL
      PARTLE(IDPART,28)=LAY
      PARTLE(IDPART,29)=IBOUND(COL,ROW,LAY)
      PARTLE(IDPART,30)=XGC
      PARTLE(IDPART,31)=XLC
      PARTLE(IDPART,32)=YGC
      PARTLE(IDPART,33)=YLC
      PARTLE(IDPART,34)=ZGC
      PARTLE(IDPART,35)=ZLC
      PARTLE(IDPART,36)=TIME
C
      IF(PARTID.LT.1) THEN
         PARTLE(IDPART,36)=0.0
         PARTLE(IDPART,37)=0.0
         PARTLE(IDPART,38)=0.0
         PARTLE(IDPART,39)=0.0
         PARTLE(IDPART,40)=0.0
         XGCVEL=0.0
         YGCVEL=0.0
         ZGCVEL=0.0
         PRTSPD=0.0
         XGCPTH=0.0
         YGCPTH=0.0
         ZGCPTH=0.0
         ARCPTH=0.0
         TRVTIM=0.0
      ELSE
         TRVTIM=ABS(TIME-PARTLE(IDPART,24))
         PARTLE(IDPART,37)=TRVTIM
        IF(TRVTIM.LE.0.0) THEN
           XGCVEL=0.0
           YGCVEL=0.0
           ZGCVEL=0.0
           PRTSPD=0.0
           XGCPTH=0.0
           YGCPTH=0.0
           ZGCPTH=0.0
           ARCPTH=0.0
        ELSE
           XGCPTH=XGC-PARTLE(IDPART,18)
           YGCPTH=YGC-PARTLE(IDPART,20)
           ZGCPTH=ZGC-PARTLE(IDPART,22)
           ARCPTH=(XGCPTH**2+YGCPTH**2+ZGCPTH**2)**0.5
           XGCVEL=XGCPTH/TRVTIM
           YGCVEL=YGCPTH/TRVTIM
           ZGCVEL=ZGCPTH/TRVTIM
           PRTSPD=ARCPTH/TRVTIM
         IF(ABS(XGCVEL).LE.0.00000001) XGCVEL=SIGN(0.00000001,XGCVEL)
         IF(ABS(YGCVEL).LE.0.00000001) YGCVEL=SIGN(0.00000001,YGCVEL)
         IF(ABS(ZGCVEL).LE.0.00000001) ZGCVEL=SIGN(0.00000001,ZGCVEL)
         IF(PRTSPD.LE.0.00000001) PRTSPD=0.00000001
        ENDIF
          PARTLE(IDPART,38)=XGCVEL
          PARTLE(IDPART,39)=YGCVEL
          PARTLE(IDPART,40)=ZGCVEL
          PARTLE(IDPART,41)=PRTSPD
      ENDIF
C
      PRESTP=PARTLE(IDPART,25)
      PREROW=PARTLE(IDPART,26)
      PRECOL=PARTLE(IDPART,27)
      PRELAY=PARTLE(IDPART,28)
      PREIBD=PARTLE(IDPART,29)
      WRITE(PREUNT,'(5(I5,'',''),
     &               6(G16.6,'',''),
     &               2(G16.3,'',''),
     &               4(G16.8,'',''))')
     &                                 PRESTP,
     &                                 PREROW,
     &                                 PRECOL,
     &                                 PRELAY,
     &                                 PREIBD,
     &                                 (PARTLE(IDPART,M), M=30,41)
C
C4Da----Load and print last attributes
C
      IF(PARTID.LT.1) THEN
         PARTLE(IDPART,14)=ROW
         PARTLE(IDPART,15)=COL
         PARTLE(IDPART,16)=LAY
         PARTLE(IDPART,17)=IBOUND(COL,ROW,LAY)
         PARTLE(IDPART,18)=XGC
         PARTLE(IDPART,19)=XLC
         PARTLE(IDPART,20)=YGC
         PARTLE(IDPART,21)=YLC
         PARTLE(IDPART,22)=ZGC
         PARTLE(IDPART,23)=ZLC
         PARTLE(IDPART,24)=TIME
      ENDIF
        LSTROW=PARTLE(IDPART,14)
        LSTCOL=PARTLE(IDPART,15)
        LSTLAY=PARTLE(IDPART,16)
        LSTIBD=PARTLE(IDPART,17)
        WRITE(LSTUNT,'(4(I10,'',''),
     &                 6(G16.6,'',''),
     &                 1(G16.3,'',''))')
     &                                   LSTROW,
     &                                   LSTCOL,
     &                                   LSTLAY,
     &                                   LSTIBD,
     &                                   (PARTLE(IDPART,M), M=18,24)
      PARTLE(IDPART,14)=PREROW
      PARTLE(IDPART,15)=PRECOL
      PARTLE(IDPART,16)=PRELAY
      PARTLE(IDPART,17)=PREIBD
      PARTLE(IDPART,18)=XGC
      PARTLE(IDPART,19)=XLC
      PARTLE(IDPART,20)=YGC
      PARTLE(IDPART,21)=YLC
      PARTLE(IDPART,22)=ZGC
      PARTLE(IDPART,23)=ZLC
      PARTLE(IDPART,24)=TIME
C
C4Db----Load final attributes
C
      PARTLE(IDPART,42)=IDPART
      PARTLE(IDPART,43)=TIMSTP
      PARTLE(IDPART,44)=ROW
      PARTLE(IDPART,45)=COL
      PARTLE(IDPART,46)=LAY
      PARTLE(IDPART,47)=IBOUND(COL,ROW,LAY)
      PARTLE(IDPART,48)=XGC
      PARTLE(IDPART,49)=XLC
      PARTLE(IDPART,50)=YGC
      PARTLE(IDPART,51)=YLC
      PARTLE(IDPART,52)=ZGC
      PARTLE(IDPART,53)=ZLC
      PARTLE(IDPART,54)=TIME
      PARTLE(IDPART,55)=PARTLE(IDPART,55)+XGCPTH
      PARTLE(IDPART,56)=PARTLE(IDPART,56)+YGCPTH
      PARTLE(IDPART,57)=PARTLE(IDPART,57)+ZGCPTH
      PARTLE(IDPART,58)=PARTLE(IDPART,58)+ARCPTH
C
C4Da----Load cumulative attributes
C
      STRTIM=PARTLE(IDPART,13)
      TRVTIM=ABS(TIME-STRTIM)
C
      IF(TRVTIM.LE.0.0) THEN
         XGCVEL=0.0
         YGCVEL=0.0
         ZGCVEL=0.0
         PRTSPD=0.0
      ELSE
         XGCPTH=PARTLE(IDPART,55)
         YGCPTH=PARTLE(IDPART,56)
         ZGCPTH=PARTLE(IDPART,57)
         XGCVEL=XGCPTH/TRVTIM
         YGCVEL=YGCPTH/TRVTIM
         ZGCVEL=ZGCPTH/TRVTIM
         ARCPTH=(XGCPTH**2+YGCPTH**2+ZGCPTH**2)**0.5
         PRTSPD=ARCPTH/TRVTIM
       IF(ABS(XGCVEL).LE.0.00000001) XGCVEL=SIGN(0.00000001,XGCVEL)
       IF(ABS(YGCVEL).LE.0.00000001) YGCVEL=SIGN(0.00000001,YGCVEL)
       IF(ABS(ZGCVEL).LE.0.00000001) ZGCVEL=SIGN(0.00000001,ZGCVEL)
       IF(PRTSPD.LE.0.00000001) PRTSPD=0.00000001
      ENDIF
C
      PARTLE(IDPART,59)=XGCVEL
      PARTLE(IDPART,60)=YGCVEL
      PARTLE(IDPART,61)=ZGCVEL
      PARTLE(IDPART,62)=PRTSPD
      PARTLE(IDPART,63)=MODSTP
C
      WRITE(CUMUNT,'(4(G16.8,'',''),
     &               I10)')
     &                                (PARTLE(IDPART,M), M=59,62),
     &                                MODSTP
C
C4----Write coordinates and label-ids to generate file
C
      WRITE(GENUNT,'(I10,'','',F20.10,'','',F20.10)')
     &                                             IDPART,
     &                                             XCOORD,
     &                                             YCOORD
C
C4E----Read another record
C
      GO TO 402
C
C5----Close files
C
  502 CALL CLOSE_FILE (TIMFIL,*9999)
      WRITE(GENUNT,'(''END'')')
      CALL CLOSE_FILE (GENFIL,*9999)
      CALL CLOSE_FILE (STRFIL,*9999)
      CALL CLOSE_FILE (LSTFIL,*9999)
      CALL CLOSE_FILE (PREFIL,*9999)
      CALL CLOSE_FILE (CUMFIL,*9999)
C
C5----Write ending time step information
C
      TOTPTS=NEWPTS+OLDPTS
      WRITE(*,'(1X,
     &              ''END OF '',I8,
     &              I15,
     &              I15,
     &              I15,
     &              I15)')
     &                    TIMSTP,
     &                    NEWPTS,
     &                    OLDPTS,
     &                    TOTPTS,
     &                    TOTREC
      WRITE(*,'(10X,80(''-''))')
      WRITE(OUTUNT,'(10X,
     &              ''END OF '',I8,
     &              I15,
     &              I15,
     &              I15,
     &              I15)')
     &                    TIMSTP,
     &                    NEWPTS,
     &                    OLDPTS,
     &                    TOTPTS,
     &                    TOTREC
      WRITE(OUTUNT,'(10X,80(''-''))')
C
C6----Write particle attributes in final file
C
      NSTAT=2
      ITALK=1
      FINFIL='final'
      CALL DELETE_FILE (FINFIL,*9999)
      CALL OPEN_FILE (FINUNT,FINFIL,NSTAT,ITALK,*9999)
C
      NSTAT=1
      ITALK=1
      CALL OPEN_FILE (GENUNT,GENFIL,NSTAT,ITALK,*9999)
C
 602    READ(GENUNT,'(A)',ERR=9990,END=604) BUFFER
      IF(INDEX(BUFFER,'END').GT.0) GO TO 602
         READ(BUFFER,*) IDPART
C
          FINSTP=PARTLE(IDPART,43)
          FINROW=PARTLE(IDPART,44)
          FINCOL=PARTLE(IDPART,45)
          FINLAY=PARTLE(IDPART,46)
          FINIBD=PARTLE(IDPART,47)
          FINTIM=PARTLE(IDPART,54)
          STRTIM=PARTLE(IDPART,13)
          TRVTIM=ABS(TIME-STRTIM)
C
        IF(TRVTIM.LE.0.0) THEN
           XGCVEL=0.0
           YGCVEL=0.0
           ZGCVEL=0.0
           PRTSPD=0.0
        ELSE
           XGCPTH=PARTLE(IDPART,55)
           YGCPTH=PARTLE(IDPART,56)
           ZGCPTH=PARTLE(IDPART,57)
           XGCVEL=XGCPTH/TRVTIM
           YGCVEL=YGCPTH/TRVTIM
           ZGCVEL=ZGCPTH/TRVTIM
           ARCPTH=(XGCPTH**2+YGCPTH**2+ZGCPTH**2)**0.5
           PRTSPD=ARCPTH/TRVTIM
         IF(ABS(XGCVEL).LE.0.00000001) XGCVEL=SIGN(0.00000001,XGCVEL)
         IF(ABS(YGCVEL).LE.0.00000001) YGCVEL=SIGN(0.00000001,YGCVEL)
         IF(ABS(ZGCVEL).LE.0.00000001) ZGCVEL=SIGN(0.00000001,ZGCVEL)
         IF(PRTSPD.LE.0.00000001) PRTSPD=0.00000001
        ENDIF
          PARTLE(IDPART,55)=XGCVEL
          PARTLE(IDPART,56)=YGCVEL
          PARTLE(IDPART,57)=ZGCVEL
          PARTLE(IDPART,58)=PRTSPD
          WRITE(FINUNT,'(I10,'','',
     &                   5(I5,'',''),
     &                   6(G16.6,'',''),
     &                   1(G16.3,'',''),
     &                   4(G16.8,'',''))')
     &                                    PARTID,
     &                                    FINSTP,
     &                                    FINROW,
     &                                    FINCOL,
     &                                    FINLAY,
     &                                    FINIBD,
     &                                    (PARTLE(IDPART,M), M=48,58)
       GO TO 602
C
 604  CALL CLOSE_FILE (GENFIL,*9999)
      CALL CLOSE_FILE (FINFIL,*9999)
C
C7----Message to user
C
      WRITE(*,'(''     .'',
     &          I10,'' particles were found for '',A,
     &          '' time-series coverage'')')
     &        NEWPRT,
     &        PRTCOV (1:INDEX(PRTCOV,' ')-1)
      WRITE(*,'(''     .'',G16.7,
     &          '' simulation reference time '')')
     &       TREF
      WRITE(OUTUNT,'(//,
     &          ''     .'',
     &          I10,'' particles were found for '',A,
     &          '' time-series coverage'')')
     &        NEWPRT,
     &        PRTCOV (1:INDEX(PRTCOV,' ')-1)
      WRITE(OUTUNT,'(''     .'',G16.7,
     &          '' simulation reference time '')')
     &       TREF
C
C8----Arc Operations to coverage and attribute files
C
      NSTAT=2
      ITALK=1
      ARCFIL='arccmd'
      CALL DELETE_FILE (ARCFIL,*9999)
      CALL OPEN_FILE (ARCUNT,ARCFIL,NSTAT,ITALK,*9999)
C
C8A----Create time-series coverage
C
      ARCCMD='create'
      WRITE(ARCUNT,'(A,1X,A)') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             PRTCOV (1:INDEX(PRTCOV,' ')-1)
C
C8B----Generate Coverage PRTCOV
C
      ARCCMD='generate'
      COVTYP='point'
      JONFIL='points'
      WRITE(ARCUNT,'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             PRTCOV (1:INDEX(PRTCOV,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVTYP (1:INDEX(COVTYP,' ')-1)
C
C8C----Build Coverage PRTCOV
C
      ARCCMD='build'
      COVTYP='point'
      WRITE(ARCUNT,'(A,2(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             PRTCOV (1:INDEX(PRTCOV,' ')-1),
     &                             COVTYP (1:INDEX(COVTYP,' ')-1)
C
C8D----Create attribute files and "add from" attributes
C
      ARCCMD='tables'
      TMPFIL='startitems'
      ADDFIL='start'
      ACTION='kill'
      WRITE(ARCUNT,'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
      ARCCMD='tables'
      TMPFIL='lastitems'
      ADDFIL='last'
      WRITE(ARCUNT,'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
      ARCCMD='tables'
      TMPFIL='presentitems'
      ADDFIL='present'
      WRITE(ARCUNT,'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
      ARCCMD='tables'
      TMPFIL='finalitems'
      ADDFIL='final'
      WRITE(ARCUNT,'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
      ARCCMD='tables'
      TMPFIL='cumitems'
      ADDFIL='cum'
      WRITE(ARCUNT,'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
C
C8E---Join attribute files start, present,final to Coverage PRTCOV AAT file
C
      ARCCMD='joinitem'
      COVFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      JONFIL='cum'
      RELITM='$recno'
      STRITM=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'-ID'
      JONTYP='link'      
      WRITE(ARCUNT,'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ARCCMD='joinitem'
      COVFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      JONFIL='present'
      RELITM='$recno'
      JONTYP='link'      
      WRITE(ARCUNT,'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ARCCMD='joinitem'
      COVFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      JONFIL='last'
      RELITM='$recno'
      JONTYP='link'      
      WRITE(ARCUNT,'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ARCCMD='joinitem'
      COVFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      JONFIL='start'
      RELITM='$recno'
      JONTYP='link'      
      WRITE(ARCUNT,'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      ARCCMD='joinitem'
      COVFIL=PRTCOV (1:INDEX(PRTCOV,' ')-1)//'.PAT'
      JONFIL='final'
      RELITM='PARTID'
      STRITM='PRESPD'
      JONTYP='ordered'      
      WRITE(ARCUNT,'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      CALL CLOSE_FILE (ARCFIL,*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,
     &      '(//,'' Unable to read first line from '',
     &      ''Timeseries file'')')
      WRITE(OUTUNT,
     &      '(//,'' Unable to read first line from '',
     &      ''Timeseries file'')')
      CALL CLOSE_FILE (TIMFIL,*9999)
      GO TO 9999
 9991 WRITE(*,
     &      '(//,'' Premature end-of-line error in '',
     &      ''Timeseries file'')')
      WRITE(OUTUNT,
     &      '(//,'' Premature end-of-line error in '',
     &      ''Timeseries file'')')
      GO TO 9999
 9992 WRITE(*,
     &      '(//,'' Wrong format in Timeseries file'')')
      WRITE(OUTUNT,
     &      '(//,'' Wrong format in Timeseries file'')')
      CALL CLOSE_FILE (TIMFIL,*9999)
      GO TO 9999
 9993 WRITE(*,
     &      '(//,'' Exceed maximum number of points > '',I10)') 
     &        MAXPTS
      WRITE(OUTUNT,
     &      '(//,'' Exceed maximum number of points > '',I10)') 
     &        MAXPTS
      CALL CLOSE_FILE (TIMFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal termination of Pthtim_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal termination of Pthtim_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
