C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Velall version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.1 VELALL 13JUNE2005
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Allocates space within the common array for cell-by-cell flow terms:
C      constant_heads, flow_front_face, flow_right_face, and
C      flow_lower_face polygon coverages and attributes.  These
C      cell-by-cell flow terms are output of the Block-Centered Package of
C      the by the U.S. Geological Survey Modular Three-Dimensional
C      Finite-Differences Ground-Water Flow model MODFLOW or MODFLOWARC
C      (McDonald and Harbaugh, 1988; Orzol and McGrath, 1992).
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT NUMBER FOR MODTOOLS OUTPUT LSTING FILE
C     LENX    : MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY
C     NLAY    : NUMBER OF LAYERS IN MODEL GRID
C     NROW    : NUMBER OF ROWS IN MODEL GRID
C     NCOL    : NUMBER OF COLUMNS IN MODEL GRID
C
C   Inputs/Outputs:
C
C     ISUM    : INDEX NUMBER TO COMMON BLOCK ARRAY FOR ALLOCATION
C
C    Outputs:
C
C     LCFRFB  : STARTING ARRAY INDEX FOR CELL_BY_CELL FLOW_RIGHT_FACE TERMS
C     LCFFFB  : STARTING ARRAY INDEX FOR CELL_BY_CELL FLOW_FRONT_FACE TERMS
C     LCFLFB  : STARTING ARRAY INDEX FOR CELL_BY_CELL FLOW_LOWER_FACE TERMS
C     LCHEAD  : STARTING ARRAY INDEX FOR CELL_BY_CELL HEAD TERMS
C     LCPORO  : STARTING ARRAY INDEX FOR POROSITY
C     LCATRB  : STARTING ARRAY INDEX FOR ATTRIBUTE TABLE (CELLS)
C
C     Program:
C
C     ISUM1   : INDEX NUMBER TO LAST COMMON BLOCK ARRAY FOR ALLOCATION
C
C-------------------------------------------------------------------------
      SUBROUTINE VELALL (
     I                   OUTUNT,
     I                   LENX,
     I                   NLAY,NROW,NCOL,
     I                   MAXATB,MAXPTS,MAXCEL,
     B                   ISUM,
     O                   LCCHDB,LCFFFB,LCFLFB,LCFRFB,
     O                   LCHEAD,LCPORO,
     O                   LCATRB,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      INTEGER
     &        OUTUNT,
     &        LENX,
     &        NLAY,NROW,NCOL,
     &        ISUM,
     &        LCCHDB,LCFFFB,LCFLFB,LCFRFB,LCHEAD,LCATRB
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(/)')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          VELOCITY GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''             VERSION 6.1 13JUNE2005 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               ALLOCATION MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(/)')
C
C1----Calculate storage parameter NRCL
C
      NRCL=NROW*NCOL*NLAY
      NRNC=NROW*NCOL
C
C1----Calculate storage parameter MAXPTS
C
      MAXCEL=10*MAXPTS
      MAXNUM=10
      MAXGRD=INT(MAXCEL/MAXNUM)
C
C2----Allocate space for arrays
C
      LCCHDB=ISUM
      ISUM=ISUM+NRCL
      LCFFFB=ISUM
      ISUM=ISUM+NRCL
      LCFLFB=ISUM
      ISUM=ISUM+NRCL
      LCFRFB=ISUM
      ISUM=ISUM+NRCL
      LCHEAD=ISUM
      ISUM=ISUM+NRCL
      LCPORO=ISUM
      ISUM=ISUM+NRCL
C      ISUM=ISUM+MAXCEL
      LCATRB=ISUM
      ISUM=ISUM+NRCL*20
C      ISUM=ISUM+MAXCEL
      ISUM1=ISUM-1
C
C3----Print Modtools operation parameters
C
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''         CELL VELOCITY VECTORS PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          VERSION 6.0 23JUNE1997 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
C
      WRITE(OUTUNT,'(//,1H1,38X,
     &             ''COMMON BLOCK INDEX LISTING'',
     &             /10X,85(''-''))')
C
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM DIMENSION OF THE COMMON BLOCK ARRAY    '',
     &             10X,I15,'' LENX'')')
     &          LENX
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF CELL VECTOR TERMS ATTRIBUTES '',
     &             10X,I15,'' MAXNUM'')')
     &          MAXNUM
      WRITE(OUTUNT,'(10X,
     &             ''MAXIMUM NUMBER OF CELLS IN MODEL GRID          '',
     &             10X,I15,'' MAXGRD'')')
     &          MAXGRD
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR CONSTANT HEAD TERMS   '',
     &             10X,I15,'' LCCHDB'')')
     &          LCCHDB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR FLOW FRONT FACE TERMS '',
     &             10X,I15,'' LCFFFB'')')
     &          LCFFFB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR FLOW LOWER FACE TERMS '',
     &             10X,I15,'' LCFLFB'')')
     &          LCFLFB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR FLOW RIGHT FACE TERMS '',
     &             10X,I15,'' LCFRFB'')')
     &          LCFRFB
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR HEAD TERMS            '',
     &             10X,I15,'' LCHEAD'')')
     &          LCHEAD
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR POROSITY TERMS        '',
     &             10X,I15,'' LCPORO'')')
     &          LCPORO
      WRITE(OUTUNT,'(10X,
     &             ''STARTING ARRAY INDEX FOR ATTRIBUTE TERMS       '',
     &             10X,I15,'' LCATRB'')')
     &          LCATRB
      WRITE(OUTUNT,'(10X,85(''-''))')
C
      ISP=ISUM1-LCCHDB
      WRITE(OUTUNT,'(/10X,
     &             I8,1X,
     &             ''ELEMENTS IN COMMON BLOCK ARRAY USED '',
     &             ''FOR CELL VELOCITY VECTORS PACKAGE'')')
     &          ISP
C
      WRITE(OUTUNT,'(10X,
     &             I8,1X,
     &             ''ELEMENTS OF COMMON BLOCK ARRAY USED OUT OF '',
     &             I10)')
     &          ISUM1,LENX
      WRITE(OUTUNT,'(10X,85(''-''))')
C
C4----Check amount of space used
C
      IF(ISUM1.GT.LENX) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER*** ISUM '',I10,'' > LENX '',I10)')
     &             ISUM1,
     &             LENX
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Velall_Subroutine'')')
         WRITE (*,'(//,'' Cell array must be dimensioned larger ISUM '',
     &              I10,'' > LENX '',I10)')
     &           ISUM1,
     &           LENX
         WRITE(*,'(//,
     &             '' Abnormal termination of Velall_Subroutine'')')
         RETURN 1
      ENDIF
C
C5----Check amount of space needed for cell velocity vectors
C
      IF(NRNC.GT.MAXGRD) THEN
         WRITE(OUTUNT,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER***'')')
         WRITE(OUTUNT,'(
     &                '' Increase maxpts or maxatb '')')
         WRITE(OUTUNT,'(//,
     &             '' Abnormal termination of Velall_Subroutine'')')
         WRITE(*,'(//,
     &                ''    ***CELL ARRAY MUST BE DIMENSIONED '',
     &                ''LARGER***'')')
         WRITE(*,'(
     &                '' Increase maxpts or maxatb '')')
         WRITE(*,'(//,
     &             '' Abnormal termination of Velall_Subroutine'')')
         RETURN 1
      ENDIF
C
C6----Return
C
      RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Velrp1 version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C---VERSION 6.0 VELRP1 30AUGUST1999
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Reads an user defined files for cell_by_cell head terms.
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C    Inputs:
C
C     OUTFIL  : OUTPUT FILE NAME DOCUMENTING VELRP1 OPERATIONS.
C     OUTUNT  : FORTRAN UNIT NUMBER FOR OUTPUT DATA FILE
C     PORFIL  : FILE CONTAINING THE CELL-BY-CELL POROSITY TERMS
C     PORUNT  : FORTRAN UNIT NUMBER FOR CELL-BY-CELL POROSITY TERMS
C     CTLFIL  : FILE CONTAINING THE CONTROL LINES
C     CTLUNT  : FORTRAN UNIT NUMBER FOR U2DREL CONTROL LINES
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C
C    Outputs:
C
C     PORBUD  : ARRAY FOR CELL_BY_CELL HEAD TERMS
C
C-------------------------------------------------------------------------
      SUBROUTINE VELRP1 (
     I                   OUTFIL,OUTUNT,
     I                   PORFIL,PORUNT,
     I                   CTLFIL,CTLUNT,
     I                   NLAY,NROW,NCOL,
     I                   MAXPMR,
     I                   COVPMR,
     O                   PORBUD,
     E                   *)
C-----------------------------------------------------------------------
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     &          COVPMR(MAXPMR),
     &          PORBUD(NCOL*NROW*NLAY)
C
      CHARACTER*(*) OUTFIL,PORFIL,CTLFIL
C
      INTEGER OUTUNT,PORUNT,CTLUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      CHARACTER*16 PORTXT,TEXT,MODTXT
      CHARACTER*24 ANAME(1)
C
      INTEGER CBCUNT,ICOL,IROW,ILAY,LAYER,KK,PORFLG,MODPER,MODSTP
C
      DATA MODTXT /'        POROSITY'/
      DATA ANAME(1) /'                POROSITY'/
C
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                 Reading basic model problem information
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C1*********************************************************************C
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(//10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          CELL-BY-CELL POROSITY TERMS '',
     &               ''PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                 VERSION 6.0 22MARCH2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''                  READ/PREPARE MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1L----Set local variables
C
      MODPER=INT(COVPMR(13))
      MODSTP=INT(COVPMR(14))
      NPER=INT(COVPMR(17))
      MODFLG=INT(COVPMR(1))
C
C2---Text formatted cell-by-cell head terms
C
      WRITE(*,'(
     &             ''      ..reading text formatted cell_by_cell '',
     &             '' porosity terms '')')
      WRITE(OUTUNT,'(
     &             ''      ..reading text formatted cell_by_cell '',
     &             '' porosity terms '')')
C
C2A----Open control file CTLFIL and text formatted file PORFIL
C
      IF(PORUNT.GT.0) THEN
       IF(CTLUNT.GT.0) THEN
          NSTAT=1
          ITALK=1
          CALL OPEN_FILE (CTLUNT,CTLFIL,NSTAT,ITALK,*9999)
          CALL OPEN_FILE (PORUNT,PORFIL,NSTAT,ITALK,*9999)
          WRITE(*,'(
     &              ''      ....recorded in file '',A)')
     &              PORFIL (1:INDEX(PORFIL,' ')-1)
          WRITE(*,'(
     &              ''      ....on unit '',I10)')
     &                                                 PORUNT
          WRITE(*,'(
     &              ''      ..using control lines '')')
          WRITE(*,'(
     &              ''      ....recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....recorded in file '',A)')
     &              PORFIL (1:INDEX(PORFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....on unit '',I10)')
     &                                                 PORUNT
          WRITE(OUTUNT,'(
     &              ''      ..using control lines '')')
          WRITE(OUTUNT,'(
     &              ''      ..recorded in file '',A)')
     &              CTLFIL (1:INDEX(CTLFIL,' ')-1)
       ELSE
          WRITE(*,'(
     &              ''      ..Can not read text formatted '',
     &              ''cell_by_cell '',A,'' terms '')')
     &                              MODTXT (1:INDEX(MODTXT,' ')-1)
          WRITE(*,'(
     &              ''      ....recorded in file '',A)')
     &              PORFIL (1:INDEX(PORFIL,' ')-1)
          WRITE(*,'(
     &              ''      ....on unit '',I10)')
     &                                                 PORUNT
          WRITE(*,'(
     &              ''      ..Need file containing U2DREL control '',
     &              ''lines for format of cell-by-cell '',
     &              A,'' file'')') 
     &                              MODTXT (1:INDEX(MODTXT,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ..Can not read text formatted '',
     &              ''cell_by_cell '',A,'' terms '')')
     &                              MODTXT (1:INDEX(MODTXT,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....recorded in file '',A)')
     &              PORFIL (1:INDEX(PORFIL,' ')-1)
          WRITE(OUTUNT,'(
     &              ''      ....on unit '',I10)')
     &                                                 PORUNT
          WRITE(OUTUNT,'(
     &              ''      ..Need file containing U2DREL control '',
     &              ''lines for format of cell-by-cell '',
     &              A,'' file'')') 
     &                              MODTXT (1:INDEX(MODTXT,' ')-1)
          CALL CLOSE_FILE (PORFIL,*9999)
          GO TO 9999
       ENDIF
C
C2B---Loop to read from text formatted file PORFIL
C
         DO 200 LAYER=1,NLAY
            KK=LAYER
            LOC=1+(KK-1)*NCOL*NROW
            CALL U2DREL (PORBUD(LOC),ANAME(1),NROW,NCOL,KK,CTLUNT,
     &                   OUTUNT)
  200    CONTINUE
C
C2C---Close text formatted file PORFIL
C
         CALL CLOSE_FILE (PORFIL,*9999)
         CALL CLOSE_FILE (CTLFIL,*9999)
C
C3---Error no file containing cell-by-cell head terms
C
      ELSE
         WRITE(*,'(
     &             ''      ..No file containing cell_by_cell '',
     &             A,'' terms specified in MODTOOLS name file'')')
     &                              MODTXT (1:INDEX(MODTXT,' ')-1)
         WRITE(OUTUNT,'(
     &             ''      ..No file containing cell_by_cell '',
     &             A,'' terms specified in MODTOOLS name file'')')
     &                              MODTXT (1:INDEX(MODTXT,' ')-1)
         GO TO 9999
      ENDIF
        GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9990 WRITE(*,'(//,
     &          '' Unable to read header line for layer '',I5,
     &          '' for porosity terms from file '',A)') 
     &          KK,
     &          PORFIL (1:INDEX(PORFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for layer '',I5,
     &          '' for porosity terms from file '',A)') 
     &          KK,
     &          PORFIL (1:INDEX(PORFIL,' ')-1)
      GO TO 9998
 9991 WRITE(*,'(//,
     &          '' Unable to read values for '',
     &          ''porosity terms from file '',A)') 
     &          PORFIL (1:INDEX(PORFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' Unable to read header line for '',
     &          ''porosity terms from file '',A)') 
     &          PORFIL (1:INDEX(PORFIL,' ')-1)
      GO TO 9998
 9992 WRITE(*,'(//,
     &          '' End-of-file; Unable to read values for '',
     &          ''porosity terms from file '',A)') 
     &          PORFIL (1:INDEX(PORFIL,' ')-1)
      WRITE(OUTUNT,'(//,
     &          '' End-of-file; Unable to read header line for '',
     &          ''porosity terms from file '',A)') 
     &          PORFIL (1:INDEX(PORFIL,' ')-1)
      GO TO 9998
 9998 CALL CLOSE_FILE (PORFIL,*9999)
 9999 WRITE(*,
     &      '(//,'' Abnormal Termination of Velrp1_Subroutine'')')
      WRITE(OUTUNT,
     &      '(//,'' Abnormal Termination of Velrp1_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
C************* U.S. Geological Survey preliminary computer program ************
C******************************************************************************
C*************                 Pthvel version 6.0                 *************
C******************************************************************************
C**                          Language: Fortran 77                            **
C**   Program must be recompiled then link (no system libraries are needed)  **
C**                The source code is available from below:                  **
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C*     Author/Site,      Date,        Event
C*   --------------------------------------------------------------------------
C*   Leonard L. Orzol                                   Phone: (503) 251-3270
C*   U. S. Geological Survey - WRD                        FAX: (503) 251-3470
C*   10615 SE. Cherry Blossom Drive
C*   Portland, Oregon 97216 USA                    Internet: llorzol@usgs.gov
C*
C*=============================================================================
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*
C* Disclaimer: 
C            Although this program has been used by the U.S. Geological Survey,
C            no warranty, expressed or implied, is made by the USGS as to the
C            accuracy and functioning of the program and related program
C            material nor shall the fact of distribution constitute any such
C            warranty, and no responsibility is assumed by the USGS in
C            connection therewith.
C*
C*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C*=============================================================================
C
C
C---VERSION 6.0 PTHVEL 07NOVEMBER2000
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Purpose:
C
C     Generates a velocity vector line coverage using the cell-by-cell 
C      flow or flux terms that are output by the Block-Centered package of the 
C      U.S. Geological Survey Modular Three-Dimensional Finite-Differences
C      Ground-Water Flow Model MODFLOW (McDonald and Harbaugh, 1988).
C      Also, the cell-by-cell head terms are compared to the cell
C      elevations (top and bottom) to calculate a saturated area. 
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C             Line coverage structure:
C              => (n,1)  COVER-ID (character representation of 
C                                    row and column)
C              => (n,2)  CELL ROWCOL (character representation of 
C                                    row and column)
C              => (n,3)  CELL ROW
C              => (n,4)  CELL COLUMN
C              => (n,5)  LAYER
C              => (n,6)  SEQNUM
C              => (n,7)  IBOUND
C              => (n,8)  X-DIRECTION [RIGHT FACE]
C              => (n,9)  Y-DIRECTION [FRONT FACE]
C              => (n,10)  X-DIRECTION [LOWER FACE]
C              => (n,11)  X-Y SPEED [X-Y PLANE]
C              => (n,12) X-Z SPEED [X-Z PLANE]
C              => (n,13) Y-Z SPEED [Y-Z PLANE]
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C
C .. Language: FORTRAN
C
C
C    Inputs:
C
C     OUTUNT  : FORTRAN UNIT WHERE OPERATIONAL OUTPUT AND ERRORS ARE RECORDED
C     VELCOV  : ARC/INFO FILE NAME FOR VELOCITY VECTOR LINE COVERAGES
C     MAXATB  : MAXIMUM NUMBER OF ATTRIBUTES FOR EACH CELL VECTOR
C     MAXPMR  : MAXIMUM NUMBER OF COVERAGE PARAMETERS
C     MAXCEL  : MAXIMUM NUMBER OF VERTICES IN A SINGLE ARC
C     NLAY    : NUMBER OF LAYERS IN MODEL
C     NROW    : NUMBER OF ROWS IN GRID
C     NCOL    : NUMBER OF COLUMNS IN GRID
C     COVPMR  : COVERAGE PARAMETERS ARRAY (MAXPMR)
C     DELX    : ARRAY FOR CELL WIDTHS IN ROW/COLUMN/PARTICLE DIRECTION
C     DELZCB  : ARRAY FOR CONFINING LAYER THICKNESS
C     LRWCL   : ARRAY INDICATING THE CELL'S ROW AND COLUMN (2,NDELX)
C     IBOUND  : ARRAY FOR IBOUND ARRAY (NCOL,NROW,NLAY)
C     FRFBUD  : ARRAY FOR FLOW RIGHT FACE VALUES (NCOL,NROW,NLAY)
C     FFFBUD  : ARRAY FOR FLOW FRONT FACE VALUES (NCOL,NROW,NLAY)
C     FLFBUD  : ARRAY FOR FLOW LOWER FACE VALUES (NCOL,NROW,NLAY)
C     HEDBUD  : ARRAY FOR LAYER/UNIT CELL_BY_CELL HEADS
C     ZTOP    : ARRAY FOR LAYER/UNIT TOP ELEVATIONS
C     ZBOT    : ARRAY FOR LAYER/UNIT BOTTOM ELEVATIONS
C     CELVEL  : ARRAY FOR CELL VELOCITY VECTORS
C     PORSTY  : ARRAY FOR CELL POROSITY
C
C   Inputs/Outputs: None
C
C    Outputs: None
C
C-----------------------------------------------------------------------
C
      SUBROUTINE PTHVEL (
     I                   OUTUNT,
     I                   VELCOV,
     I                   MAXCEL,MAXPMR,
     I                   NLAY,NROW,NCOL,
     I                   COVPMR,LAYCON,
     I                   DELX,
     I                   DELZ,LAYCBD,
     I                   LRWCL,
     I                   IBOUND,
     I                   FRFBUD,FFFBUD,FLFBUD,
     I                   HEDBUD,ZTOP,ZBOT,
     I                   CELVEL,PORSTY,
     E                   *)
C-----------------------------------------------------------------------
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
C
C:::::::::::::::::::::::  GLOBAL_VARIABLES  ::::::::::::::::::::::::::::
C
      DIMENSION 
     1          COVPMR(MAXPMR),
     &          LAYCON(NLAY),
     1          DELX(NROW*NCOL),
     1          DELZ(NLAY),
     1          LAYCBD(NLAY),
     2          LRWCL(2,NROW*NCOL),
     2          CELVEL(MAXCEL),
     3          IBOUND(NCOL,NROW,NLAY),
     3          HEDBUD(NCOL,NROW,NLAY),
     3          FRFBUD(NCOL,NROW,NLAY),
     3          FFFBUD(NCOL,NROW,NLAY),
     3          FLFBUD(NCOL,NROW,NLAY),
     3          ZTOP(NCOL,NROW,NLAY),
     3          ZBOT(NCOL,NROW,NLAY),
     3          PORSTY(NCOL,NROW,NLAY)
C
      CHARACTER*(*) VELCOV
      INTEGER OUTUNT
C
C::::::::::::::::::::::::  PROGRAM_VARIABLES  ::::::::::::::::::::::::::
C
      PARAMETER (MAXFIL=4,
     &           MAXITM=13)
C
      CHARACTER*128 LABFIL,FMTSTR
      CHARACTER*80 FILES(MAXFIL),ADDFIL,INPFIL,TMPFIL,JONFIL,COVFIL
      CHARACTER*16 COVTYP,ITEM,ITEMS(MAXITM),ARCCMD,ACTION,
     &             LAYITM,STRITM,RELITM,JONTYP
      CHARACTER*10 CHRLAY,CHRWCL,LAYFMT,ITMFMT
      CHARACTER*1 TYPITM,TYPES(MAXITM)
C
      REAL
     &     HEDDRY,HNOFLO,HEDIBD,
     &     RIGFLO,FRTFLO,LOWFLO,
     &     RIGFAC,FRTFAC,LOWFAC,
     &     RIGIBD,FRTIBD,LOWIBD,
     &     XCENTR,YCENTR,XTOADD,YTOADD
C
      INTEGER DECIML,DECIMLS(MAXITM),IOUT(MAXFIL),
     &        OUTPUT,OUTPUTS(MAXITM),WIDTH,WIDTHS(MAXITM)
C
      INTEGER 
     &        IRWCL,IVIEW,
     &        IDPART,ICOLOR,TIMSTP,OLDSTP,
     &        STRCHR,STRCOL,STRROW,STRLAY,STRIBD,STRDEL,STRCLL,
     &        PRECOL,PREROW,PRELAY,PREIBD,
     &        FINCHR,FINCOL,FINROW,FINLAY,FINIBD,FINDEL,FINCLL,
     &        BADIBD,DRYCLL,BADRIG,BADFRT,BADLOW,BADTHK,BADFLO,BADTOT,
     &        ROW,COLUMN,CELL,COVDEX,COL,ROWCOL,SEQNUM,
     &        NLABS,
     &        NUMCOL,NUMROW,NUMLAY,
     &        IRWCOL
C
      EXTERNAL OPEN_FILE,CLOSE_FILE,DELETE_FILE
C
      DATA FILES /'veloitems',
     &            'veloitem',
     &            'arcs',
     &            'arccmd'/
      DATA IOUT /91,92,93,94/
      DATA ITEMS /'CELL','ROWCOL','ROW','COLUMN','LAYER','SEQNUM',
     &            'IBOUND','XVELOCITY','YVELOCITY','ZVELOCITY',
     &            'XY-SPEED','XZ-SPEED','YZ-SPEED'/
      DATA DECIMLS /0,6*0,6*10/
      DATA TYPES /'B','C',5*'I',6*'F'/
      DATA WIDTHS /4,6*10,6*8/
      DATA OUTPUTS /5,6*10,6*16/
C
      DATA TINY,THRESD /1.0E-36,1.0E-10/
C
C     ------------------------------------------------------------------
C
CH----Print Modtools header
C
      WRITE(OUTUNT,'(/10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''          VELOCITY GIS OUTPUT PACKAGE'')')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''             VERSION 6.1 05MAY2002 '')')
      WRITE(OUTUNT,'(10X,85(''*''))')
      WRITE(OUTUNT,'(10X,85(''-''))')
      WRITE(OUTUNT,'(30X,
     &               ''               GIS OUTPUT MODULE '')')
      WRITE(OUTUNT,'(10X,85(''=''))')
      WRITE(OUTUNT,'(//)')
C
C1----Set local variables
C
      XYSCALE=COVPMR(3)
      ZSCALE=COVPMR(4)
      ANGLE=COVPMR(5)
      XOGRID=COVPMR(6)
      YOGRID=COVPMR(7)
      IF(COVPMR(9).LE.0.0) COVPMR(9)=1.0
      HSCALE=12.0/COVPMR(9)
      IF(COVPMR(8).LE.0.0) COVPMR(8)=1.0
      VSCALE=HSCALE*COVPMR(8)
      IVIEW=INT(COVPMR(10))
      IRWCL=INT(COVPMR(11))
      VECTOR=COVPMR(12)
      HEDDRY=COVPMR(15)
      HNOFLO=COVPMR(16)
      YMIN=COVPMR(MAXPMR)
C
C2----Delete and create files for attribute definitions, attribute value, 
C      and coverage arc data and tolerance/dangle values
C
       NSTAT=2
       ITALK=1
C      FILES(2) = VELCOV(1:INDEX(VELCOV,' ')-1)
C
C2A----Delete/create files containing Arc/Info item definitions for
C      attribute files
C
      DO 200 N=1,MAXFIL-1
         CALL DELETE_FILE (FILES(N),*9999)
         CALL OPEN_FILE (IOUT(N),FILES(N),NSTAT,ITALK,*9999)
  200 CONTINUE
C
C2B----Calculate rowcol attribute definition
C
      IF(NROW.LT.10) THEN
         LENROW=1
      ELSE IF(NROW.GT.9 .AND. NROW.LT.100) THEN
         LENROW=2
      ELSE IF(NROW.GT.99 .AND. NROW.LT.1000) THEN
         LENROW=3
      ELSE
         LENROW=4
      ENDIF
C
      IF(NCOL.LT.10) THEN
         LENCOL=1
      ELSE IF(NCOL.GT.9 .AND. NCOL.LT.100) THEN
         LENCOL=2
      ELSE IF(NCOL.GT.99 .AND. NCOL.LT.1000) THEN
         LENCOL=3
      ELSE
         LENCOL=4
      ENDIF
C
      IF(NLAY.LT.10) THEN
         LENLAY=1
      ELSE IF(NLAY.GT.9 .AND. NLAY.LT.100) THEN
         LENLAY=2
      ELSE IF(NLAY.GT.99 .AND. NLAY.LT.1000) THEN
         LENLAY=3
      ELSE
         LENLAY=4
      ENDIF
C
      IF((NROW*NCOL).LT.10) THEN
         LENSEQ=1
      ELSE IF((NROW*NCOL).GT.9 .AND. (NROW*NCOL).LT.100) THEN
         LENSEQ=2
      ELSE IF((NROW*NCOL).LT.1000) THEN
         LENSEQ=3
      ELSE IF((NROW*NCOL).LT.10000) THEN
         LENSEQ=4
      ELSE IF((NROW*NCOL).LT.100000) THEN
         LENSEQ=5
      ELSE IF((NROW*NCOL).LT.100000) THEN
         LENSEQ=6
      ELSE IF((NROW*NCOL).LT.1000000) THEN
         LENSEQ=7
      ELSE IF((NROW*NCOL).LT.10000000) THEN
         LENSEQ=8
      ELSE IF((NROW*NCOL).LT.100000000) THEN
         LENSEQ=9
      ELSE IF((NROW*NCOL).LT.1000000000) THEN
         LENSEQ=10
      ELSE
         LENSEQ=15
      ENDIF
C
      WIDTHS(2)=LENROW+LENCOL
      OUTPUTS(2)=LENROW+LENCOL
      WIDTHS(3)=LENROW
      OUTPUTS(3)=LENROW
      WIDTHS(4)=LENCOL
      OUTPUTS(4)=LENCOL
      WIDTHS(5)=LENLAY
      OUTPUTS(5)=LENLAY
      WIDTHS(6)=LENSEQ
      OUTPUTS(6)=LENSEQ
C
C2C----Write standard attribute definitions
C
      DO 210 K=1,MAXITM
         ITEM=ITEMS(K)
       IF(K.EQ.1) ITEM=VELCOV (1:INDEX(VELCOV,' ')-1)//'-id'
         DECIML=DECIMLS(K)
         OUTPUT=OUTPUTS(K)
         TYPITM=TYPES(K)
         WIDTH=WIDTHS(K)
         WRITE(IOUT(1),'(A,'','',I2,'','',I2,'','',A1,'','',
     &                   I2,'',,,,,,,,,,'')')
     &                                 ITEM,
     &                                 WIDTH,
     &                                 OUTPUT,
     &                                 TYPITM,
     &                                 DECIML
  210 CONTINUE
      CALL CLOSE_FILE (FILES(1),*9999)
C
C3----Build mapview for single layer (iview=0,4)
C
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) THEN
       IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
          WRITE(*,'(''      .Processing velocity vectors '',
     &             ''for layer '',I10,'' now...'')')
     &          IRWCL
       ELSE IF(IVIEW.EQ.5) THEN
         WRITE(*,'(''      .Processing velocity vectors '',
     &             ''for ibound value '',I10,'' now...'')')
     &          IRWCL
       ELSE
         WRITE(*,'(''      .Processing velocity vectors '',
     &             ''for uppermost active cell now...'')')
       ENDIF
C
C3H----Data header
C
       IF(IVIEW.LE.0 .OR. IVIEW.EQ.4) THEN
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS CONDITIONS FOR SPECIFIED LAYER '',
     &               /10X,85(''-''))')
       ELSE IF(IVIEW.EQ.5) THEN
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS CONDITIONS FOR SPECIFIED IBOUND'',
     &               /10X,85(''-''))')
       ELSE
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS CONDITIONS FOR UPPERMOST ACTIVE CELL'',
     &               /10X,85(''-''))')
       ENDIF
C
         WRITE(OUTUNT,'(40X,
     &               ''       ROW'',
     &               ''    COLUMN'',
     &               ''     LAYER'',
     &               ''    IBOUND'',
     &               /10X,85(''-''))')
C
C3Cb----Cell flow attributes
C
         NCELLS=0
         BADIBD=0
         DRYCLL=0
         NOFLOW=0
C
       IF(IVIEW.EQ.0 .OR. IVIEW.EQ.4) THEN
          MINLAY=IRWCL
          MAXLAY=IRWCL
       ELSE
          MINLAY=1
          MAXLAY=NLAY
       ENDIF
C
         XVLMAX=0.0
         XVLMIN=99E18
         YVLMAX=0.0
         YVLMIN=99E18
         ZVLMAX=0.0
         ZVLMIN=99E18
         SPDMAX=0.0
         SPDMIN=99E18
C
         IROW=0
         DO 300 ROW=NCOL+1,NCOL+NROW
            IROW=IROW+1
            DO 310 COL=1,NCOL
               ICOL=COL
               NCELLS=NCELLS+1
               MCELLS=1+(NCELLS-1)*7
               NUMCLL=(IROW-1)*NCOL+ICOL
               ROWCOL=IROW*(10**LENCOL)+ICOL
               WRITE(CHRWCL,'(I10)') ROWCOL
               FINCHR=10
               STRCHR=FINCHR-(LENROW+LENCOL-1)
C
               SATTHK=0.0
               TOTTHK=0.0
C
               DO 320 ILAY=MINLAY,MAXLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
                IF(IBDCEL.EQ.0) THEN
                   BADIBD=BADIBD+1
                   WRITE(OUTUNT,'(10X,
     &                     ''INACTIVE CELL                 '',
     &                     4I10)')
     &                             IROW,
     &                             ICOL,
     &                             ILAY,
     &                             IBDCEL
                   GO TO 320
                ENDIF
                IF(IVIEW.EQ.5 .AND. IBDCEL.NE.IRWCL) THEN
                   WRITE(OUTUNT,'(10X,
     &                     ''WRONG CELL IBOUND             '',
     &                     4I10)')
     &                             IROW,
     &                             ICOL,
     &                             ILAY,
     &                             IBDCEL
                   GO TO 320
                ENDIF
                IF(HEDBUD(ICOL,IROW,ILAY).EQ.HEDDRY) THEN
                   DRYCLL=DRYCLL+1
                   WRITE(OUTUNT,'(10X,
     &                     ''DRY CELL                      '',
     &                     4I10)')
     &                             IROW,
     &                             ICOL,
     &                             ILAY,
     &                             IBDCEL
                   IBOUND(ICOL,IROW,ILAY)=0
                   GO TO 320
                ENDIF
                IF(HEDBUD(ICOL,IROW,ILAY).EQ.HNOFLO) THEN
                   NOFLOW=NOFLOW+1
                   WRITE(OUTUNT,'(10X,
     &                     ''NOFLOW CELL                   '',
     &                     4I10)')
     &                             IROW,
     &                             ICOL,
     &                             ILAY,
     &                             IBDCEL
                   IBOUND(ICOL,IROW,ILAY)=0
                   GO TO 320
                ENDIF
C
                  WRITE(OUTUNT,'(10X,
     &                    ''CELL                          '',
     &                    4I10)')
     &                            IROW,
     &                            ICOL,
     &                            ILAY,
     &                            IBDCEL
C
                  TOP=ZTOP(ICOL,IROW,ILAY)
                  BOT=ZBOT(ICOL,IROW,ILAY)
                  HEDIBD=HEDBUD(ICOL,IROW,ILAY)
C
                  WRITE(OUTUNT,'(10X,
     &                    ''     HEAD  TOP  BOTTOM        '',
     &                            3G10.3)')
     &                            HEDIBD,
     &                            TOP,
     &                            BOT
C
                IF(LAYCON(ILAY).EQ.0 .OR. LAYCON(ILAY).EQ.2) HEDIBD=TOP
C
                IF(HEDIBD.GE.TOP) THEN
                   SATTHK=SATTHK+TOP-BOT
                ELSE IF(HEDIBD.LE.BOT) THEN
                   BADTHK=BADTHK+1
                ELSE
                   SATTHK=SATTHK+HEDIBD-BOT
                ENDIF
C
                IF(IVIEW.NE.5) GO TO 312
C
 320           CONTINUE
C
 312           TOTTHK=SATTHK
               SATTHK=0.0
               RIGFLO=0.0
               FRTFLO=0.0
               LOWFLO=0.0
               XGCVEL=0.0
               YGCVEL=0.0
               ZGCVEL=0.0
               XYSPED=0.0
               XZSPED=0.0
               YZSPED=0.0
C
               DO 330 ILAY=MINLAY,MAXLAY
                  IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
C
                IF(IVIEW.EQ.4 .AND. IBDCEL.EQ.0) GO TO 314
                IF(IVIEW.EQ.5 .AND. IBDCEL.NE.IRWCL) GO TO 330
                IF(IBDCEL.EQ.0) GO TO 330
                IF(HEDBUD(ICOL,IROW,ILAY).EQ.HEDDRY) GO TO 330
                IF(HEDBUD(ICOL,IROW,ILAY).EQ.HNOFLO) GO TO 330
C
                  TOP=ZTOP(ICOL,IROW,ILAY)
                  BOT=ZBOT(ICOL,IROW,ILAY)
                  HEDIBD=HEDBUD(ICOL,IROW,ILAY)
C
                IF(LAYCON(ILAY).EQ.0 .OR. LAYCON(ILAY).EQ.2) HEDIBD=TOP
C
                IF(HEDIBD.GE.TOP) THEN
                   SATTHK=TOP-BOT
                ELSE IF(HEDIBD.LE.BOT) THEN
                   BADTHK=BADTHK+1
                ELSE
                   SATTHK=HEDIBD-BOT
                ENDIF
C
                  RIGFAC=SATTHK*DELX(ROW)
                  FRTFAC=SATTHK*DELX(COL)
                  LOWFAC=DELX(ROW)*DELX(COL)
                  WRITE(OUTUNT,'(10X,
     &                     ''     FACE WIDTH               '',
     &                            3G10.3)')
     &                             DELX(ROW),
     &                             DELX(COL),
     &                             LOWFAC
                  WRITE(OUTUNT,'(10X,
     &                     ''     FACE SATURATED AREA      '',
     &                            5G10.3)')
     &                             RIGFAC,
     &                             FRTFAC,
     &                             LOWFAC,
     &                             SATTHK,
     &                             TOTTHK
C
                IF(ICOL.NE.1) THEN
                   RIGFLO=RIGFLO+(FRFBUD(ICOL,IROW,ILAY)+
     &                    FRFBUD(ICOL-1,IROW,ILAY))/2.0
                ELSE
                   RIGFLO=RIGFLO+FRFBUD(ICOL,IROW,ILAY)/2.0
                ENDIF
C
                IF(RIGFLO.EQ.0.) BADRIG=BADRIG+1                
C
                IF(IROW.NE.1) THEN
                   FRTFLO=FRTFLO+(-FFFBUD(ICOL,IROW,ILAY)-
     &                    FFFBUD(ICOL,IROW-1,ILAY))/2.0
                ELSE
                   FRTFLO=FRTFLO-FFFBUD(ICOL,IROW,ILAY)/2.0
                ENDIF
C
                IF(FRTFLO.EQ.0.) BADFRT=BADFRT+1
C
                IF(ILAY.NE.1) THEN
                   LOWFLO=LOWFLO+(-FLFBUD(ICOL,IROW,ILAY)-
     &                     FLFBUD(ICOL,IROW,ILAY-1))/2.0
                ELSE
                   LOWFLO=LOWFLO-FLFBUD(ICOL,IROW,ILAY)/2.0
                ENDIF
C
                IF(LOWFLO.EQ.0.) BADLOW=BADLOW+1
C
                IF(RIGFLO.EQ.0. .AND. FRTFLO.EQ.0.) BADFLO=BADFLO+1
C
                  WRITE(OUTUNT,'(10X,
     &                     ''     FLOW                     '',
     &                            3G10.3)')
     &                             RIGFLO,
     &                             FRTFLO,
     &                             LOWFLO
C
                IF(IVIEW.EQ.5) THEN
                   RIGFLO=RIGFLO*SATTHK/TOTTHK
                   FRTFLO=FRTFLO*SATTHK/TOTTHK
                   WRITE(OUTUNT,'(10X,
     &                     ''     FLOW                     '',
     &                     ''     WEIGHTED FLOW            '',
     &                            3G10.3)')
     &                             RIGFLO,
     &                             FRTFLO,
     &                             LOWFLO
                ENDIF
C
                IF(RIGFAC.NE.0.0) THEN
                   XGCVEL=XGCVEL+
     &                    RIGFLO/PORSTY(ICOL,IROW,ILAY)/RIGFAC
                ELSE
                   XGCVEL=0.0
                ENDIF
                IF(FRTFAC.NE.0.0) THEN
                   YGCVEL=YGCVEL+
     &                    FRTFLO/PORSTY(ICOL,IROW,ILAY)/FRTFAC
                ELSE
                   YGCVEL=0.0
                ENDIF
                IF(LOWFAC.NE.0.0) THEN
                   ZGCVEL=LOWFLO/PORSTY(ICOL,IROW,ILAY)/LOWFAC
                ELSE
                   ZGCVEL=0.0
                ENDIF
C
                IF(IVIEW.NE.5) GO TO 314
C
 330           CONTINUE
C
 314           WRITE(OUTUNT,'(10X,
     &                  ''     FACE VELOCITIES          '',
     &                         3G10.3)')
     &                          XGCVEL,
     &                          YGCVEL,
     &                          ZGCVEL
C
               XYSPED=(XGCVEL**2.0+YGCVEL**2.0)**0.5
               XZSPED=(XGCVEL**2.0+ZGCVEL**2.0)**0.5
               YZSPED=(YGCVEL**2.0+ZGCVEL**2.0)**0.5
               WRITE(OUTUNT,'(10X,
     &                  ''     XYFACE SPEED             '',
     &                         F16.8)')
     &                          XYSPED
C
             IF(XGCVEL.GT.XVLMAX) XVLMAX=XGCVEL
             IF(XGCVEL.LT.XVLMIN) XVLMIN=XGCVEL
             IF(YGCVEL.GT.YVLMAX) YVLMAX=YGCVEL
             IF(YGCVEL.LT.YVLMIN) YVLMIN=YGCVEL
             IF(ZGCVEL.GT.ZVLMAX) ZVLMAX=ZGCVEL
             IF(ZGCVEL.LT.ZVLMIN) ZVLMIN=ZGCVEL
             IF(XYSPED.GT.SPDMAX) THEN
                SPDMAX=XYSPED
                MAXROW=IROW
                MAXCOL=ICOL
             ENDIF
             IF(XYSPED.LT.SPDMIN .AND. XYSPED.GT.TINY) SPDMIN=XYSPED
C
               CELVEL(MCELLS)=XGCVEL
               CELVEL(MCELLS+1)=YGCVEL
               CELVEL(MCELLS+2)=ZGCVEL
               CELVEL(MCELLS+3)=XYSPED
               CELVEL(MCELLS+4)=XZSPED
               CELVEL(MCELLS+5)=YZSPED
               CELVEL(MCELLS+6)=FLOAT(ILAY)
               CELVEL(MCELLS+7)=FLOAT(IBDCEL)
  310       CONTINUE
  300    CONTINUE
C
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C3----Determine Log tranformation for values > 1
C
      IF(ABS(VECTOR).LE.TINY) VECTOR=1.0
       OFFLOG=ALOG10(SPDMIN)
        LOGFLG=0
      IF(VECTOR.LT.0.0) THEN
         VECTOR=ABS(VECTOR)
         LOGFLG=1
      ENDIF
c
      IF(OFFLOG.LT.0.0 .AND. LOGFLG.GT.0) THEN
         OFFLOG=10.**AINT(ABS(OFFLOG)+1.5)
      ELSE
         OFFLOG=1.0
      ENDIF
C
C3Cb----Normalize velocities
C
         XLEFT=0.0
         NCELLS=0
         IROW=0
         YLEFT=0.0
C
       IF(IVIEW.EQ.0 .OR. IVIEW.EQ.4) THEN
          ILAY=IRWCL
       ELSE
          ILAY=0
       ENDIF
C
C3H----Data header
C
       IF(IVIEW.EQ.4) THEN
         WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELL VELOCITY IN MAPVIEW'',
     &               '' FOR LAYER '',
     &               I10, /10X,85(''-''))') IRWCL
       ELSE IF(IVIEW.EQ.5) THEN
         WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELL VELOCITY IN MAPVIEW'',
     &               '' FOR SPECIFIED IBOUND '',
     &               I10, /10X,85(''-''))') IRWCL
       ELSE
         WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELL VELOCITY IN MAPVIEW'',
     &               '' FOR UPPERMOST ACTIVE CELL '',
     &               /10X,130(''-''))')
       ENDIF
C
         WRITE(OUTUNT,'(10X,
     &               ''       ROW'',
     &               ''    COLUMN'',
     &               ''     LAYER'',
     &               ''    IBOUND'',
     &               ''         X'',
     &               ''         Y'',
     &               ''         Z'',
     &               ''       X-Y'',
     &               ''       X-Y'',
     &               ''    FACTOR'',
     &               ''       X-Y''))')
         WRITE(OUTUNT,'(50X,
     &               ''  VELOCITY'',
     &               ''  VELOCITY'',
     &               ''  VELOCITY'',
     &               ''     SPEED'',
     &               ''  DISTANCE'',
     &               ''          '',
     &               ''     ANGLE'',
     &               /10X,130(''-''))')
C
C3Cb----Loop to calculate vectors
C
         DO 340 ROW=NCOL+1,NCOL+NROW
            IROW=IROW+1
            XLEFT=0.0
            YLABEL=YLEFT-1.0*DELX(ROW)*XYSCALE*0.5
            YLEFT=YLEFT-1.0*DELX(ROW)*XYSCALE
            DO 350 COL=1,NCOL
               NCELLS=NCELLS+1
               MCELLS=1+(NCELLS-1)*7
               ICOL=COL
               XCENTR=XLEFT+DELX(COL)*XYSCALE*0.5
               XLEFT=XLEFT+DELX(COL)*XYSCALE
               YCENTR=YLABEL
C
               XGCVEL=CELVEL(MCELLS)
               YGCVEL=CELVEL(MCELLS+1)
               ZGCVEL=CELVEL(MCELLS+2)
               XYSPED=CELVEL(MCELLS+3)
               XZSPED=CELVEL(MCELLS+4)
               YZSPED=CELVEL(MCELLS+5)
               ILAY=INT(CELVEL(MCELLS+6))
               IBDCEL=INT(CELVEL(MCELLS+7))
               FACTLOG=0.0
               XYDIST=0.0
               XYGANG=0.0
C
             IF(XYSPED.LE.0.0) THEN
                NOSPED=NOSPED+1
             ELSE
                SEQNUM=(IROW-1)*NCOL+ICOL
                ROWCOL=IROW*(10**LENCOL)+ICOL
                WRITE(CHRWCL,'(I10)') ROWCOL
                FINCHR=10
                STRCHR=FINCHR-(LENROW+LENCOL-1)
C
              IF(LOGFLG.GT.0) THEN
                 FACTLG=VECTOR*ALOG10(XYSPED*OFFLOG)/XYSPED
              ELSE
                 FACTLG=VECTOR
              ENDIF
C
                XYDIST=FACTLG*XYSPED
C
              IF(ABS(XGCVEL).GT.0.0) THEN
                 XYVELO=ABS(YGCVEL/XGCVEL)
                 XYGRAD=ATAN(XYVELO)
                 XYGANG=XYGRAD/0.01745329
              ELSE
                 XYGANG=0.0
              ENDIF
C
              IF(XGCVEL.GT.0.0) THEN
                 IXFLAG=1
               IF(YGCVEL.GT.0.0) THEN
                  IYFLAG=1
                  XTOADD=XYDIST*COS(XYGRAD)
                  YTOADD=XYDIST*SIN(XYGRAD)
               ELSE IF(YGCVEL.LT.0.0) THEN
                  IYFLAG=-1
                  XTOADD=XYDIST*COS(XYGRAD)
                  YTOADD=-XYDIST*SIN(XYGRAD)
               ELSE
                  IYFLAG=0
                  XTOADD=XYDIST
                  YTOADD=0.0
               ENDIF
              ELSE IF(XGCVEL.LT.0.0) THEN
                 IXFLAG=-1
               IF(YGCVEL.GT.0.0) THEN
                  IYFLAG=1
                  XTOADD=-XYDIST*COS(XYGRAD)
                  YTOADD=XYDIST*SIN(XYGRAD)
               ELSE IF(YGCVEL.LT.0.0) THEN
                  IYFLAG=-1
                  XTOADD=-XYDIST*COS(XYGRAD)
                  YTOADD=-XYDIST*SIN(XYGRAD)
               ELSE
                  IYFLAG=0
                  XTOADD=-XYDIST
                  YTOADD=0.0
               ENDIF
              ELSE
                 IXFLAG=0
               IF(YGCVEL.GT.0.0) THEN
                  IYFLAG=1
                  XTOADD=0.0
                  YTOADD=XYDIST
               ELSE IF(YGCVEL.LT.0.0) THEN
                  IYFLAG=-1
                  XTOADD=0.0
                  YTOADD=-XYDIST
               ELSE
                  IYFLAG=0
                  GO TO 350
               ENDIF
              ENDIF
C
                XTO=XCENTR+XTOADD
                XFROM=XCENTR-XTOADD
                YTO=YCENTR+YTOADD
                YFROM=YCENTR-YTOADD
              IF(ANGLE.NE.0.0) THEN
                  CALL ROTATE (
     I                         ANGLE,
     B                         XTO,YTO)
                  CALL ROTATE (
     I                         ANGLE,
     B                         XFROM,YFROM)
              ENDIF
C                  
                XTO=XTO+XOGRID
                YTO=YTO+YOGRID
                XFROM=XFROM+XOGRID
                YFROM=YFROM+YOGRID
C
                XGCDIF=ABS(XTO-XFROM)
                YGCDIF=ABS(YTO-YFROM)
              IF(XGCDIF.LE.0.0001 .AND. YGCDIF.LE.0.0001) THEN
                 NOVECT=NOVECT+1
                 WRITE(OUTUNT,'(10X,
     &                  4I10,
     &                  '' ##---> CONCIDENT VECTOR IN CELL <---##'')')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                 GO TO 350
              ENDIF
C
                WRITE(IOUT(3),'(A)')
     &                       CHRWCL (STRCHR:FINCHR)
                WRITE(IOUT(3),'(F20.10,'','',F20.10)')
     &                       XFROM,
     &                       YFROM
                WRITE(IOUT(3),'(F20.10,'','',F20.10)')
     &                       XTO,
     &                       YTO
                WRITE(IOUT(3),'(''END'')')
C
              IF(ABS(XGCVEL).LE.THRESD .AND. XGCVEL.NE.0.) 
     &          XGCVEL=SIGN(THRESD,XGCVEL)
              IF(ABS(YGCVEL).LE.THRESD .AND. YGCVEL.NE.0.) 
     &          YGCVEL=SIGN(THRESD,YGCVEL)
              IF(ABS(ZGCVEL).LE.THRESD .AND. ZGCVEL.NE.0.) 
     &          ZGCVEL=SIGN(THRESD,ZGCVEL)
C
              IF(XYSPED.LE.THRESD .AND. XYSPED.NE.0.) 
     &          XYSPED=SIGN(THRESD,XYSPED)
              IF(XZSPED.LE.THRESD .AND. XZSPED.NE.0.) 
     &          XZSPED=SIGN(THRESD,XZSPED)
              IF(YZSPED.LE.THRESD .AND. YZSPED.NE.0.) 
     &          YZSPED=SIGN(THRESD,YZSPED)
C
                WRITE(IOUT(2),'(
     &                          A,'','',
     &                          A,'','',              
     &                          5(I10,'',''),
     &                          6(F16.10,'',''))')
     &                       CHRWCL (STRCHR:FINCHR),
     &                       CHRWCL (STRCHR:FINCHR),
     &                       IROW,
     &                       ICOL,
     &                       ILAY,
     &                       SEQNUM,
     &                       IBDCEL,
     &                       XGCVEL,
     &                       YGCVEL,
     &                       ZGCVEL,
     &                       XYSPED,
     &                       XZSPED,
     &                       YZSPED
             ENDIF
C
               WRITE(OUTUNT,'(10X,
     &                        4I10,
     &                        6(1X,G10.3),
     &                        1X,F10.3)')
     &                      IROW,
     &                      ICOL,
     &                      ILAY,
     &                      IBDCEL,
     &                      XGCVEL,
     &                      YGCVEL,
     &                      ZGCVEL,
     &                      XYSPED,
     &                      XYDIST,
     &                      FACTLG,
     &                      XYGANG
C
               WRITE(OUTUNT,'(30X,
     &                        8(1X,F15.2))')
     &                      XCENTR,
     &                      YCENTR,
     &                      XTOADD,
     &                      YTOADD,
     &                      XTO,
     &                      YTO,
     &                      XFROM,
     &                      YFROM
               WRITE(OUTUNT,'(/)')
  350       CONTINUE
  340    CONTINUE
C
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C4----Build cross-sectional view (iview=1,2,3)
C
      ELSE IF(IVIEW.GE.1 .AND. IVIEW.LE.3) THEN
       IF(IVIEW.EQ.1) THEN
          WRITE(*,'(''      .Processing velocity vectors '',
     &             ''along row '',I10,'' now...'')')
     &          IRWCL
       ELSE IF(IVIEW.EQ.2) THEN
          WRITE(*,'(''      .Processing velocity vectors '',
     &             ''along column '',I10,'' now...'')')
     &          IRWCL
       ELSE
          WRITE(*,'(''      .Processing velocity vectors '',
     &             ''along particle '',I10,'' now...'')')
     &          IRWCL
       ENDIF
C
C4H----Data header
C
       IF(IVIEW.EQ.1) THEN
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS CONDITIONS ALONG ROW DIRECTION'',
     &               /10X,85(''-''))')
       ELSE IF(IVIEW.EQ.2) THEN
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS CONDITIONS ALONG COLUMN DIRECTION'',
     &               /10X,85(''-''))')
       ELSE
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS CONDITIONS ALONG PARTICLE DIRECTION'',
     &               /10X,85(''-''))')
       ENDIF
         WRITE(OUTUNT,'(40X,
     &               ''       ROW'',
     &               ''    COLUMN'',
     &               ''     LAYER'',
     &               ''    IBOUND'',
     &               /10X,85(''-''))')
C
C4A----Set variables
C
         NCELLS=0
         BADIBD=0
         DRYCLL=0
         NOFLOW=0
         NUMCLL=0
         NCON=0
         CLLMIN=COVPMR(MAXPMR)
C
         XVLMAX=0.0
         XVLMIN=99E18
         YVLMAX=0.0
         YVLMIN=99E18
         ZVLMAX=0.0
         ZVLMIN=99E18
         SPDMAX=0.0
         SPDMIN=99E18
C
       IF(IVIEW.EQ.1) THEN
          STRCLL=1
          FINCLL=NCOL
       ELSE IF(IVIEW.EQ.2) THEN
          STRCLL=NCOL+1
          FINCLL=NCOL+NROW
       ELSE
          STRCLL=1
          FINCLL=IRWCL
       ENDIF
C
C4B----Loop to build cell arcs from elevation and row/column data
C
         DO 400 N=STRCLL,FINCLL
            NUMCLL=NUMCLL+1
          IF(IVIEW.LE.1) THEN
             IROW=IRWCL
             ICOL=NUMCLL
             ROW=NCOL+IRWCL
             COL=NUMCLL
          ELSE IF(IVIEW.EQ.2) THEN
             IROW=NUMCLL
             ICOL=IRWCL
             ROW=NCOL+NUMCLL
             COL=IRWCL
          ELSE
             ICOL=LRWCL(1,NUMCLL)
             IROW=LRWCL(2,NUMCLL)
             ROW=NCOL+IRWCL
             COL=NUMCLL
          ENDIF
C
C4Ba----Write attributes of each cell
C
            DO 410 ILAY=1,NLAY
               NCELLS=NCELLS+1
               MCELLS=1+(NCELLS-1)*7
               XGCVEL=0.0
               YGCVEL=0.0
               ZGCVEL=0.0
               XYSPED=0.0
               XZSPED=0.0
               YZSPED=0.0
C
               IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
             IF(IBDCEL.EQ.0) THEN
                BADIBD=BADIBD+1
                WRITE(OUTUNT,'(10X,
     &                  ''IBOUND = 0                    '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                GO TO 412
             ENDIF
             IF(HEDBUD(ICOL,IROW,ILAY).EQ.HEDDRY) THEN
                DRYCLL=DRYCLL+1
                WRITE(OUTUNT,'(10X,
     &                  ''DRY CELL                      '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                IBOUND(ICOL,IROW,ILAY)=0
                GO TO 412
             ENDIF
             IF(HEDBUD(ICOL,IROW,ILAY).EQ.HNOFLO) THEN
                NOFLOW=NOFLOW+1
                WRITE(OUTUNT,'(10X,
     &                  ''NOFLOW CELL                   '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                IBOUND(ICOL,IROW,ILAY)=0
                GO TO 412
             ENDIF
C
                WRITE(OUTUNT,'(10X,
     &                  ''CELL                          '',
     &                  4I10)')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
C
                IF(ICOL.NE.1) THEN
                   RIGFLO=(FRFBUD(ICOL,IROW,ILAY)+
     &                     FRFBUD(ICOL-1,IROW,ILAY))/2.0
                ELSE
                   RIGFLO=FRFBUD(ICOL,IROW,ILAY)/2.0
                ENDIF
C
                IF(RIGFLO.EQ.0.) BADRIG=BADRIG+1                
C
                IF(IROW.NE.1) THEN
                   FRTFLO=(FFFBUD(ICOL,IROW,ILAY)+
     &                     FFFBUD(ICOL,IROW-1,ILAY))/2.0
                ELSE
                   FRTFLO=FFFBUD(ICOL,IROW,ILAY)/2.0
                ENDIF
C
                IF(FRTFLO.EQ.0.) BADFRT=BADFRT+1
C
                IF(ILAY.NE.1) THEN
                   LOWFLO=(-FLFBUD(ICOL,IROW,ILAY)-
     &                     FLFBUD(ICOL,IROW,ILAY-1))/2.0
                ELSE
                   LOWFLO=-FLFBUD(ICOL,IROW,ILAY)/2.0
                ENDIF
C
                IF(LOWFLO.EQ.0.) BADLOW=BADLOW+1
C
             IF(IVIEW.LE.1 .AND.
     &          RIGFLO.EQ.0. .AND. LOWFLO.EQ.0.) BADFLO=BADFLO+1
               
             IF(IVIEW.GE.2 .AND.
     &          FRTFLO.EQ.0. .AND. LOWFLO.EQ.0.) BADFLO=BADFLO+1
C
               WRITE(OUTUNT,'(10X,
     &                  ''     FLOW                     '',
     &                         3G10.3)')
     &                          RIGFLO,
     &                          FRTFLO,
     &                          LOWFLO
C
               TOP=ZTOP(ICOL,IROW,ILAY)
               BOT=ZBOT(ICOL,IROW,ILAY)
               HEDIBD=HEDBUD(ICOL,IROW,ILAY)
C
             IF(LAYCON(ILAY).EQ.0 .OR. LAYCON(ILAY).EQ.2) HEDIBD=TOP
C
             IF(HEDIBD.GE.TOP) THEN
                SATTHK=TOP-BOT
             ELSE IF(HEDIBD.LE.BOT) THEN
                SATTHK=0.0
                BADTHK=BADTHK+1
             ELSE
                SATTHK=HEDIBD-BOT
             ENDIF
C
               RIGFAC=SATTHK*DELX(ROW)
               FRTFAC=SATTHK*DELX(COL)
               LOWFAC=DELX(ROW)*DELX(COL)
               WRITE(OUTUNT,'(10X,
     &                  ''     FACE WIDTH               '',
     &                         3G10.3)')
     &                          DELX(ROW),
     &                          DELX(COL),
     &                          SATTHK
               WRITE(OUTUNT,'(10X,
     &                  ''     FACE SATURATED AREA      '',
     &                         4G10.3)')
     &                          RIGFAC,
     &                          FRTFAC,
     &                          LOWFAC,
     &                          SATTHK
C
             IF(ABS(RIGFAC).NE.0.0) THEN
                XGCVEL=RIGFLO/PORSTY(ICOL,IROW,ILAY)/RIGFAC
             ELSE
                XGCVEL=0.0
             ENDIF
             IF(ABS(FRTFAC).NE.0.0) THEN
                YGCVEL=FRTFLO/PORSTY(ICOL,IROW,ILAY)/FRTFAC
             ELSE
                YGCVEL=0.0
             ENDIF
             IF(ABS(LOWFAC).NE.0.0) THEN
                ZGCVEL=LOWFLO/PORSTY(ICOL,IROW,ILAY)/LOWFAC
             ELSE
                ZGCVEL=0.0
             ENDIF
C
               WRITE(OUTUNT,'(10X,
     &                  ''     FACE VELOCITIES          '',
     &                         4G10.3)')
     &                          XGCVEL,
     &                          YGCVEL,
     &                          ZGCVEL,
     &                          PORSTY(ICOL,IROW,ILAY)
C
               XYSPED=(XGCVEL**2.0+YGCVEL**2.0)**0.5
               XZSPED=(XGCVEL**2.0+ZGCVEL**2.0)**0.5
               YZSPED=(YGCVEL**2.0+ZGCVEL**2.0)**0.5
C
             IF(IVIEW.EQ.1) THEN
                WRITE(OUTUNT,'(10X,
     &                  ''     XZFACE SPEED             '',
     &                         F16.8)')
     &                          XZSPED
             ELSE
                WRITE(OUTUNT,'(10X,
     &                  ''     YZFACE SPEED             '',
     &                         F16.8)')
     &                          YZSPED
             ENDIF
C
             IF(XGCVEL.GT.XVLMAX) XVLMAX=XGCVEL
             IF(XGCVEL.LT.XVLMIN) XVLMIN=XGCVEL
             IF(YGCVEL.GT.YVLMAX) YVLMAX=YGCVEL
             IF(YGCVEL.LT.YVLMIN) YVLMIN=YGCVEL
             IF(ZGCVEL.GT.ZVLMAX) ZVLMAX=ZGCVEL
             IF(ZGCVEL.LT.ZVLMIN) ZVLMIN=ZGCVEL
             IF(XYSPED.GT.SPDMAX) THEN
                SPDMAX=XYSPED
                MAXROW=IROW
                MAXCOL=ICOL
             ENDIF
C
             IF(IVIEW.EQ.1) THEN
              IF(XZSPED.LT.SPDMIN .AND. XZSPED.GT.TINY) SPDMIN=XZSPED
             ELSE
              IF(YZSPED.LT.SPDMIN .AND. YZSPED.GT.TINY) SPDMIN=YZSPED
             ENDIF
C
  412          CELVEL(MCELLS)=XGCVEL
               CELVEL(MCELLS+1)=YGCVEL
               CELVEL(MCELLS+2)=ZGCVEL
               CELVEL(MCELLS+3)=XYSPED
               CELVEL(MCELLS+4)=XZSPED
               CELVEL(MCELLS+5)=YZSPED
  410       CONTINUE
  400    CONTINUE
C
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C4----Determine Log tranformation for values > 1
C
      IF(ABS(VECTOR).LE.TINY) VECTOR=1.0
       OFFLOG=ALOG10(SPDMIN)
        LOGFLG=0
      IF(VECTOR.LT.0.0) THEN
         VECTOR=ABS(VECTOR)
         LOGFLG=1
      ENDIF
c
      IF(OFFLOG.LT.0.0 .AND. LOGFLG.GT.0) THEN
         OFFLOG=10.**AINT(ABS(OFFLOG)+1.5)
      ELSE
         OFFLOG=1.0
      ENDIF
C
C4Cb----Normalize flows
C
         XLEFT=0.0
         NCELLS=0
         NUMCLL=0
C
C4H----Data header
C
       IF(IVIEW.EQ.1) THEN
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELL FLOWS ALONG ROW DIRECTION'',
     &               /10X,85(''-''))')
       ELSE IF(IVIEW.EQ.2) THEN
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELL FLOWS ALONG COLUMN DIRECTION'',
     &               /10X,85(''-''))')
       ELSE
          WRITE(OUTUNT,'(//,1H1,40X,
     &               '' CELLS FLOW ALONG PARTICLE DIRECTION'',
     &               /10X,85(''-''))')
       ENDIF
         WRITE(OUTUNT,'(10X,
     &               ''       ROW'',
     &               ''    COLUMN'',
     &               ''     LAYER'',
     &               ''    IBOUND'',
     &               ''         X'',
     &               ''         Y'',
     &               ''         Z'',
     &               ''     SPEED'',
     &               /10X,85(''-''))')
         WRITE(OUTUNT,'(50X,
     &               ''  VELOCITY'',
     &               ''  VELOCITY'',
     &               ''  VELOCITY'',
     &               /10X,85(''-''))')
C
       IF(IVIEW.EQ.1) THEN
          STRCLL=1
          FINCLL=NCOL
       ELSE IF(IVIEW.EQ.2) THEN
          STRCLL=NCOL+1
          FINCLL=NCOL+NROW
       ELSE
          STRCLL=1
          FINCLL=IRWCL
       ENDIF
C
C4B----Loop to build cell arcs from elevation and row/column data
C
         DO 420 N=STRCLL,FINCLL
            NUMCLL=NUMCLL+1
          IF(IVIEW.LE.1) THEN
             IROW=IRWCL
             ICOL=NUMCLL
          ELSE IF(IVIEW.EQ.2) THEN
             IROW=NUMCLL
             ICOL=IRWCL
          ELSE
             ICOL=LRWCL(1,NUMCLL)
             IROW=LRWCL(2,NUMCLL)
          ENDIF
C
C4Ba----Write attributes of each cell
C
            XCENTR=XLEFT+DELX(N)*XYSCALE*0.5*HSCALE
            XLEFT=XLEFT+DELX(N)*XYSCALE*HSCALE
            DO 430 ILAY=1,NLAY
               NCELLS=NCELLS+1
               MCELLS=1+(NCELLS-1)*7
               IBDCEL=ABS(IBOUND(ICOL,IROW,ILAY))
C
              IF(IBDCEL.EQ.0) GO TO 430
C
               TOP=ZTOP(ICOL,IROW,ILAY)
               BOT=ZBOT(ICOL,IROW,ILAY)
               YCENTR=TOP-(TOP-BOT)*0.5
C
               YCENTR=YCENTR*VSCALE-CLLMIN
C
              IF(IVIEW.EQ.1) THEN
                 XGCVEL=CELVEL(MCELLS)
                 YGCVEL=CELVEL(MCELLS+2)
                 XYSPED=CELVEL(MCELLS+4)
              ELSE
                 XGCVEL=CELVEL(MCELLS+1)
                 YGCVEL=CELVEL(MCELLS+2)
                 XYSPED=CELVEL(MCELLS+5)
              ENDIF
C
             IF(XYSPED.LE.0.0) THEN
                NOSPED=NOSPED+1
             ELSE
                ROWCOL=IROW*(10**LENCOL)+ICOL
                WRITE(CHRWCL,'(I10)') ROWCOL
                FINCHR=10
                STRCHR=FINCHR-(LENROW+LENCOL-1)
C
              IF(LOGFLG.GT.0) THEN
                 FACTLG=VECTOR*ALOG10(XYSPED*OFFLOG)/XYSPED
              ELSE
                 FACTLG=VECTOR
              ENDIF
C
                XYDIST=FACTLG*XYSPED*HSCALE
C
              IF(ABS(XGCVEL).GT.0.0) THEN
                 XYVELO=ABS(YGCVEL/XGCVEL)
                 XYGRAD=ATAN(XYVELO)
                 XYGANG=XYGRAD/0.01745329
              ELSE
                 XYGANG=0.0
              ENDIF
C
              IF(XGCVEL.GT.0.0) THEN
                 IXFLAG=1
               IF(YGCVEL.GT.0.0) THEN
                  IYFLAG=1
                  XTOADD=XYDIST*COS(XYGRAD)
                  YTOADD=XYDIST*SIN(XYGRAD)
               ELSE IF(YGCVEL.LT.0.0) THEN
                  IYFLAG=-1
                  XTOADD=XYDIST*COS(XYGRAD)
                  YTOADD=-XYDIST*SIN(XYGRAD)
               ELSE
                  IYFLAG=0
                  XTOADD=XYDIST
                  YTOADD=0.0
               ENDIF
              ELSE IF(XGCVEL.LT.0.0) THEN
                 IXFLAG=-1
               IF(YGCVEL.GT.0.0) THEN
                  IYFLAG=1
                  XTOADD=-XYDIST*COS(XYGRAD)
                  YTOADD=XYDIST*SIN(XYGRAD)
               ELSE IF(YGCVEL.LT.0.0) THEN
                  IYFLAG=-1
                  XTOADD=-XYDIST*COS(XYGRAD)
                  YTOADD=-XYDIST*SIN(XYGRAD)
               ELSE
                  IYFLAG=0
                  XTOADD=-XYDIST
                  YTOADD=0.0
               ENDIF
              ELSE
                 IXFLAG=0
               IF(YGCVEL.GT.0.0) THEN
                  IYFLAG=1
                  XTOADD=0.0
                  YTOADD=XYDIST
               ELSE IF(YGCVEL.LT.0.0) THEN
                  IYFLAG=-1
                  XTOADD=0.0
                  YTOADD=-XYDIST
               ELSE
                  IYFLAG=0
                  GO TO 430
               ENDIF
              ENDIF
C                  
                XTO=XCENTR+XTOADD
                XFROM=XCENTR-XTOADD
                YTO=YCENTR+YTOADD
                YFROM=YCENTR-YTOADD
C
                XGCDIF=ABS(XTO-XFROM)
                YGCDIF=ABS(YTO-YFROM)
              IF(XGCDIF.LE.0.0001 .AND. YGCDIF.LE.0.0001) THEN
                 NOVECT=NOVECT+1
                 WRITE(OUTUNT,'(10X,
     &                  4I10,
     &                  '' ##---> CONCIDENT VECTOR IN CELL <---##'')')
     &                          IROW,
     &                          ICOL,
     &                          ILAY,
     &                          IBDCEL
                 GO TO 430
              ENDIF
C
               WRITE(IOUT(3),'(A)')
     &                      CHRWCL (STRCHR:FINCHR)
               WRITE(IOUT(3),'(F20.10,'','',F20.10)')
     &                      XFROM,
     &                      YFROM
               WRITE(IOUT(3),'(F20.10,'','',F20.10)')
     &                      XTO,
     &                      YTO
               WRITE(IOUT(3),'(''END'')')
C
               XGCVEL=CELVEL(MCELLS)
               YGCVEL=CELVEL(MCELLS+1)
               ZGCVEL=CELVEL(MCELLS+2)
               XYSPED=CELVEL(MCELLS+3)
               XZSPED=CELVEL(MCELLS+4)
               YZSPED=CELVEL(MCELLS+5)
C
             IF(ABS(XGCVEL).LE.THRESD .AND. XGCVEL.NE.0.) 
     &         XGCVEL=SIGN(THRESD,XGCVEL)
             IF(ABS(YGCVEL).LE.THRESD .AND. YGCVEL.NE.0.) 
     &         YGCVEL=SIGN(THRESD,YGCVEL)
             IF(ABS(ZGCVEL).LE.THRESD .AND. ZGCVEL.NE.0.) 
     &         ZGCVEL=SIGN(THRESD,ZGCVEL)
C
             IF(XYSPED.LE.THRESD .AND. XYSPED.NE.0.) 
     &         XYSPED=SIGN(THRESD,XYSPED)
             IF(XZSPED.LE.THRESD .AND. XZSPED.NE.0.) 
     &         XZSPED=SIGN(THRESD,XZSPED)
             IF(YZSPED.LE.THRESD .AND. YZSPED.NE.0.) 
     &         YZSPED=SIGN(THRESD,YZSPED)
C
               WRITE(IOUT(2),'(
     &                         A,'','',
     &                         A,'','',              
     &                         5(I10,'',''),
     &                         6(F16.10,'',''))')
     &                      CHRWCL (STRCHR:FINCHR),
     &                      CHRWCL (STRCHR:FINCHR),
     &                      IROW,
     &                      ICOL,
     &                      ILAY,
     &                      SEQNUM,
     &                      IBDCEL,
     &                      XGCVEL,
     &                      YGCVEL,
     &                      ZGCVEL,
     &                      XYSPED,
     &                      XZSPED,
     &                      YZSPED
             ENDIF
C
             IF(IVIEW.EQ.1) THEN
                WRITE(OUTUNT,'(10X,
     &                        4I10,
     &                        4(G10.3,1X))')
     &                      IROW,
     &                      ICOL,
     &                      ILAY,
     &                      IBDCEL,
     &                      XGCVEL,
     &                      YGCVEL,
     &                      ZGCVEL,
     &                      XZSPED
              ELSE
                WRITE(OUTUNT,'(10X,
     &                        4I10,
     &                        4(G10.3,1X))')
     &                      IROW,
     &                      ICOL,
     &                      ILAY,
     &                      IBDCEL,
     &                      XGCVEL,
     &                      YGCVEL,
     &                      ZGCVEL,
     &                      YZSPED
              ENDIF
  430       CONTINUE
  420    CONTINUE
      ENDIF
C
         WRITE(OUTUNT,'(10X,85(''-''))')
C
C7----Close attribute files
C
      CALL CLOSE_FILE (FILES(2),*9999)
      WRITE(IOUT(3),'(''END'')')
      CALL CLOSE_FILE (FILES(3),*9999)
C
C7----Messages
C
      BADTOT=NOSPED+NOVECT
C
C7A----Message to user
C
      WRITE(*,'(''      .Velocity vector line coverage '',A,
     &          '' consists of '')')
     &        VELCOV (1:INDEX(VELCOV,' ')-1)
      WRITE(*,'(''      .'',I10,'' cells'')') NCELLS
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (dry)'')') DRYCLL
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (noflow)'')') NOFLOW
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (saturated thickness = 0)'')') BADTHK
      IF(IVIEW.LE.0 .OR. IVIEW.GE.4) THEN
         WRITE(*,'(''      .'',I10,'' cells with '',
     &          ''ommited (no flow right and front faces)'')') BADFLO
      ELSE IF(IVIEW.EQ.1) THEN
         WRITE(*,'(''      .'',I10,'' cells with '',
     &          ''ommited (no flow right and lower faces)'')') BADFLO
      ELSE IF(IVIEW.EQ.2) THEN
         WRITE(*,'(''      .'',I10,'' cells with '',
     &          ''ommited (no flow front and lower faces)'')') BADFLO
      ENDIF
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (no flow through cell)'')') NOSPED
      WRITE(*,'(''      .'',I10,'' cells were '',
     &          ''ommited (too little flow through cell)'')') NOVECT
      WRITE(*,'(''      .'',I10,'' total cells with '',
     &          ''ommited (no flow through faces)'')') BADTOT
      WRITE(*,'(''      .'',I10,'' cells with '',
     &          ''no flow through right face of cell'')') BADRIG
      WRITE(*,'(''      .'',I10,'' cells with '',
     &          ''no flow through front face of cell'')') BADFRT
      WRITE(*,'(''      .'',I10,'' cells with '',
     &          ''no flow through lower face of cell'')') BADLOW
      WRITE(*,'(''      .'',G16.6,'' minimum speed in cells '')') 
     &          SPDMIN
      WRITE(*,'(''      .'',G16.6,'' maximum speed in cells '')') 
     &          SPDMAX
      WRITE(*,'(''      .'',I10,'' row contains maximum speed '')') 
     &          MAXROW
      WRITE(*,'(''      .'',I10,'' column contains maximum speed '')') 
     &          MAXCOL
      WRITE(*,'(''      .'',G16.6,'' vector multipler '')') 
     &          VECTOR
C
C7B----Message to operational output file (modpatharc.out)
C
      WRITE(OUTUNT,'(''      .Cell-by-cell velocity term file '',A,
     &          '' consists of '')')
     &        VELCOV (1:INDEX(VELCOV,' ')-1)
      WRITE(OUTUNT,'(''      .'',I10,'' cells'')') NCELLS
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (ibound = 0)'')') BADIBD
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (dry)'')') DRYCLL
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (noflow)'')') NOFLOW
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (saturated thickness = 0)'')') BADTHK
      IF(IVIEW.LE.0 .OR. IVIEW.GE.5) THEN
         WRITE(OUTUNT,'(''      .'',I10,'' cells with '',
     &          ''ommited (no flow right and front faces)'')') BADFLO
      ELSE IF(IVIEW.EQ.1) THEN
         WRITE(OUTUNT,'(''      .'',I10,'' cells with '',
     &          ''ommited (no flow right and lower faces)'')') BADFLO
      ELSE IF(IVIEW.EQ.2) THEN
         WRITE(OUTUNT,'(''      .'',I10,'' cells with '',
     &          ''ommited (no flow front and lower faces)'')') BADFLO
      ENDIF
      WRITE(OUTUNT,'(''      .'',I10,'' cells were '',
     &          ''ommited (too little flow through cell)'')') NOVECT
      WRITE(OUTUNT,'(''      .'',I10,'' total cells with '',
     &          ''ommited (no flow through faces)'')') BADTOT
      WRITE(OUTUNT,'(''      .'',I10,'' cells with '',
     &          ''no flow through right face of cell'')') BADRIG
      WRITE(OUTUNT,'(''      .'',I10,'' cells with '',
     &          ''no flow through front face of cell'')') BADFRT
      WRITE(OUTUNT,'(''      .'',I10,'' cells with '',
     &          ''no flow through lower face of cell'')') BADLOW
      WRITE(OUTUNT,'(''      .'',G16.6,'' minimum speed in cells '')') 
     &          SPDMIN
      WRITE(OUTUNT,'(''      .'',G16.6,'' maximum speed in cells '')') 
     &          SPDMAX
      WRITE(OUTUNT,'(''      .'',I10,
     &               '' row contains maximum speed '')') 
     &          MAXROW
      WRITE(OUTUNT,'(''      .'',I10,
     &               '' column contains maximum speed '')')
     &          MAXCOL
      WRITE(OUTUNT,'(''      .'',G16.6,'' vector multipler '')') 
     &          VECTOR
C
C8----Arc Operations to coverage and attribute files
C
      NSTAT=2
      ITALK=1
      CALL DELETE_FILE (FILES(4),*9999)
      CALL OPEN_FILE (IOUT(4),FILES(4),NSTAT,ITALK,*9999)
C
C8A----Create pathline coverage
C
      ARCCMD='create'
      WRITE(IOUT(4),'(A,1X,A)') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             VELCOV (1:INDEX(VELCOV,' ')-1)
C
C8B----Generate Coverage VELCOV
C
      ARCCMD='generate'
      COVTYP='lines'
      JONFIL='arcs'
      WRITE(IOUT(4),'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             VELCOV (1:INDEX(VELCOV,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVTYP (1:INDEX(COVTYP,' ')-1)
C
C8C----Build Coverage VELCOV
C
      ARCCMD='build'
      COVTYP='line'
      WRITE(IOUT(4),'(A,2(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             VELCOV (1:INDEX(VELCOV,' ')-1),
     &                             COVTYP (1:INDEX(COVTYP,' ')-1)
C
C8D----Create attribute files and "add from" attributes
C
      ARCCMD='tables'
      TMPFIL='veloitems'
      ADDFIL='veloitem'
      ACTION='kill'
      WRITE(IOUT(4),'(A,3(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             TMPFIL (1:INDEX(TMPFIL,' ')-1),
     &                             ADDFIL (1:INDEX(ADDFIL,' ')-1),
     &                             ACTION (1:INDEX(ACTION,' ')-1)
C
C8E---Join attribute files start, present,final to Coverage VELCOV AAT file
C
      ARCCMD='joinitem'
      COVFIL=VELCOV (1:INDEX(VELCOV,' ')-1)//'.AAT'
      JONFIL=FILES(2) (1:INDEX(FILES(2),' ')-1)
      RELITM='$recno'
      STRITM=VELCOV (1:INDEX(VELCOV,' ')-1)//'-ID'
      JONTYP='link'      
      WRITE(IOUT(4),'(A,6(1X,A))') 
     &                             ARCCMD (1:INDEX(ARCCMD,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             JONFIL (1:INDEX(JONFIL,' ')-1),
     &                             COVFIL (1:INDEX(COVFIL,' ')-1),
     &                             RELITM (1:INDEX(RELITM,' ')-1),
     &                             STRITM (1:INDEX(STRITM,' ')-1),
     &                             JONTYP (1:INDEX(JONTYP,' ')-1)
      CALL CLOSE_FILE (FILES(MAXFIL),*9999)
       GO TO 1000
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                        Program Errors
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
CE*********************************************************************C
 9999 WRITE(*,
     &    '(//,'' Abnormal termination of Pthvel_Subroutine'')')
      WRITE(OUTUNT,
     &    '(//,'' Abnormal termination of Pthvel_Subroutine'')')
      RETURN 1
C**********************************************************************C
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C                  Return to Calling Program
C<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>C
C**********************************************************************C
 1000 RETURN
      END
